#!/usr/bin/env python3

# usage: dedupe-by-mph.py FILE [FILE...] < INFILE > OUTFILE
#    FILEs are the files to use as the set of items to dedupe against
#    INFILE (on stdin) is the file to dedupe
#    OUTFILE is the deduped file

import sys
import re

space_re = re.compile(r'\s+')
hyp_re = re.compile(r'\s*-\s*')
def normalize_mph(s):
    return hyp_re.sub('-', space_re.sub(' ', s))

base = set()
for f in sys.argv[1:]:
    for line in map(str.strip, open(f, 'r')):
        if line.startswith(r'\mph'):
            base.add(normalize_mph(line))

inst = []
mph = None
for line in map(str.strip, sys.stdin):
    if line == '' and len(inst) > 0:
        if mph is not None and mph not in base:
            print('\n'.join(inst))
            print() # blank line to separate
        inst = []
        mph = None
    elif line != '':
        if line.startswith(r'\mph'):
            mph = normalize_mph(line)
        inst.append(line)
# in case there was no blank line at the end, check for non-empty inst
if len(inst) > 0 and mph is not None and mph not in base:
    print('\n'.join(inst))
