#!/bin/bash

# usage: toolbox-dup-check PATTERN FILE [FILE...]
# output: lines exact-match normalized-space

if [ -z "$AGGREGATIONHOME" ]; then
  echo "AGGREGATIONHOME is not set. Attempting to locate." >&2
  curdir=$( cd `dirname "$0"`/../.. && pwd)
  if [ -z "$curdir" ]; then
    echo "AGGREGATIONHOME directory could not be found. Exiting." >&2
    exit 1
  fi
  AGGREGATIONHOME="$curdir"
fi

PATTERN="$1"
shift

cat <<EOS
                                       exact-match        normalized spacing
        filename          #lines    uniq:dup     percent   uniq:dup     percent
-----------------------|--------|-------------|--------|-------------|--------|
EOS

ALL_LC=0
ALL_EU=0
ALL_ED=0
ALL_NU=0
ALL_ND=0

for f in "$@"; do
    fn=`basename "$f"`
    lines=`grep -e "$PATTERN" < "$f"`
    # line count
    LC=`echo "$lines" | wc -l`
    # exact {unique, duplicate, duplicate_percentage}
    EU=`echo "$lines" | sort -u | wc -l`
    ED=`echo "$LC - $EU" | bc`
    EP=`echo "scale=4; $ED/$LC" | bc`
    # normalized spaces {unique, duplicate, duplicate_percentage}
    NU=$(echo "$lines"\
                    | sed -r 's/\s+/ /g; s/\s?-\s?/-/g'\
                    | sort -u | wc -l)
    ND=`echo "$LC - $NU" | bc`
    NP=`echo "scale=4; $ND/$LC" | bc`
    printf "%-23s|%7d |%6d:%-6d  %-6g |%6d:%-6d  %-6g |\n" \
      "$fn" $LC $EU $ED $EP $NU $ND $NP
    # tally totals
    ALL_LC=`echo "$ALL_LC + $LC" | bc`
    ALL_EU=`echo "$ALL_EU + $EU" | bc`
    ALL_ED=`echo "$ALL_ED + $ED" | bc`
    ALL_NU=`echo "$ALL_NU + $NU" | bc`
    ALL_ND=`echo "$ALL_ND + $ND" | bc`
done

# total counts over individual files
cat <<EOS
-----------------------|--------|-------------|--------|-------------|--------|
EOS
ALL_EP=`echo "scale=4; $ALL_ED/$ALL_LC" | bc`
ALL_NP=`echo "scale=4; $ALL_ND/$ALL_LC" | bc`
printf "%-23s|%7d |%6d:%-6d  %-6g |%6d:%-6d  %-6g |\n" \
  "TOTALS" $ALL_LC $ALL_EU $ALL_ED $ALL_EP $ALL_NU $ALL_ND $ALL_NP

# total counts of all files grouped
cat <<EOS
-----------------------|--------|-------------|--------|-------------|--------|
EOS

lines=`cat "$@" | grep -e "$PATTERN"`
# line count
LC=`echo "$lines" | wc -l`
# exact {unique, duplicate, duplicate_percentage}
EU=`echo "$lines" | sort -u | wc -l`
ED=`echo "$LC - $EU" | bc`
EP=`echo "scale=4; $ED/$LC" | bc`
# normalized spaces {unique, duplicate, duplicate_percentage}
NU=$(echo "$lines"\
                | sed -r 's/\s+/ /g; s/\s?-\s?/-/g'\
                | sort -u | wc -l)
ND=`echo "$LC - $NU" | bc`
NP=`echo "scale=4; $ND/$LC" | bc`
printf "%-23s|%7d |%6d:%-6d  %-6g |%6d:%-6d  %-6g |\n" \
  "COMBINED" $LC $EU $ED $EP $NU $ND $NP
