#!/usr/bin/env python3

import sys
from random import shuffle
import argparse

parser = argparse.ArgumentParser(
    description='Split Toolbox items from STDIN into separate files.',
    epilog='example: split-toolbox.py -r -s 0.1 test 0.1 dev 0.8 train'
           '< inputfile')
parser.add_argument('-a', '--append', action='store_true',
                    help='append to output files instead of overwriting them')
parser.add_argument('-r', '--random', action='store_true',
                    help='randomize the input items before splitting')
parser.add_argument('-s', '--split', nargs=2, metavar=('RATIO', 'FILE'),
                    dest='splits', action='append', required=True,
                    help='add a split file; RATIO is a number between 0 and '
                         '1, and all RATIOs must add to 1.0')
args = parser.parse_args()

splits = [(float(p),f) for p, f in args.splits]
if abs(1.0 - sum(sp[0] for sp in splits)) > 0.00001:
    print('split percentages do not add to 1.0')
    sys.exit(1)

insts = []
inst = []
for line in map(str.strip, sys.stdin):
    if line == '' and len(inst) > 0:
        insts.append(inst)
        inst = []
    elif line != '':
        inst.append(line)

if args.random:
    shuffle(insts)

openmode='w'
if args.append:
    openmode='a'

n = len(insts)
for p, outfile in splits:
    outn = int(n * p)
    outinsts = insts[:outn]
    with open(outfile, openmode) as out:
        for inst in outinsts:
            print('\n'.join(inst), file=out)
            print(file=out)
    insts = insts[outn:]
# in case we didn't get all, add to last one
if len(insts) > 0:
    with open(splits[-1][1],'a') as out:
        for inst in insts:
            print('\n'.join(inst), file=out)
            print(file=out)
