#!/usr/bin/env perl

# Purpose: take an IGT enriched file, and build src PS from English PS
#
# usage: $0 IGT_file output_file {format}

use strict;

use IGT_word_align;
use ttree;
use indent_ttree;


########## contants
my $YES = 1;
my $NO = 0;

my $NON_NULL_LINK = 1;  # a link that both positions are not NULL
my $NULL_LINK = 2;      # a link that one position is NULL.

my $UNALIGNED = -2;     # in a DS, if the dep link is "pos UNALIGNED", it means pos is not aligned.

my $INTERNAL_NODE = 1;
my $LEAF_NODE = 2;


main();

1;


sub main {
    ########## variables to store the results
    my $debug = 1;

    my $igt_num = 0;       # the number of igt appears in the IGT file
    my $igt_clean_num = 0;      # number of clean IGTs
    my $igt_wrong_format = 0;   # among the clean ones, the ones that are in 
                            #  wrong format


    ######## among all the igts that are annotated by both
    my $ENG_PARSE = 0;
    my $ENG_DS = 1;
    my $WS_12 = 2;   # src and gloss alignment
    my $WS_23 = 3;   # gloss and trans alignment
    my $SRC_DS = 4;

    my $QUEST_NUM = 5;   # Q2-Q6 
    
    ########## step 0: open the files
    my $arg_num = scalar @ARGV;

    if($arg_num < 2){
	die "usage: $0 IGT_file output_file {format} {rem_all_unary} {rem_unary_w_same_label} {highest_attach} {attach_to_left} {include_f_tag_in_CFG} \n \
      format: \n \
        1: Q2-Q6 asked once; \n \
        2 (default): Q2-Q6 asked twice\n \
      remove_all_unary: the default value is 0. \n\
      remove_unary_w_same_label: the default value is 1 \n\ 
      highest_attach: \n \
        1 (default): attach to the highest node; \n \
        0: attach to the lowest\n \
      attach_to_left (meaningful only if highest_attach=0): \n \
         1: attach to the leftmost; \n \
         0 (default): attach to the rightmost\n \
      include_f_tag_in_CFG: the default value is 1\n";
    }
    
    my $igt_file = $ARGV[0];
    my $output_file = $ARGV[1];

    my $format = 2;  # 1: each question appears once.
                     # >1: Q2-Q6 each appear twice.
    
    my $rem_all_unary = 0;
    my $rem_unary_w_same_label = 1;
    
    my $highest_attach = 1;
    my $attach_to_left = 0;

    my $include_f_tag_in_CFG = 1;

    if($arg_num > 2){
	$format = $ARGV[2];
    }

    if($arg_num > 3){
	$rem_all_unary = $ARGV[3];
    }

    if($arg_num > 4){
	$rem_unary_w_same_label = $ARGV[4];
    }

    if($arg_num > 5){
	$highest_attach = $ARGV[5];
    }

    if($arg_num > 6){
	$attach_to_left = $ARGV[6];
    }

    if($arg_num > 7){
        $include_f_tag_in_CFG = $ARGV[7];
    }

    open(my $igt_fp, "$igt_file") or die "cannot open $igt_file\n";
    open(my $output_fp, ">$output_file") or die "cannot open $output_file\n";
    open(my $log_fp, ">$output_file.log") or die "cannot open $output_file.log\n";

    print STDERR "include_f_tag_in_CFG=$include_f_tag_in_CFG\n";

    ############ step 1: read IGTs
    my %igt_strs = ();
    my %id_lines = ();

    $igt_num = read_igt_file($igt_fp, \%igt_strs, \%id_lines);
    print STDERR "Finish reading $igt_num IGTs from $igt_file\n";


    ########### step 2: proc IGTs
    my $un_ans_cnt = 0;  # the number of questions answered in file 1

    my %align_12_map = ();
    my %align_23_map = ();
    my @depend_E = ();
    my @depend_F = ();

    my $src_line = "";
    my $gloss_line = "";
    my $trans_line = "";

    my $parse_line_E = "";

    my $project_suc_num = 0;

    my %rule_hash1 = ();
    my %rule_hash2 = ();

    foreach my $key (keys %igt_strs){
	my $igt_str = $igt_strs{$key};
	my $id_line = $id_lines{$key};
	
	my $q1_ans = -1;

	##### step 2.1. get the igt structure
        my @answers = ();
	my $res = 
	    str_to_rich_igt_struct_w_parse_line_and_answers($igt_str, $format,
						\$q1_ans, \$un_ans_cnt,
						\$src_line, \$gloss_line, 
						\$trans_line,
						\%align_12_map, \%align_23_map, 
						\@depend_E, \@depend_F, 
						\$parse_line_E, \@answers);

	if($res){
	    $igt_clean_num += ($q1_ans == $YES);
	    if($q1_ans != $YES){
		next;
	    }
	}else{
	    $igt_wrong_format ++;
	    next;
	}
    
	##### step 2.2: print out the IGT
	print_igt_w_q1_to_q6($output_fp, $id_line,
			     $src_line, $gloss_line, $trans_line,
			     \%align_12_map, \%align_23_map,
			     \@depend_E, \@depend_F, $q1_ans, \@answers);


	my %align_13_map = ();
	my %align_31_map = ();

	alignment_composition(\%align_12_map, \%align_23_map, \%align_13_map);
	get_reverse_alignment(\%align_13_map, \%align_31_map);

	#### step 2.3: proc IGT
	print STDERR "igt=$key\n";
        my $ttree_E = new Ttree;
	$res = proc_IGT($output_fp, \%align_31_map, 
			\%align_13_map, $parse_line_E, $src_line, 
			$rem_all_unary, $rem_unary_w_same_label,
			$highest_attach, $attach_to_left, $ttree_E);
	
	if($res){
	    $project_suc_num ++;
	}else{
            next;
        }

        ### step 2.4: extract CFG rules
        extract_CF_rules_from_ttree($ttree_E, \%rule_hash1, 
                                    \%rule_hash2, $include_f_tag_in_CFG);

        if($debug){
           my %tmp_rule_hash = ();
           my %tmp_rule_hash2 = ();
           my $cnt = extract_CF_rules_from_ttree($ttree_E, \%tmp_rule_hash,
                                                 \%tmp_rule_hash2, 
                                                 $include_f_tag_in_CFG);
           my $tmp_rule_tokens = 0;
           my $tmp_rule_types = 0;


           print $log_fp "$id_line\n";
           print $log_fp "$src_line\n$gloss_line\n$trans_line\n";
           print_CF_rules($log_fp, \%tmp_rule_hash,
                          \$tmp_rule_tokens, \$tmp_rule_types);
           print $log_fp "\n";
        }

	## last;
    }

    print STDERR "IGT numbers: file=$igt_num (clean=$igt_clean_num)\n";
    print STDERR "Unanswered questions: file=$un_ans_cnt\n\n";
    print STDERR "Out of clean ones, wrong_format=$igt_wrong_format\n";

    print STDERR "suc_num=$project_suc_num\n\n";
    close($output_fp);
    close($log_fp);

    print_CF_rules_to_two_files($output_file, \%rule_hash1, \%rule_hash2);
}



### return suc
sub proc_IGT {
    my ($output_fp, $align_31_map_ptr, 
	$align_13_map_ptr, $parse_line_E, $src_line,
	$rem_all_unary, $rem_unary_w_same_label,
	$highest_attach, $attach_to_left, $ttree_E) = @_;

    ##### step 1: get the Eng parse tree
    print $output_fp "\n\nQ7: src_PS: \n\n";
    my $suc = str_to_ttree($parse_line_E, $ttree_E);
    if(!$suc){
	print STDERR "str_to_ttree failed for Eng PS: +$parse_line_E\n";
	return 0;
    }

    my $indent_str = indentTtree($parse_line_E);
    print $output_fp "$indent_str\n";

    ####### step 2: get the src words
    my @src_words = split(/\s+/, $src_line);
    my %src_word_hash = ();
    for(my $i=0; $i<scalar @src_words; $i++){
	$src_word_hash{$i+1} = $src_words[$i];
    }

    ####### step 3: get the src PS
    my $src_sent_leng = scalar @src_words;
    $suc = project_phrase_structure($output_fp, $ttree_E, $align_31_map_ptr,
				    $align_13_map_ptr,
				    $src_sent_leng, \%src_word_hash,
				    $rem_all_unary, $rem_unary_w_same_label,
				    $highest_attach, $attach_to_left);

    if(!$suc){
	print $output_fp "projecting failed\n";
	return 0;
    }

    #### step 4: check the resulting sent
    my $sent = sent_str($ttree_E);
    if($sent ne $src_line){
	print $output_fp "different src_sent: orig_sent=+$src_line+, sent_str=+$sent+\n";
    }
    return 1;
}

sub print_ttree {
    my ($fp, $ttree, $show_head) = @_;
    my $parse_line = "";

    my $suc = ttree_to_str($ttree, \$parse_line, $show_head);
    if($suc){
	my $indent_str = indentTtree($parse_line);
	print $fp "$indent_str\n";
    }else{
	print STDERR "print_ttree() failed for src DS: +$parse_line\n";
	return 0;
    }

    return 1;
}
    



###############################################

#### Steps:
####  1. replace Eng words with src words
####  2. remove the subtree that covers nothing, remove "unary" rules
####        e.g.,    (S (NP x) (VP (TO empty) (VP yy)
###         becomes  (S (NP x) (VP yy))
####  3. reorder children of each node, flatten the tree if needed
####
####  4. attached unaligned src words
####
#### All the indexes start from 1.
#### 
sub project_phrase_structure {
    my ($fp, $ttree_E, $align_31_map_ptr, $align_13_map_ptr, 
	$src_sent_leng, $src_word_hash_ptr, 
	$rem_all_unary, $rem_unary_w_same_label,
	$highest_attach, $attach_to_left) = @_;

    my $show_head = 0;

    ########### step 1: replace words
    replace_words_in_PS($ttree_E, $align_31_map_ptr, $align_13_map_ptr, 
			$src_word_hash_ptr);
    print $fp "\nreplace_words:\n";
    print_ttree($fp, $ttree_E, $show_head);

    ########### step 2: remove nodes for unaligned Eng words
    my $res = remove_unaligned_Eng_words_in_PS($fp, $ttree_E, 
				     $rem_all_unary, $rem_unary_w_same_label);
    if(!$res){
	### No src words are aligned to any Eng words
	print STDERR "remove_unaligned_Eng_words_in_PS failed.\n Build flat ttree\n";
	return 0;
    }

    print $fp "\nAfter remove_unaligned_words:\n";
    print_ttree($fp, $ttree_E, $show_head);

    ########### step 3: reorder children
    my $remove_tnode_num = 0;
    my $suc = reorder_children_in_PS($ttree_E, \$remove_tnode_num);
    print $fp "\nAfter reordering: remove_tnode_num=$remove_tnode_num, suc=$suc\n";
    print_ttree($fp, $ttree_E, $show_head);


    ########## step 4: attached unaligned src words to PS
    my $cnt = 0;
    my $suc_cnt = 
	attach_unaligned_words_to_PS($ttree_E, $src_sent_leng, 
				     $src_word_hash_ptr,
				     $highest_attach, $attach_to_left, \$cnt);
    print $fp "\nAfter attaching unaligned src word: total_cnt=$cnt, suc=$suc_cnt\n";
    print_ttree($fp, $ttree_E, $show_head);

    return 1;
}



############################### step 4: attach unaligned src words

### word_hash_ptr hash src position to src word
### return the number of words attached
sub attach_unaligned_words_to_PS {
    my ($ttree, $sent_leng, $word_hash_ptr,
	$highest_attach, $attach_to_left, $cnt_ptr) = @_;

    my $suc = 0;
    my $cnt = 0;

    my %leaf_node_ptrs = ();
    get_leaf_node_ptrs($ttree, \%leaf_node_ptrs);
    
    for(my $i=0; $i<$sent_leng; $i++){
	my $word_pos = $i+1;
	if(defined($leaf_node_ptrs{$word_pos})){
	    next;
	}

	my $word_spell = $word_hash_ptr->{$word_pos};
	if(!defined($word_spell)){
	    print STDERR "This should not happen: word_pos $word_pos is not defined in word_hash_ptr: ", join("+", keys(%$word_hash_ptr)), "\n";
	    next;
	}

	$suc += attach_unaligned_word_to_PS($ttree, $sent_leng,
					    $word_pos, $word_spell, 
					    $highest_attach, $attach_to_left);
	$cnt ++;
    }

    $$cnt_ptr = $cnt;

    return $suc;
}


### find an attaching place for position $word_pos, 
###  and then create a leaf node
###
### attachment stategies: let x1 and x2 be the left and right neighbors of x
###   - highest_attach=1: 
##       find the first common ancester p of x1 and x2, attach x to p
##    - highest_attach=0 && ((attach_to_left=1) || (x2 does not exist))
##       p=x1's parent, make x p's rightmost child
##    - highest_attach=0 && ((attach_to_right=1) || (x1 does not exist))
##       p=x2's parent, make x p's leftmost child
##   
##  word_pos starts from 1
##
## return suc
sub attach_unaligned_word_to_PS {
    my ($ttree, $sent_leng, $word_pos, $word_spell, 
	$highest_attach, $attach_to_left) = @_;

    
    my $x1_pos = -1;
    my $x2_pos = -1;

    my $x1;    # x's left neighbor
    my $x2;    # x's right neighbor

    my %leaf_node_hash = ();
    get_leaf_node_ptrs($ttree, \%leaf_node_hash);

    #### step 0: make the new tnode
    my $insert_child = new Tnode;
    $insert_child->type($LEAF_NODE);
    $insert_child->s_tag("Unaligned");
    $insert_child->word($word_spell);
    $insert_child->word_pos($word_pos);
    my @tmp = ($word_pos);
    $insert_child->word_pos_array('()');
    @{$insert_child->word_pos_array} = @tmp;

    #### step 1: find x1
    for(my $i=$word_pos-1; $i>=1; $i--){
	my $tnode1 = $leaf_node_hash{$i};
	if(defined($tnode1)){
	    $x1 = $tnode1;
	    $x1_pos = $i;
	    last;
	}
    }

    #### step 2: find x2
    for(my $i=$word_pos+1; $i<=$sent_leng; $i++){
	my $tnode2 = $leaf_node_hash{$i};
	if(defined($tnode2)){
	    $x2 = $tnode2;
	    $x2_pos = $i;
	    last;
	}
    }

    if($x1_pos <= 0 || $x2_pos <= 0){
	## cannot find left neighbor, or cannot find the right neighbor
	if($x1_pos > 0){
	    ## with left neighbor only, attach the node to x1's parent as
            ## rightmost child
	    my $p = $x1->parent;
	    push(@{$p->children}, $insert_child);
	    $insert_child->parent($p);
	    return 1;
	}

	if($x2_pos > 0){
	    ## with right neighbor only, attach the node to x2's parent as
            ## the leftmost child
	    my $p = $x2->parent;
	    unshift(@{$p->children}, $insert_child);
	    $insert_child->parent($p);
	    return 1;
	}
	    
	return 0;
    }

    #### step 3: find attachment position p
    my $p;

    if($x1_pos > 0 && $x2_pos > 0 && $x1->parent == $x2->parent){
	$p = $x1 -> parent;
	my $x1_ancest_pos = find_child_pos($ttree, $x1, $p);
	splice(@{$p->children}, $x1_ancest_pos+1, 0, $insert_child);
	$insert_child->parent($p);
	return 1;
    }

    if($highest_attach){
	if($x1_pos > 0 && $x2_pos > 0){
	    my $x1_ancest_pos;
	    my $x2_ancest_pos;
	    my $suc = find_common_ancestor_of_two_tnodes($ttree, 
							 $x1, $x2, \$p);

	    if(!$suc){
		return $suc;
	    }
	
	    my $x1_ancest_pos = find_child_pos($ttree, $x1, $p);

	    if($x1_ancest_pos < 0){
		return 0;
	    }

	    splice(@{$p->children}, $x1_ancest_pos+1, 0, $insert_child);
	}else{
	    $p = $ttree->root;
	    if($x1_pos > 0){
		## make x the rightmost child of the root
		push(@{$p->children}, $insert_child);
	    }else{
		## make x the leftmost child of the root
		unshift(@{$p->children}, $insert_child);
	    }
	}
	$insert_child->parent($p);
	return 1;
    }

    #### lowest attachment or and x1 and x2 do not have the same parent.
    if(($attach_to_left && $x1_pos > 0)
       || $x2_pos <= 0){
	## make x the rightmost child of x1's parent
	$p = $x1->parent;
	if(!defined($p)){
	    return 0;
	}
	my $x1_ancest_pos = find_child_pos($ttree, $x1, $p);
	if($x1_ancest_pos < 0){
	    return 0;
	}
	
	splice(@{$p->children}, $x1_ancest_pos+1, 0, $insert_child);
    }else{
	## make x the leftmost children of x2's parent
	$p = $x2->parent;
	if(!defined($p)){
	    return 0;
	}

	my $x2_ancest_pos = find_child_pos($ttree, $x2, $p);
	if($x2_ancest_pos < 0){
	    return 0;
	}
	
	if($x2_ancest_pos == 0){
	    unshift(@{$p->children}, $insert_child);
	}else{
	    splice(@{$p->children}, $x2_ancest_pos, 0, $insert_child);
	}
    }

    $insert_child->parent($p);
    return 1;
}




############################### step 2: remove unaligned English words
### return $suc
### 
sub remove_unaligned_Eng_words_in_PS {
    my ($fp, $ttree, $rem_all_unary, $rem_unary_w_same_label) = @_;
    my $new_root;
    my $suc = 
	remove_unaligned_Eng_words_in_DS_from_tnode($fp, $ttree, $ttree->root,
						    $rem_all_unary,
						    $rem_unary_w_same_label,
						    \$new_root);
    $ttree->root($new_root);
    return $suc;
}

### return suc
### tnode might be deleted in which case $new_root_ptr will be the 
###  child of tnode.
sub remove_unaligned_Eng_words_in_DS_from_tnode {
    my ($fp, $ttree, $tnode, $rem_all_unary, $rem_unary_w_same_label,
	$new_root_ptr) = @_;

    $$new_root_ptr = $tnode;
    
    if($tnode->type == $LEAF_NODE){
	return 1;
    }

    my $children_ptr = $tnode->children;
    my $children_num = scalar @$children_ptr;

    my @new_children_list = ();
    for(my $i=0; $i<$children_num; $i++){
	my $child = $children_ptr->[$i];
	
	my $pos_array = $child->word_pos_array;
	my $span_leng = scalar @$pos_array;
	## print $fp $child->s_tag, " pos_array=", join("+", @$pos_array), "\n";

	if($span_leng > 0){
	    my $new_root;

	    remove_unaligned_Eng_words_in_DS_from_tnode($fp, $ttree, $child,
							$rem_all_unary,
							$rem_unary_w_same_label,
							\$new_root);
	    push(@new_children_list, $new_root);
	}
    }

    my $new_children_num = scalar @new_children_list;
    
    ## print $fp $tnode->s_tag, "new_children_num=$new_children_num\n";

    @$children_ptr = @new_children_list;


    if(($new_children_num > 1) || 
       (!$rem_all_unary && !$rem_unary_w_same_label)){
	return 1;
    }
    
    ### new_child_num == 1 && at least one of the flag is 1
    if($new_children_num <= 0){
	print STDERR "this should never happen: new_children_num=$new_children_num old_children_num=$children_num\n";

	my $tmp_ptr = $tnode->word_pos_array;
	print STDERR "parent: word_pos_array=+", join("+", @$tmp_ptr), "+ span_leng=", scalar @$tmp_ptr, "\n";
	return 0;
    }

    my $only_child = $new_children_list[0];
    if($rem_all_unary || ($only_child->s_tag eq $tnode->s_tag)){
	## remove the tnode
	$$new_root_ptr = $only_child;
    }

    return 1;
}
    
######################## step 3: reorder the src PS
#### return suc
sub reorder_children_in_PS {
    my ($ttree, $remove_tnode_num_ptr) = @_;

    my $troot = $ttree->root;
    if(!$troot){
	### ttree does not have any node.
	return 0;
    }

    return reorder_children_in_PS_from_tnode($ttree, $ttree->root,
					     $remove_tnode_num_ptr);
}

### return suc
### 
sub reorder_children_in_PS_from_tnode {
    my ($ttree, $tnode, $remove_tnodes_ptr) = @_;

    print STDERR "current_tnode=", $tnode->s_tag, "\n";

    if($tnode->type == $LEAF_NODE){
	return 1;
    }

    ########### internal nodes
    my $children_ptr = $tnode->children;
    my $children_num = scalar @$children_ptr;

    if($children_num <= 1){
	my $only_child = $children_ptr->[0];
	return reorder_children_in_PS_from_tnode($ttree, $only_child,
						 $remove_tnodes_ptr);
    }

    ########## internal nodes with multiple children
    my @new_children_list = ();

    my $res = reorder_children_of_a_tnode($children_ptr, 
					  \@new_children_list,
					  $remove_tnodes_ptr);
    if(!$res){
	print STDERR "reorder_children_of_a_tnode() failed: ", $tnode->s_tag, "\n";
	return $res;
    }

    @{$tnode->children} = @new_children_list;

    for(my $i=0; $i<scalar @new_children_list; $i++){
	my $child = $new_children_list[$i];
	$child->parent($tnode);
	$res = reorder_children_in_PS_from_tnode($ttree, $child,
						 $remove_tnodes_ptr);
	if(!$res){
	    print STDERR "reorder failed 2: ", tnode->s_tag, "\n";
	    return 0;
	}
    }

    return 1;
}

#### return suc
#### 
sub reorder_children_of_a_tnode {
    my ($orig_children_ptr, $new_children_ptr, $remove_tnodes_ptr) = @_;
    my $cnt = 0;

    my @starts = ();
    my @ends = ();

    my @remain_children = @$orig_children_ptr;
    my @new_children = ();

    my $first = 1;

    ### step 1: make sure that no two children's spans overlap
    while(scalar @remain_children > 0){
	my $child = shift(@remain_children);
	my $pos_ptr = $child->word_pos_array;
	my $span_leng = scalar @$pos_ptr;
	if($span_leng == 0){
	    print STDERR "reorder_children: this should not happen\n";
	    next;
	}
	my $start = $pos_ptr->[0];              ## the start of a subtree
	my $end = $pos_ptr->[$span_leng - 1];   ## the end of a subtree

	#### first child
	if($first){
	    push(@starts, $start);
	    push(@ends, $end);
	    push(@new_children, $child);
	    $first = 0;
	    next;
	}
	
	my @tmp_children = ();
	my $flatten_child = check_spans(\@starts, \@ends, \@new_children, 
					0, $child, $start, $end, 
					\@tmp_children, $remove_tnodes_ptr);
	if($flatten_child){
	    unshift(@remain_children, @tmp_children);
	}else{
	    ## no need to flatten the child
	    push(@starts, $start);
	    push(@ends, $end);
	    push(@new_children, $child);
	}
    }

    #### step 2: reorder the children
    my @ordered_children = ();
    my $children_num = scalar @new_children;
    
    my $prev_min_idx = 0;
    my $prev_min_start = 0;
    for(my $i=0; $i<$children_num; $i++){
	## find the child with smallest start, remove dup if any
	my $min_idx = 0;
	my $min_start = 0;
	for(my $j=0; $j<scalar @new_children; $j++){
	    my $start = $starts[$j];
	    if(!$min_start || $min_start > $start){
		$min_start = $start;
		$min_idx = $j;
	    }
	}

	if(!$prev_min_start || $min_start > $prev_min_start){
	    ### push the child only if it is not a duplicate of other child
	    push(@ordered_children, $new_children[$min_idx]);
	}else{
	    ### create sth like "IN+DET"
	    my $last_child = $ordered_children[$#ordered_children];
	    my $new_tag = $last_child->s_tag . "+" . 
		$new_children[$min_idx]->s_tag;
	    $last_child->s_tag($new_tag);
	}
	
	$prev_min_idx = $min_idx;
	$prev_min_start = $min_start;

	### delete the elements
	splice(@starts, $min_idx, 1);
	splice(@new_children, $min_idx, 1);
	splice(@ends, $min_idx, 1);  ## this is not necessary
    }

    @$new_children_ptr = @ordered_children;

    return 1;
}


## check each span starting from $start_pos,
##   and see whether it overlaps with [start1, end1]
##
##  return 1 if the root of subtree rooted at $child ([start1, end1])
##     has to be removed (flattened)
##
sub check_spans {
    my ($starts_ptr, $ends_ptr, $children_ptr, $start_pos,
	$child1, $start1, $end1,
	$tmp_children_ptr, $remove_tnodes_ptr) = @_;

    if($start_pos >= scalar @$starts_ptr){
	return 0;
    }
    
    my $child2 = $children_ptr->[$start_pos];
    my $start2 = $starts_ptr->[$start_pos];
    my $end2 = $ends_ptr->[$start_pos];

    ## case 1: no overlapping, check the next span
    if( ($end2 < $start1) || ($start2 > $end1) ){
	return check_spans($starts_ptr, $ends_ptr, $children_ptr, 
			   $start_pos + 1,
			   $child1, $start1, $end1, 
			   $tmp_children_ptr, $remove_tnodes_ptr);
    }

    #### case 2: span2 is a superset of span1
    ####  flatten node2
    if( ($end2 >= $end1) && ($start2 <= $start1) ){
	if($child2->type == $INTERNAL_NODE){
	    ## node2 is an internal node, remove node2
	    my @tmp_starts = ();
	    my @tmp_ends = ();
	    my @tmp_children = ();
	    
	    get_children_spans($child2, \@tmp_starts, \@tmp_ends,
			       \@tmp_children);
	    splice(@$starts_ptr, $start_pos, 1, @tmp_starts);
	    splice(@$ends_ptr, $start_pos, 1, @tmp_ends);
	    splice(@$children_ptr, $start_pos, 1, @tmp_children);

	    $$remove_tnodes_ptr ++;
	    return check_spans($starts_ptr, $ends_ptr, $children_ptr,
			      $start_pos, $child1, $start1, $end1,
			      $tmp_children_ptr, $remove_tnodes_ptr);
	}else{
	    ## node2 is a leaf node
	    return check_spans($starts_ptr, $ends_ptr, $children_ptr,
			      $start_pos+1, $child1, $start1, $end1,
			      $tmp_children_ptr, $remove_tnodes_ptr);
	}
    } 
    

    #### case 3: span1 is a superset of span2
    #### flatten node1 and return
    if( $end2 <= $end1 && $start2 >= $start1){
	if($child1->type == $LEAF_NODE){
	    ## leaf node, cannot flatten node1
	    return check_spans($starts_ptr, $ends_ptr, $children_ptr, 
			       $start_pos + 1,
			       $child1, $start1, $end1, 
			       $tmp_children_ptr, $remove_tnodes_ptr);
	}else{
	    @$tmp_children_ptr = @{$child1->children};
	    $$remove_tnodes_ptr ++;
	    return 1;
	}
    }
    

    #### case 4: crossing
    #### flatten both
    my $res = 0;
    
    if(($child2->type == $LEAF_NODE) ||
       ($child1->type == $LEAF_NODE)){
	print STDERR "this should never happen: [", $child2->s_tag, 
	", $start2, $end2] cross [", $child1->s_tag, ", $start1, $end1]\n";
	die "check this\n";
    }

    ### step 4.1: flatten span2    
    my @tmp_starts = ();
    my @tmp_ends = ();
    my @tmp_children = ();
	    
    get_children_spans($child2, \@tmp_starts, \@tmp_ends,
		       \@tmp_children);
    splice(@$starts_ptr, $start_pos, 1, @tmp_starts);
    splice(@$ends_ptr, $start_pos, 1, @tmp_ends);
    splice(@$children_ptr, $start_pos, 1, @tmp_children);



    #### step 2: flatten span1
    @$tmp_children_ptr = @{$child1->children};
    $$remove_tnodes_ptr += 2;

    return 1;
}

# return suc
sub get_children_spans {
    my ($tnode, $starts_ptr, $ends_ptr, $children_ptr) = @_;
    
    if($tnode->type == $LEAF_NODE){
	return 0;
    }

    my $tmp_children_ptr = $tnode->children;

    @$children_ptr = @$tmp_children_ptr;
    @$starts_ptr = ();
    @$ends_ptr = ();

    for(my $i=0; $i<scalar @$tmp_children_ptr; $i++){
	my $child = $tmp_children_ptr->[$i];
	my $start;
	my $end;
	my $span = get_tnode_span($child, \$start, \$end);
	push(@$starts_ptr, $start);
	push(@$ends_ptr, $end);
    }
    
    return 1;
}


####################### step 1: replace words
### word_hash_ptr hash src position to src word. Positions start from 1.
sub replace_words_in_PS {
    my ($ttree, $align_31_map_ptr, $align_13_map_ptr, $word_hash_ptr) = @_;

    return replace_words_in_PS_from_tnode($ttree, $ttree->root, 
					  $align_31_map_ptr, $align_13_map_ptr,
					  $word_hash_ptr);
}


### return suc
sub replace_words_in_PS_from_tnode {
    my ($ttree, $tnode, $align_31_map_ptr, $align_13_map_ptr, $word_hash_ptr) = @_;

    my $array_ptr = $tnode->word_pos_array;    
    if($tnode->type == $LEAF_NODE){
	my $pos_E = $tnode->word_pos;
	my $pos_str_F = $align_31_map_ptr->{$pos_E};
	
	if(!defined($pos_str_F) || ($pos_str_F !~ /\d/)){
	    ### The Eng word is not aligned to any src word
	    $tnode->word_pos(-1);
	    $tnode->word("");
	    @$array_ptr = ();
	}else{
	    ### The Eng word is aligned to one or more src words
	    ### remove \d+.\d+ (e.g., 4.1) from $pos_str_F;
	    $pos_str_F =~ s/\d+\.\d+\,?//g;
	    $pos_str_F =~ s/\,$//;

	    my @pos = split(/\,/, $pos_str_F);
	    my $num = scalar @pos;
	    my $first_pos = $pos[0];

	    $tnode->word_pos($first_pos);
	    my $tmp_word = $word_hash_ptr->{$first_pos};
	    $tnode->word($tmp_word);
	    @$array_ptr = ($first_pos);
	    
	    my $parent_ptr = $tnode->parent;
	    my $tmp_children_ptr = $parent_ptr->children;
	    for(my $i=1; $i<$num; $i++){
		my $new_tnode = new Tnode;
		$new_tnode->type($LEAF_NODE);
		$new_tnode->s_tag($tnode->s_tag);
		$new_tnode->parent($tnode->parent);

		my $tmp_pos = $pos[$i];
		$new_tnode->word_pos($tmp_pos);
		my $tmp_word = $word_hash_ptr->{$tmp_pos};
		$new_tnode->word($tmp_word);
		$new_tnode->word_pos_array('()');
		@{$new_tnode->word_pos_array} = ($tmp_pos);

		push(@$tmp_children_ptr, $new_tnode);
	    }
		
	}
	
	return 1;
    }

    ##### for internal node
    my $children_ptr = $tnode->children;
    my @orig_children = @$children_ptr;

    my $children_num = scalar @$children_ptr;

    my @tmp_array = ();
    for(my $i=0; $i<scalar @orig_children; $i++){
	my $child = $orig_children[$i];
	my $suc = replace_words_in_PS_from_tnode($ttree, $child, $align_31_map_ptr,
						 $align_13_map_ptr, $word_hash_ptr);
	if(!$suc){
	    return 0;
	}

	push(@tmp_array, @{$child->word_pos_array});
    }

    ### deal with newly added children: 
    ### this happens  when one Eng word maps to mult src words
    for(my $i=scalar @orig_children; $i<scalar @$children_ptr; $i++){
	my $child = $children_ptr->[$i];
	push(@tmp_array, @{$child->word_pos_array});
    }

    sort_and_remove_dup_in_array(\@tmp_array, $array_ptr);

    return 1;
}


### idx is based on 1, $@words_ptr is based on 0.
sub get_spelling {
    my ($words_ptr, $leng, $idx) = @_;

    if($idx == -1){
	return "*TOP*";
    }

    if(($idx > 0) && ($idx <= $leng)){
	return $words_ptr->[$idx-1];
    }

    return "*UNK*";

}

