
### common perl functions.

use strict;


################### function declaration
## open_gb_file($fp_ptr, $file_name, $for_write);
## print_gb($file_name);

## systemx($cmd)
## createDir($dir);     # return $dir

## getLines($file_name, $lines_ptr);  ## return # of non-empty lines
## getLinesFromFp($fp, $lines_ptr);

## readVoc($file_name, $hash_ptr);    ## return # of valid entries in file
## readVocFromFp($fp, $hash_ptr);
## readVocFromFpWithMaxLeng($fp, $hash_ptr, $max_leng_ptr); # 

## read_SRI_LM($fp, $hash_ptr, $delim)


## line2WordTagSeq($line, $words_ptr, $tags_ptr); ## return word num
## Str2WordTag($str, $word_ptr, $tag_ptr);         ## return suc
## $new_str = remove_prob($str);

## log10(x)

## add_to_2D_hash(hash_ptr, key1, key2); ## hash_ptr is a pointer to a 2D-hash

################### function definitions
my $word_tag_delim = '\/';


## return $suc 
sub open_gb_file {
    my ($fp_ptr, $file, $for_write) = @_;
    my $option = "<";
    if($for_write){
	$option = ">";
    }
    open($$fp_ptr, "$option:encoding(euc_cn)", $file) 
	or return 0;

    return 1;
}

sub remove_prob {
    my ($str) = @_;

    my @parts = split(/\s+/, $str);
    my $part_num = scalar @parts;

    my $last_part = $parts[$part_num - 1];

    my $patt = '(\+|\-)?\d+(\.\d+)?';
    if($last_part =~ /^$patt$/ ||
       $last_part =~ /^($patt)?e\^?($patt)?$/){
	$part_num --;
    }

    return join(' ', @parts[0..$part_num-1]);
}


######  simiar to system()
sub systemx {
    my ($cmd) = @_;

    print STDERR "\n\n***************$cmd\n\n";
    system($cmd);
    if($?){
	die "$cmd failed\n";
    }else{
	print STDERR "+++++$cmd++++ succeeds\n\n\n";
    }
}


##  read a file and put the non-empty lines into an array
##     $file_name => @lines
##
##  return the number of non-empty lines.
## 
sub getLines {
    my ($file_name, $lines_ptr) = @_;

    open(my $fp, "$file_name") or die "cannot open $file_name\n";

    return getLinesFromFp($fp, $lines_ptr);
}

sub getLinesFromFp {
    my ($fp, $lines_ptr) = @_;
    
    @$lines_ptr = ();

    while(<$fp>){
	chomp;
	if(/^\s*$/){
	    next;
	}

	s/^\s+//;
	s/\s+$//g;
	push(@$lines_ptr, $_);
    }

    return scalar @$lines_ptr;
}



sub readVoc{
    my ($file_name, $voc_ptr) = @_;

    open(my $fp, "$file_name") or die "cannot open $file_name\n";

    return readVocFromFp($fp, $voc_ptr);
}

sub readVocFromFp {
    my ($fp, $voc_ptr) = @_;
    
    %$voc_ptr = ();

    my $cnt = 0;
    while(<$fp>){
	chomp;
	if(/^\s*$/){
	    next;
	}

	s/^\s+//;
	s/\s+$//g;
	
	my @parts = split(/\s+/);
	my $tok = $parts[0];

	my $foo = $voc_ptr->{$tok};
	if(defined($foo)){
	    print STDERR "$tok appears multiple times in voc\n";
	}else{
	    $voc_ptr->{$tok} = 1;
	    $cnt ++;
	}
    }

    return $cnt;
}

### the voc is of the format "token cost"
sub readVocFromFpWithCost {
    my ($fp, $voc_ptr) = @_;
    
    %$voc_ptr = ();

    my $cnt = 0;
    while(<$fp>){
	chomp;
	if(/^\s*$/){
	    next;
	}

	s/^\s+//;
	s/\s+$//g;
	
	my @parts = split(/\s+/);
	if(@parts < 2){
	    print STDERR "wrong format in voc: +$_+\n";
	    next;
	}

	my $tok = $parts[0];

	my $foo = $voc_ptr->{$tok};
	if(defined($foo)){
	    print STDERR "$tok appears multiple times in voc\n";
	    next;
	}else{
	    $voc_ptr->{$tok} = $parts[1];
	    $cnt ++;
	}
    }

    return $cnt;
}


sub readVocFromFpWithMaxLeng {
    my ($fp, $voc_ptr, $max_leng_ptr) = @_;
    
    %$voc_ptr = ();

    my $max_leng = 0;

    my $cnt = 0;
    while(<$fp>){
	chomp;
	if(/^\s*$/){
	    next;
	}

	s/^\s+//;
	s/\s+$//g;
	
	my @parts = split(/\s+/);
	my $tok = $parts[0];

	my $foo = $voc_ptr->{$tok};
	if(defined($foo)){
	    print STDERR "$tok appears multiple times in voc\n";
	}else{
	    $voc_ptr->{$tok} = 1;
	    $cnt ++;
	}

	my $len = length($tok);
	if($len > $max_leng){
	    $max_leng = $len;
	}
    }

    $$max_leng_ptr = $max_leng;
    return $cnt;
}



## $line => @words, @tags
## return the number of word/tag in the line
## 
sub line2WordTagSeq {
    my ($line, $words_ptr, $tags_ptr) = @_;

    @$words_ptr = ();
    @$tags_ptr = ();

    my @parts = split(/\s+/, $line);
    foreach my $part (@parts){
	my $word;
	my $tag;
	my $suc = Str2WordTag($part, \$word, \$tag);
	if(!$suc){
	    return 0;
	}

	push(@$words_ptr, $word);
	push(@$tags_ptr, $tag);
    }

    return scalar @$tags_ptr;
}

## return 1 if succeeds, return 0 o.w.
sub Str2WordTag {
    my ($str, $word_ptr, $tag_ptr) = @_;

    if($str =~ /^(.+)$word_tag_delim([^$word_tag_delim]+)$/){
	$$word_ptr = $1;
	$$tag_ptr = $2;
	return 1;
    }else{
	print STDOUT "wrong format: +$str+\n";
	return 0;
    }
}


sub createDir {
    my ($dir) = @_;
    if (!(-d $dir)) {
	my $cmd = "mkdir $dir"; 
	systemx($cmd); 
    }
    return $dir;
}

sub log10 {
    my ($n) = @_;
    return log($n)/log(10);
}



## return the numbers of ngrams
## p(w3|w2,w1) is stored in $hash_ptr->{$str}, 
##   where $str = w3 . $delim . w2 . $delim . w1 . $delim
sub read_SRI_LM {
    my ($fp, $ngram, $delim, $hash_ptr) = @_;

    my @real_cnts = ();
    my @spec_cnts = ();

    my $valid_line = 0;
    my $invalid_line = 0;
    my $dup_line = 0;

    my $stage = -1;
    my $ngram_cnt = 0;

    while(<$fp>){
	chomp;
	if(/^\s*$/){
	    next;
	}

	## "\data\"
	if(/^\\data\\/){
	    $stage = 0;
	    next;
	}
	
	## "ngram 1=nnn"
	if(/^ngram\s+(\d)=(\d+)\s*$/){
	    my $cnt = $2;
	    my $n = $1;

	    if($stage != 0){
		print STDERR "warning: ngram spec counts in the wrong place: stage=$stage\n";
	    }

	    if($n > $ngram){
		print STDERR "Warning: higher-order then needed: $n > $ngram\n";
	    }
	    push(@spec_cnts, $cnt);
	    next;
	}

	## "\1-grams:"
	if(/^\\(\d)\-grams\:\s*$/){
	    $stage = $1;

	    if($ngram_cnt > 0){
		push(@real_cnts, $ngram_cnt);
		$ngram_cnt = 0;
	    }
	    next;
	}


	## "\end\"
	if(/^\\end\\/){
	    $real_cnts[$stage-1] = $ngram_cnt;
	    $stage ++;
	    next;
	}

	## "logprob w1 w2 w3 ..."
	my $key = "";
	my $logprob = -1;
	my $suc = read_LM_line($_, $delim, $stage, \$key, \$logprob);
	if(!$suc){
	    print STDOUT "wrong format: +$_+\n";
	    $invalid_line ++;
	    next;
	}

	my $cost = - $logprob;
	my $val = $hash_ptr->{$key};
	if(defined($val)){
	    print STDOUT "$key appeared multiple times: $cost and $val\n";
	    $dup_line ++;
	    next;
	}else{
	    $hash_ptr->{$key} = $cost;
	    $valid_line ++;
	    $ngram_cnt ++;
	}

	next;
    }


    if($stage != $ngram + 1){
	print STDERR "wrong format: stage=$stage, ngram=$ngram\n";
    }
	
	
    print STDERR "Finish reading ngrams: valid_line=$valid_line dup_line=$dup_line, invalid_line=$invalid_line\n";

    print STDERR "***spec_counts=", join(' ', @spec_cnts), "\n";
    print STDERR "***real_counts=", join(' ', @real_cnts), "\n";
	
    return @spec_cnts;
}


sub read_LM_line {
    my ($line, $delim, $ngram, $key_ptr, $logprob_ptr) = @_;

    my @parts = split(/\s+/, $line);
    if(scalar @parts < $ngram + 1){
	return 0;
    }

    my $str = "";

    for(my $i=$ngram; $i>=1; $i--){
	$str .= $parts[$i] . $delim;
    }

    my $val = $parts[0];
    if(($val !~ /^\-?\d+(.\d+)?$/) || ($val > 0)){
	return 0;
    }

    $$logprob_ptr = $val;

    $$key_ptr = $str;

    return 1;
}

sub add_to_2D_hash {
    my ($hash_ptr, $key1, $key2) = @_;

    my $ptr = $hash_ptr->{$key1};

    if(defined($ptr)){
	if(defined($ptr->{$key2})){
	    $ptr->{$key2} ++;
	}else{
	    $ptr->{$key2} = 1;
	}
    }else{
	$hash_ptr->{$key1} = {};
	$hash_ptr->{$key1}->{$key2} = 1;
    }
}


### return (prec, recall, f-score)
sub calc_fscore {
    my ($gold_num, $sys_num, $match_num) = @_;

    my $prec = 0;
    my $recall = 0;
    my $fscore = 0;

    if($gold_num > 0){
	$recall = $match_num/$gold_num;
    }

    if($sys_num > 0){
	$prec = $match_num/$sys_num;
    }

    if($prec+$recall){
	$fscore = 2*$prec*$recall /($prec+$recall);
    }

    return ($prec, $recall, $fscore);
}


sub get_fscore {
    my ($prec, $recall) = @_;

    my $fscore = 2*$prec*$recall /($prec+$recall);
	
    return $fscore;
}

1;


