#!/usr/bin/perl
###############
# take plain text IGT
# and count the words and
# gloss pairs
#
# output a sorted list of
# words, with counts for
# each unique gloss

use warnings;
use strict;
use File::Basename;

# if -a flag is set, order the headwords
# in the output asciibetically (rather
# than by frequency as is default)

my $ascii_order=0;
my $src_dir=0;

while($ARGV[0]) {
  if ($ARGV[0] eq '-a') {
    # set ascii and pop;
    $ascii_order=1;
    print STDERR "ascii output order is set\n";
    shift @ARGV;
  } elsif ($ARGV[0] eq '-d') {
    # set src_dir and pop pop
    $src_dir=$ARGV[1] || die "-d flag requires dir name\n";
    print STDERR "set src dir to $src_dir\n";
    shift @ARGV;
    shift @ARGV;
  } else {
    die "unrecognized option $ARGV[0]\n";
  }
}

if($src_dir){
  @ARGV = <$src_dir/*.igt>;
  if($#ARGV==-1){
    die "cannot find any .igt files in $src_dir\n";
  }
}

my %types; # hash of types to counts
my %t_srces; # hash of types to sources we've seen them in
my %glosses; # hash of types to array(gloss, count)
my $sum = 0;
while(<>){
  s/["`]//g; # remove " and backtics, we don't need em 
  s/([\w'ɬ])[.,!]/$1/g; # remove punc stuck to words
  $_=lc($_); # lower case everything
  my @toks = split(/\s/,$_);
  for (@toks){  
    $types{$_}+=1;
    if ($src_dir){
      $t_srces{$_}{$ARGV} = 1;
    }
    $sum+=1;
  }
  
  # read gloss line and parse it
  my @gltoks = split(/\s/,<>); 
  my $i;

  # make exception for case where there are less
  # toks than gltoks
  while (scalar(@toks) < scalar(@gltoks)) {
    # shorten the gltoks list till it fits 
    # (makes one big gltok at the end)

    if (scalar(@gltoks) > 2){
      splice(@gltoks, $#gltoks-1, 2, join(" ", ($gltoks[$#gltoks-1],$gltoks[$#gltoks]))); 
    } else {
      @gltoks = join(" ", @gltoks);
    }

    print STDERR "warning [line $.]: compacting final gloss (less tokens than glosses): \n  $toks[$#toks]: $gltoks[$#gltoks]\n     gltoks: @gltoks\n       toks: @toks\n";
  }

  # make exception for less glosses than toks
  while (scalar(@gltoks) < scalar(@toks)) {
    # solution: toks get mapped to empty gloss
    push(@gltoks, "");
    print STDERR "warning [line $.]: token mapped to empty gloss (less glosses than toks)\n  $toks[$#toks]\n";
  }
   

  # now step through the gloss tokens and bump appropriate counters
  for ($i=0;$i<=$#gltoks;$i++){
#    print "toks, gltoks\n";
#    print scalar(@toks).",".scalar(@gltoks)."\n";
    if($glosses{$toks[$i]}{$gltoks[$i]}){ 
      $glosses{$toks[$i]}{$gltoks[$i]} += 1; 
    } else {
      $glosses{$toks[$i]}{$gltoks[$i]} = 1;
    }
  }

  # now throw away the free translation line and the
  # blank line after
  my $next = <>;
  $next = <>;
  if ($next ne "\n") {
    # complain
    print STDERR "this line is supposed to be blank '[$next]'\n";
  }
}

print "=====================================\nformat of this file:\n";
print "-------------------------------------\n";
print "word : frequency";

if ($src_dir){
  print " (source filenames containing this word)";
}

print "\n  gloss1 : frequency\n";
print "  gloss2 : frequency\n";
print "  ...\n";
print "-------------------------------------\n";
print "=====================================\n";
print "Global info:\n";
print "Total number of word types: ".keys(%types)."\n";
print "Total number of word tokens: ".$sum."\n";
print "=====================================\n\n";

if ($ascii_order == 0) {
  foreach my $k (sort tbycount keys(%types)){
    print "$k : $types{$k}";
    if($src_dir){
      print "  ( ";
      foreach (keys(%{$t_srces{$k}})){
        print basename($_)." ";
      }
      print ")";
    }
    print "\n";
    foreach my $l (sort {$glosses{$k}{$b} <=> $glosses{$k}{$a}} keys(%{$glosses{$k}})){
      print "    $l : $glosses{$k}{$l}\n";
    }
  }
} else {
  foreach my $k (sort keys(%types)){
    print "$k : $types{$k}";
    if($src_dir){
      print "  ( ";
      foreach (keys(%{$t_srces{$k}})){
        print basename($_)." ";
      }
      print ")";
    }
    print "\n";
    foreach my $l (sort {$glosses{$k}{$b} <=> $glosses{$k}{$a}} keys(%{$glosses{$k}})){
      print "    $l : $glosses{$k}{$l}\n";
    }
  }
}


sub tbycount {
  $types{$b} <=> $types{$a};
}
