import re, os, sys
xigt_path = os.path.abspath("/Users/stephanie/projects/xigt/")
sys.path.append(xigt_path)
from xigt.codecs import xigtxml
#choices_reader_path = os.path.abspath('../')
#sys.path.append(choices_reader_path)
import ChoicesReader

# author Stephanie Peterson slp24@uw.edu

def evaluate_morphology_section(output_file, gold_file, pos):
        print("begin evaluate morphology section")
        generatedMorphology = ChoicesMorphology(ChoicesReader.ChoicesFile(output_file))
        generatedMorphology.parse_position_classes(pos)
        goldMorphology = ChoicesMorphology(ChoicesReader.ChoicesFile(gold_file))
        goldMorphology.parse_position_classes(pos)
        print("output file: "+output_file)
        print("gold file: "+gold_file)
        print("score: "+str(generatedMorphology.score(goldMorphology)))

def evaluate_lexicon_section(output_file, gold_file):
        print("begin evaluate lexicon section")
        print("begin evaluate case marking adpositions")
        adp_generatedLexicon = ChoicesLexicon(ChoicesReader.ChoicesFile(output_file))
        adp_generatedLexicon.parse_lex_items('adp')
        adp_goldLexicon = ChoicesLexicon(ChoicesReader.ChoicesFile(gold_file))
        adp_goldLexicon.parse_lex_items('adp')
        print("output file: "+output_file)
        print("gold file: "+gold_file)
        print("case marking adposition recall score: "+str(adp_generatedLexicon.score(adp_goldLexicon)))

# encapsulate a choices file lexicon section paying attention only to what we wish to compare
class ChoicesLexicon():

    def __init__(self, choices):
        self.choices = choices
        self.lexical_item_dict = {}

    def parse_lex_items(self, pos):
        lex_items = self.choices[pos]
        item_dict = {}
        i=1
        for l in lex_items:
            if l['value'] is None or l['value'] is '':
                # the info is stored in features
                for f in l['feat']:
                    # todo collapse all features into one lexical item, this will create one entry for this orth for each feature
                    item_dict[i] = LexicalItem(l['orth'], Feature(f['name'], f['value']))
                    i+=1
            else:
                # the info is not stored in features, bug in my generated output, fix!
                item_dict[i] = LexicalItem(l['orth'], Feature(l['name'], l['value']))
                i+=1

        self.lexical_item_dict[pos] = item_dict

    def score(self, other):
        # todo handle duplicates differently?
        generated_orth_features = self.get_case()
        gold_orth_features = other.get_case()
        if len(gold_orth_features) > 0:
            true_positives = 0
            #false_positives = 0
            true_negatives = 0
            #false_negatives = 0
            for pair in generated_orth_features:
                if pair in gold_orth_features:
                    true_positives+=1
                else:
                    true_negatives+=1
            return true_positives/len(gold_orth_features)

    def get_case(self):
        orth_to_feat = []
        for pos in self.lexical_item_dict.keys():
            li_dict = self.lexical_item_dict[pos]
            for li in li_dict.values():
                if OrthFeaturePair(li.orth, li.feature) not in orth_to_feat:
                    orth_to_feat.append(OrthFeaturePair(li.orth, li.feature))
        return orth_to_feat

class LexicalItem():

    def __init__(self, orth, feature):
        self.orth = orth
        self.feature = feature

# encapsulates a choices file morphology section paying attention only to what we wish to compare
class ChoicesMorphology():

    def __init__(self, choices):
        self.choices = choices
        self.position_class_dict = {}

    def parse_position_classes(self, pos):
        pos_pcs = self.choices[pos+'-pc']
        pc_dict = {}
        i=1
        for entry in pos_pcs:
            name = entry['name']
            order = entry['order']
            input = entry['inputs']
            lrts = entry['lrt']
            lrt_list = []
            for lrt in lrts:
                lrt_list.append(LexicalRuleType(lrt['feat'], lrt['lri']))
            pc_dict[i] = PositionClass(pos, name, order, input, lrt_list)
            i+=1
        self.position_class_dict[pos] = pc_dict

    def score(self, other):
        # todo handle duplicates differently?
        generated_orth_features = self.get_case()
        gold_orth_features = other.get_case()
        true_positives = 0
        #false_positives = 0
        true_negatives = 0
        #false_negatives = 0
        for pair in generated_orth_features:
            if pair in gold_orth_features:
                true_positives+=1
            else:
                true_negatives+=1
        if len(gold_orth_features)==0:
            print('no results to compare to in choices.up morphology')
            return 0
        return true_positives/len(gold_orth_features)

    def get_case(self):
        # use orthography as key because the position classes to lexical rule types might not line up
        # find self's orthography with case (wherever it is) and compare to other
        orth_to_feat = []
        for pos in self.position_class_dict.keys():
            for pc in self.position_class_dict[pos].values():
                for lrt in pc.lexical_rule_types:
                    for f in lrt.feature_list:
                        if f.name == 'case':
                            for lri in lrt.lri_list:
                                orth_to_feat.append(OrthFeaturePair(lri.orth, f))
        return orth_to_feat

class PositionClass():

    def __init__(self, pos, name, order, input, lexical_rule_types):
        self.pos = pos
        self.name = name
        self.order = order
        self.input = input
        self.lexical_rule_types = lexical_rule_types

    def __eq__(self, other):
        for lexical_rule in self.lexical_rule_types:
            if lexical_rule not in other.feature_list:
                return
        return True

# only pay attention to case features and instances with orthography
class LexicalRuleType():

    def __init__(self, features, lris):
        self.feature_list = self.parse_features(features)
        self.lri_list = self.parse_lris(lris)

    def parse_features(self, features):
        feature_list = []
        for f in features:
        # todo this could be changed to look in set of features
            if str(f['name'])=='case':
                feature_list.append(Feature('case', f['value']))
        return feature_list

    def parse_lris(self, lris):
        lri_list = []
        for lri in lris:
            if 'orth' in lri:
                lri_list.append(LexicalRuleInstance(lri['orth'].strip('-')))
        return lri_list

    def __eq__(self, other):
        for feature in self.feature_list:
            if feature not in other.feature_list:
                return False
        for lri in self.lri_list:
            if lri not in other.lri_list:
                return False
        return True

class LexicalRuleInstance():

    def __init__(self, orth):
        self.orth = orth

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

class Feature():

    valid_feature_names = ['case']
    # todo look at leipzig mappings
    valid_feature_values_map = {'nom' : {'nom', 'nominative'}, 'acc' : {'acc', 'accusative'},
                                    'erg' : {'erg', 'ergative'}, 'abs' : {'abs', 'absolutive'},
                                    'dat' : {'dat', 'dative'}, 'gen': {'gen', 'genative'},
                                    'loc': {'loc', 'locative'}, 'instrumental': {'instrumental'},
                                    'prepositional': {'prepositional'}}

    def __init__(self, name, value):
        self.name = name
        self.value = self.normalize_value(value)


    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def normalize_value(self, value):
        for standard in self.valid_feature_values_map.keys():
            if value in self.valid_feature_values_map[standard]:
                return standard

class OrthFeaturePair():

    def __init__(self, orth, feature):
        self.orth = orth
        self.name = feature.name
        self.value = feature.value

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __hash__(self):
        return hash(self.orth)+hash(self.name)+hash(self.value)







