#!/opt/python-3.4/bin/python3.4
# -*- coding: utf-8 -*-

import sys
from nltk import *
import nltk_tgrep
from xigt.codecs import xigtxml
from aggutils.xigt_nltk_tree import *

# Step through Xigt corpus and create array of NLTK ParentedTree objects, one per item. 
# FIXME: Check first that each item actually has a syntax tier.


def wo_counts_from_file(file_name):

    file = open(file_name, encoding='utf-8')
    xc = xigtxml.load(file)
    file.close()

    return wo_counts(get_forest(xc))

def get_forest(igtlist):
    forest = []
    missing_ps = 0
    for igt in igtlist:
        if 'w-ps' not in igt._dict:
            missing_ps += 1
            continue
        tree_str = format_ptb_tree(igt)
        #print(tree_str)
        ptbtree = ParentedTree.fromstring(tree_str)
        forest.append(ptbtree)

    if missing_ps:
        print('{} out of {} igt missing w-ps tier'.format(missing_ps, len(igtlist)))

    return forest

def wo_counts(forest):
    # Step through forest, and using nltk_tgrep, produce counts for each of the word
    # order patterns.

    # Initialize counters as dictionary:

    orders = {}
    orders['svo'] = 0
    orders['sov'] = 0
    orders['vos'] = 0
    orders['ovs'] = 0
    orders['vso'] = 0
    orders['osv'] = 0
    orders['sv'] = 0
    orders['vs'] = 0
    orders['ov'] = 0
    orders['vo'] = 0
    orders['v'] = 0
    orders['det_n'] = 0
    orders['n_det'] = 0
    orders['adj_n'] = 0
    orders['n_adj'] = 0

    # Dictionary with the same keys for the tgrep patterns.   Assumign I'll
    # want these to be array-valued, since I may need multiple tgrep patterns
    # per word-order pattern, given that nlkt_tgrep doesn't support macros.

    pats = {}
    pats['svo'] = [] # S < @SBJ=s < (@VP=v < (@VB $.. @OBJ)) : =s .. =v;
    pats['sov'] = [] # S < @SBJ=s < (@VP=v < (@OBJ $.. @VB)) : =s .. =v;
    pats['vos'] = [] # S < @SBJ=s < (@VP=v < (@VB $.. @OBJ)) : =v .. =s;
    pats['ovs'] = [] # S < @SBJ=s < (@VP=v < (@OBJ $.. @VB)) : =v .. =s;
    pats['vso'] = [] # S < (@VB $.. @SBJ $.. @OBJ);
    pats['osv'] = [] # S < (@OBJ $.. @SBJ $.. @VB);
    pats['sv'] = ['S < (NP $.. VP)', 'S < (NP $.. VB)', 'S < (NP $.. VBP)', 'S < (NP $.. VBZ)', 'S < (NP $.. VBG)',
                  'SQ < (NP $.. VP)', 'SQ < (NP $.. VB)', 'SQ < (NP $.. VBP)', 'SQ < (NP $.. VBZ)', 'SQ < (NP $.. VBG)'] # S < @SBJ=s < (@VPoB=v !< @OBJ !$ @OBJ): =s .. =v;
    pats['vs'] = ['S < (VP $.. NP)', 'S < (VB $.. NP)', 'S < (VBP $.. NP)', 'S < (VBZ $.. NP)', 'S < (VBG $.. NP)',
                  'SQ < (VP $.. NP)', 'SQ < (VB $.. NP)', 'SQ < (VBP $.. NP)', 'SQ < (VBZ $.. NP)', 'SQ < (VBG $.. NP)'] # S < @SBJ=s < (@VPoB=v !< @OBJ !$ @OBJ) : =v .. =s;
    pats['ov'] = [] # S [< (@VP < (@OBJ $.. @VB)) | < (@OBJ $.. @VB)] !< @SBJ;
    pats['vo'] = [] # S [< (@VP < (@VB $.. @OBJ)) | < (@VB $.. @OBJ)] !< @SBJ;
    pats['v'] = [] # S [< (@VP < @VB !< @OBJ) | < @VB !< @OBJ] !< SBJ;
    pats['det_n'] = [] # /NP/ < (DT .. /NN/);
    pats['n_det'] = [] # /NP/ < (/NN/ .. DT);
    pats['adj_n'] = [] # /NP/ < (JJ .. /NN/);
    pats['n_adj'] = [] # /NP/ < (/NN/ .. JJ);


    for tree in forest:
        for order in orders.keys():
            for pat in pats[order]:
                orders[order] += len(nltk_tgrep.tgrep_nodes(tree,pat)) # Might need to change this, not sure tgrep_nodes is the right function
    return orders

if __name__ == '__main__':
    file_name = sys.argv[1]
    orders = wo_counts_from_file(file_name)