#!/opt/python-3.4/bin/python3.4
# -*- coding: utf-8 -*-

import sys
from nltk import *
import nltk_tgrep
from xigt.codecs import xigtxml
#from aggutils.xigt_nltk_tree import *

# LING 575
# Spring 2015
#
# AGGREGATION Project - Sentential Negation
#
# Anthony Princing

numargs = len(sys.argv)

if numargs < 3:
   print("Usage: infer-negation.py XIGT_FILE CHOICES_FILE")
   os._exit(1)
else:
   xigt_input = sys.argv[1]
   choices_output = sys.argv[2]


def key4max(d):
   d_vals = list(d.values())
   d_keys = list(d.keys())
   retval = d_keys[d_vals.index(max(d_vals))]
   if retval != '':
      return retval
   else:
      return ''


negation_grams = ['NEG', 'NOT']
aux_tags = ['VERB']
mod_tags = ['ADV']
comp_tags = ['PRT']

exp_num = 0
neg_igts = {}
exponence = {}
neg1_orth_infl = {}
neg1_orth_free = {}
neg2_orth_infl = {}
neg2_orth_free = {}
neg1_morph_type = {'b': 0, 'f': 0}
neg2_morph_type = {'b': 0, 'f': 0}
neg1_free_type = {'h': 0, 'm': 0, 'c': 0, 'u': 0}
neg2_free_type = {'h': 0, 'm': 0, 'c': 0, 'u': 0}
neg1_loc_type = {'before': 0, 'after': 0}
neg2_loc_type = {'before': 0, 'after': 0}
neg1_mod_type = {'v': 0, 'vp': 0, 's': 0}
neg2_mod_type = {'v': 0, 'vp': 0, 's': 0}
bineg_types = {'bb': 'infl-infl',
               'bfh': 'infl-head',
               'bfm': 'infl-mod',
               'bfc': 'infl-comp',
               'fhb': 'infl-head',
               'fmb': 'infl-mod',
               'fcb': 'infl-comp',
               'fhfh': 'head-head',
               'fhfm': 'head-mod',
               'fhfc': 'head-comp',
               'fmfh': 'head-mod',
               'fcfh': 'head-comp',
               'fcfc': 'comp-comp',
               'fcfm': 'comp-mod',
               'fmfc': 'comp-mod',
               'fmfm': 'mod-mod'}


xigt_file = open(xigt_input, encoding='utf-8')
xc = xigtxml.load(xigt_file)
xigt_file.close()

for i,an_igt in enumerate(xc):

   neg_gram_count = 0
   gloss_tier = an_igt['g']


# Look at the morpheme gloss tier and count the number of negation
# grams in each phrase (igt). Keep track of how many of each count/type
# there are and the most common number will determine the negation 
# strategy (neg-exp).
#
   for j,gloss_item in enumerate(gloss_tier):
      if gloss_item.value().upper() in negation_grams:
         neg_gram_count += 1
         if neg_gram_count > 1:
            if gloss_item.value().upper() != prev_neg_gram:
               neg_gram_count -= 1
         prev_neg_gram = gloss_item.value().upper()
         if i in neg_igts:
            neg_igts[i].append(j)
         else:
            neg_igts[i] = [j]

   if neg_gram_count != 0:
      if str(neg_gram_count) in exponence: 
         exponence[str(neg_gram_count)] += 1
      else:
         exponence[str(neg_gram_count)] = 1


# Find the most common form of sentential negation represented
# in the IGT data and set the exponence accordingly.
#
if len(neg_igts) > 0:
   (exp_count, strategy) = max((exponence[keyval], keyval) for keyval in exponence)
   exp_num = int(strategy)
   if exp_num < 1 or exp_num > 2:
      print("Negation exponence of " + strategy + " is not supported.")
      os._exit(0)
else:
   print("No negation grams present in IGT.")
   os._exit(0)

# Only check igts with negation grams. Access various tiers
# to determine presence and values of other sentential negation
# features.
#
for a_key in neg_igts:
   an_igt = xc[a_key]
   neg_count = 0			# Identifies which negation gram within an IGT we are checking.
   word_tier   = an_igt['w']		# From here we get orthography of free negation morphemes.
   morph_tier  = an_igt['m']		# And this tier gives us bound (affix) negator orthography.
   gloss_tier  = an_igt['g']		# Helps us get bound negator orthography.
   wgloss_tier = an_igt['gw']		# Kind of the starting point for our tiered exploration.
   wpos_tier   = an_igt['w-pos']	# Used to determine negatore type (aux, mod, comp) and position (before, after).
   gwpos_tier  = an_igt['gw-pos']	# Also used to determine negator type and position.
   try:
      wps_tier    = an_igt['w-ps']	# Used to find what a modifier modifies (V, VP or S).
      wps_tier_exists = True
   except (KeyError, ValueError):
      wps_tier_exists = False

# Use the word gloss tier to determine if the negation morphemes
# are bound or free. If bound, they should contain at least one dash.
# Only look at igts with negation grams in them. 
#
   for wgloss_item in wgloss_tier:
      wgloss_value = wgloss_item.value().upper()
      if ("NEG" in wgloss_value) or ("NOT" in wgloss_value):
         neg_count += 1
         morph_type = ''
         neg_ortho = ''
         gwpos_tag = 'u'
         wpos_tag = 'u'
         if "-" in wgloss_value:
            morph_type = 'b'
            for gloss_item in gloss_tier:
               if (gloss_item.value().upper() in negation_grams) and (wgloss_item.id == gloss_item.content.split('[')[0]):
                  neg_ortho = morph_tier[gloss_item.alignment].value().lower()
         else:
            morph_type = 'f'
            neg_ortho = word_tier[wgloss_item.alignment].value().lower()

# Check the POS tag of the gloss line to try and determine what type
# of free negation morpheme this is. Also keep track of the relative
# positions of the negation morpheme and the verb. This can be used
# to select the order (just 'before' or 'after' currently).
#
            neg_loc = ''
            verb_loc = ''
            for gwpos_item in gwpos_tier:
               if wgloss_item.id == gwpos_item.alignment:
                  neg_loc = gwpos_item.alignment
                  if gwpos_item.value() in aux_tags:
                     gwpos_tag = 'h'
                  elif gwpos_item.value() in mod_tags:
                     gwpos_tag = 'm'
                  elif gwpos_item.value() in comp_tags:
                     gwpos_tag = 'c'
                  else:
                     gwpos_tag = 'u'
               elif gwpos_item.value() == 'VERB':
                  verb_loc = gwpos_item.alignment
               else:
                  pass
            if neg_loc == '' or verb_loc == '':
               gwloc_type = ''
            elif neg_loc > verb_loc:
               gwloc_type = 'after'
            elif verb_loc > neg_loc:
               gwloc_type = 'before'
            else:
               gwloc_type = ''

# Check the projected POS tag of the language line to try and determine
# what type of free negation morpheme this is. Also keep track of the relative
# positions of the negation morpheme and the verb. This can be used
# to select the order (just 'before' or 'after' currently).
#
            neg_loc = ''
            verb_loc = ''
            for wpos_item in wpos_tier:
               if word_tier[wgloss_item.alignment].id == wpos_item.alignment:
                  neg_loc = wpos_item.alignment
                  if wpos_item.value() in aux_tags:
                     wpos_tag = 'h'
                  elif wpos_item.value() in mod_tags:
                     wpos_tag = 'm'
                  elif wpos_item.value() in comp_tags:
                     wpos_tag = 'c'
                  else:
                     wpos_tag = 'u'
               elif wpos_item.value() == 'VERB':
                  verb_loc = wpos_item.alignment
               else:
                  pass
            if neg_loc == '' or verb_loc == '':
               wloc_type = ''
            elif neg_loc > verb_loc:
               wloc_type = 'after'
            elif verb_loc > neg_loc:
               wloc_type = 'before'
            else:
               wloc_type = ''

# Look through the w-ps tier to find what type of phrase contains the negator
# then assume it modifies a POS one step lower.
#
            neg_mod = 'v'
            adjacent_index = -1
            if wps_tier_exists:
               for wps_index, wps_item in enumerate(wps_tier):
                  if wps_index == adjacent_index:
                     adjacent_ps = wps_item.value().lower()
                     break
                  if wps_item.alignment == wgloss_item.alignment:
                     adjacent_index = wps_index + 1

               if adjacent_ps in ['root', 'sbar', 's']:
                  neg_mod = 's'
               elif adjacent_ps == 'vp':
                  neg_mod = 'vp'
               else:
                  neg_mod = 'v'


# Update the counters that will determine what type of each feature
# is the most common.
#
         if neg_count > 1:
            neg2_morph_type[morph_type] += 1
            if morph_type == 'f':
               neg2_free_type[gwpos_tag] += 1
               neg2_free_type[wpos_tag] += 1
               neg2_mod_type[neg_mod] += 1
               if gwloc_type != '':
                  neg2_loc_type[gwloc_type] += 1
               if wloc_type != '':
                  neg2_loc_type[wloc_type] += 1
               if neg_ortho in neg2_orth_free:
                  neg2_orth_free[neg_ortho] += 1
               else:
                  neg2_orth_free[neg_ortho] = 1
            else:
               if neg_ortho in neg2_orth_infl:
                  neg2_orth_infl[neg_ortho] += 1
               else:
                  neg2_orth_infl[neg_ortho] = 1
         else:
            neg1_morph_type[morph_type] += 1
            if morph_type == 'f':
               neg1_free_type[gwpos_tag] +=1
               neg1_free_type[wpos_tag] +=1
               neg1_mod_type[neg_mod] += 1
               if gwloc_type != '':
                  neg1_loc_type[gwloc_type] += 1
               if wloc_type != '':
                  neg1_loc_type[wloc_type] += 1
               if neg_ortho in neg1_orth_free:
                  neg1_orth_free[neg_ortho] += 1
               else:
                  neg1_orth_free[neg_ortho] = 1
            else:
               if neg_ortho in neg1_orth_infl:
                  neg1_orth_infl[neg_ortho] += 1
               else:
                  neg1_orth_infl[neg_ortho] = 1

        
# Open the choices file and start adding features.
#
choices_fd = open(choices_output, "w")
choices_fd.write("section=sentential-negation\n")
if exp_num != 0:
      choices_fd.write("neg-exp=" + str(exp_num) + "\n")
else:
      choices_fd.write("neg-exp=0\n")

# For simple negation, the morphemes can be of more that one type
# so simple will not always only pick the most frequent.
#
if exp_num == 1:
   if neg1_morph_type['b'] > 0:
      choices_fd.write("infl-neg=on\n")
   if neg1_morph_type['f'] > 0:
      neg1_free_type['u'] = 0
      free_type = key4max(neg1_free_type)
      free_orth = key4max(neg1_orth_free)
      free_loc = key4max(neg1_loc_type)
      if free_type == 'h':
         choices_fd.write("neg-aux=on\n")
      elif free_type == 'm':
         free_mod = key4max(neg1_mod_type)
         choices_fd.write("adv-neg=on\n")
         choices_fd.write("neg-mod=" + free_mod + "\n")
         choices_fd.write("neg-order=" + free_loc + "\n")
         choices_fd.write("neg-adv-orth=" + free_orth + "\n")
      elif free_type == 'c':
         choices_fd.write("comp-neg=on\n")
         choices_fd.write("comp-neg-order=" + free_loc + "\n")
         choices_fd.write("comp-neg-orth=" + free_orth + "\n")

else:	# Bipartite (exp_num = 2)

# Now that we've collected frequency values for features of the negators we can
# determine the most frequent orthography, morpheme type, free type, and
# negator location relative to what it modifies, for the negation morpheme(s)
#
   neg1_free = ''
   neg2_free = ''
   neg1_type = key4max(neg1_morph_type)
   neg2_type = key4max(neg2_morph_type)
   if neg1_type == 'f':
      neg1_free_type['u'] = 0
      neg1_free = key4max(neg1_free_type)
      neg1_orth = key4max(neg1_orth_free)
      if neg1_free in ('m', 'c'):
         neg1_loc = key4max(neg1_loc_type)
      if neg1_free == 'm':
         neg1_mod = key4max(neg1_mod_type)
   if neg2_type == 'f':
      neg2_free_type['u'] = 0
      neg2_free = key4max(neg2_free_type)
      neg2_orth = key4max(neg2_orth_free)
      if neg2_free in ('m', 'c'):
         neg2_loc = key4max(neg2_loc_type)
      if neg2_free == 'm':
         neg2_mod = key4max(neg1_mod_type)

# Finally, we determine what bipartite features and values to add to the 
# choices file based on the settings above.
#
   if neg1_type == 'b' or neg2_type == 'b':
      choices_fd.write("infl-neg=on\n")
   if neg1_type == 'f' or neg2_type == 'f':
      if neg1_free == 'h' or neg2_free == 'h':
         choices_fd.write("neg-aux=on\n")
      if neg1_free == 'm' or neg2_free == 'm':
         choices_fd.write("adv-neg=on\n")
      if neg1_free == 'c' or neg2_free == 'c':
         choices_fd.write("comp-neg=on\n")

   bineg_key = neg1_type + neg1_free + neg2_type + neg2_free
   bineg_val = bineg_types[bineg_key]
   choices_fd.write("bineg-type=" + bineg_types[bineg_key] + "\n")
   if neg1_type == 'b':
      choices_fd.write("neg1-type=b\n")
   else:
      choices_fd.write("neg1-type=f" + neg1_free + "\n")
      if neg1_free == 'm':
         if neg1_free == neg2_free:
            choices_fd.write("neg1-mod=" + neg1_mod + "\n")
            choices_fd.write("neg1-mod-orth=" + neg1_orth + "\n")
            choices_fd.write("neg1-mod-order=" + neg1_loc + "\n")
         else:
            choices_fd.write("neg-mod-" + bineg_val + "-neg=" + neg1_mod + "\n")
            choices_fd.write("neg1-mod-orth=" + neg1_orth + "\n")
            choices_fd.write("neg-mod-order-" + bineg_val + "-neg=" + neg1_loc + "\n")
      elif neg1_free == 'c':
         if neg1_free == neg2_free:
            choices_fd.write("comp-neg1-orth=" + neg1_orth + "\n")
            choices_fd.write("comp-neg1-order=" + neg1_loc + "\n")
         else:
            choices_fd.write("comp-neg-orth=" + neg1_orth + "\n")
            choices_fd.write("comp-neg-order-" + bineg_val + "=" + neg1_loc + "\n")
  
   if neg2_type == 'b':
      choices_fd.write("neg2-type=b\n")
   else:
      choices_fd.write("neg2-type=f" + neg2_free + "\n")
      if neg2_free == 'm':
         if neg2_free == neg1_free:
            choices_fd.write("neg2-mod=" + neg2_mod + "\n")
            choices_fd.write("neg2-mod-orth=" + neg2_orth + "\n")
            choices_fd.write("neg2-mod-order=" + neg2_loc + "\n")
         else:
            choices_fd.write("neg-mod-" + bineg_val + "-neg=" + neg2_mod + "\n")
            choices_fd.write("neg-mod-orth=" + neg2_orth + "\n")
            choices_fd.write("neg-mod-order-" + bineg_val + "-neg=" + neg2_loc + "\n")
      elif neg2_free == 'c':
         if neg2_free == neg1_free:
            choices_fd.write("comp-neg2-orth=" + neg2_orth + "\n")
            choices_fd.write("comp-neg2-order=" + neg2_loc + "\n")
         else:
            choices_fd.write("comp-neg-orth=" + neg2_orth + "\n")
            choices_fd.write("comp-neg-order-" + bineg_val + "=" + neg2_loc + "\n")


choices_fd.close()
