#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
infer-yn-ques.py [-options] Xigt-input [Xigt-input ...]

auther: Brandon Gahler
"""

import sys
import getopt
import xigt
from xigt.codecs import xigtxml
import re
import glob
from collections import Counter
from extract_wo_xigt import *
import codecs

numargs = len(sys.argv)

if numargs < 3:
    # run for all of 567
    input_files = list(glob.glob("/../data/567/*/*/testsuite-enriched-2.xml"))
    output_files = list("/../analyses/2015-575/yesno/" + x[-33:-25] for x in input_files)
    #print(output_files)
else:
    # run for only the xigt file specified
    input_files = [sys.argv[1]]
    output_files = [sys.argv[2]]

segment = re.compile('p1\\[(\\d+):(\\d+)\\]')

# an a priori collection of yn ques grams
ques_grams = ['Q', '?', 'INTERROGATIVE', 'Interrogative', 'ques', 'QUES', 'QUEST', 'if+not:CONJ', 'INT', 'IP', 'interr']
neg_grams = ['not', 'NEG']
particle_grams = ques_grams + neg_grams

auxiliaries = ['do', 'did', 'will', 'can', 'could', 'is', 'are', 'was', 'where']
wh_words = {'who', 'what', 'when', 'where', 'why', 'how'}

for xigt_input, out_file in zip(input_files,output_files):

    choices_output = codecs.open(out_file, 'w', 'utf-8')
    print(out_file[-3:])

    # process the Xigt file into a XigtCorpus class so that we can work with it
    corpus = xigtxml.load(open(xigt_input))

    init_particles = Counter()
    final_particles = Counter()
    other_particles = Counter()
    neg_particles = Counter()

    question_igt = list()
    other_igt = list()

    ques_morph = False

    # process all of the IGTs in the corpus
    for igt in corpus.igts:
        # get the tier with the translation
        trans_item = igt.get('t')[0]
        trans = trans_item.get_content()
        is_question = False
        #for a in auxiliaries:
        trans_str = trans.encode('ascii', 'ignore').lower()
        trans_str_split = trans_str.split()
        #print(trans_str)
        if trans_str[-1] == '?' or trans_str_split[0] in auxiliaries:
            if len(filter(wh_words.__contains__, trans_str_split)):
                continue


            question_igt.append(igt)
            is_question = True

            phrase = igt.get('p')[0].get_content()
            phrase_no_q_mark = phrase.rstrip('?').rstrip()
            particle = False
            for x in igt.get('gw'):
                if x.get_content() in particle_grams:
                    particle = True
                    word = igt.get_item(x.alignment)
                    word_content = word.get_content()
                    if word_content == '?':
                        continue

                    s = segment.match(word.segmentation)
                    if int(s.group(1)) == 0:
                        init_particles[word_content] += 1
                    elif int(s.group(2)) == len(phrase) or int(s.group(2)) == len(phrase_no_q_mark):
                        final_particles[word_content] += 1
                    else:
                        other_particles[word_content] += 1
                    if x.get_content() in neg_grams:
                        neg_particles[word_content] += 1

            if not particle:
                for x in igt.get('g'):
                    if x.get_content() in ques_grams:
                        morph = igt.get_item(x.alignment)
                        print("morph: " + morph.get_content())
                        print(x.get_content())
                        print(morph.segmentation)
                        if "[" in morph.segmentation:
                            ques_morph = True


        #print(is_question)
        if not is_question:
            other_igt.append(igt)


    for mid_particle in other_particles.keys():
        if mid_particle not in neg_particles:
            if mid_particle in init_particles:
                del init_particles[mid_particle]
            if mid_particle in final_particles:
                del final_particles[mid_particle]


    ip = [(x[1], x[0], True) for x in init_particles.items()] if len(init_particles) else [(0,None,True)]
    fp = [(x[1], x[0], False) for x in final_particles.items()] if len(final_particles) else [(0,None,False)]

    p = max(ip + fp)
    if p[1]:
        particle_out = "q-part=on\nq-part-order={}\nq-part-orth=".format( ("before" if p[2] else "after"))
        choices_output.write(particle_out)
        choices_output.write(p[1])
        choices_output.write("\n")
        print(particle_out)
        print(p[1])

    if len(other_particles):
        print("warning: potential sentence middle question particles: " + ", ".join(other_particles.keys()))

    if ques_morph:
        choices_output.write("q-infl=on\n")
        print("q-infl=on")

    print("")
    question_forest = get_forest(question_igt)

    for i in range(len(question_igt)):
        print(question_igt[i].get('n').get('n2').get_content())
        print(question_igt[i].get('n').get('n3').get_content())
        print(question_igt[i].get('n').get('n4').get_content())
        if i < len(question_forest):
            print(question_forest[i])
        print("")

    question_wo_counts = wo_counts(question_forest)
    other_forest = get_forest(other_igt)
    other_wo_counts = wo_counts(other_forest)

    #print( len(corpus.igts) )
    #print( len(other_igt) )
    question_sv = (question_wo_counts['sv'], question_wo_counts['vs'])
    other_sv = (other_wo_counts['sv'], other_wo_counts['vs'])
    if sum(question_sv):
        if max(question_sv) < 1.5*min(question_sv) or max(other_sv) < 1.5*min(other_sv):
            print(question_sv)
            print(other_sv)
            print("word order data inconclusive")
        elif (question_sv[0] > question_sv[1]) != (other_sv[0] > other_sv[1]):
            print(question_sv)
            print(other_sv)
            print("inversion?")
            choices_output.write("q-inv=on\nq-inv-verb=main-aux\n")
        else:
            print("wo matches")
    else:
        print("no wo data from questions")
    print("")
    choices_output.flush()
    choices_output.close()