#!/opt/python-2.7/bin/python2.7
# -*- coding: iso-8859-15 -*-
import re, sys, copy;

###########
# Variables
###########

#TODO:Change these dictionaries into an external mapping file which can be read by choices.
subjectFeatures = {"nom":"nominative", "Nom":"nominative"};
objectFeatures = {"acc":"accusative", "Acc":"accusative"};
subject_feature_vals = {'SBJ','SUBJECT','S','SUBJ'}
object_feature_vals = {'OBJ','OBJECT','O','OBJECT'}

reflexiveFeatures = {};
noneFeatures = {"loc":"locative", "gen":"genitive", "dat":"dative", "refl":"reflexive"};

perFeatures = {"1":"1st", "1st":"1st", "2nd":"2nd", "2":"2nd", "3":"3rd", "3rd":"3rd", "4":"4th", "4th":"4th", "n1":"non-1st", "non-1st":"non-1st", "n2":"non-2nd", "non-2nd":"non-2nd", "non-3rd":"non-3rd", "n3":"non-3rd"};
numFeatures = {"s":"singular", "sg":"singular", "pl":"plural", "singular":"singular", "plural":"plural"};
genFeatures = {"f":"feminine", "m":"masculine", "n":"neuter"};
featureGramsMap = {'number': { 'SG': { 'SG', 'SING', 'SINGULAR'}, 'PL': {'PL', 'PLURAL'}, 'DU': {'DUAL', 'DU'},
                               'PAU': {'PAUCAL', 'PAU', 'PA'}, 'COL': {'COL', 'COLLECTIVE'}, 'TRN': {'TRN', 'TRANSNUMERAL', 'TRANSNUM'}},
               'person': { '1': {'1', '1ST'}, '2': {'2', '2D', '2ND'},
                           '3': {'3', '3RD', '3D'}, 'INCL': {'INCL', 'INCLUSIVE'},
                           'EXCL': {'EXCL', 'EX', 'EXCLUSIVE'},
               'gender': {'MASC': {'MASC','M','MASCULINE'}, 'FEM': {'FEM','F','FEMININE'},
                          'NEUT': {'NEUT', 'N','NEUTER'}, 'ANIM': {'ANIM', 'ANIMATE'},
                          'INANIM': {'INANIM', 'INAN', 'INANIMATE'}, 'ZO': {'ZO', 'ZOO', 'ZOIC'},
                          'R': {'R', 'RATIONAL', 'RAT'} }}}
gramsToFeatName = {'SG': 'number', 'SING': 'number', 'SINGULAR':'number', 'PL':'number', 'PLURAL': 'number',
               '1': 'person', '1ST': 'person','2': 'person', '2D': 'person', '2ND': 'person','3': 'person',
               '3RD': 'person', '3D': 'person',
               'MASC': 'gender','M': 'gender','MASCULINE': 'gender', 'FEM': 'gender','F': 'gender','FEMININE': 'gender',
               'NEUT': 'gender', 'N': 'gender','NEUTER': 'gender', 'DUAL':'number', 'DU':'number',
               'PAUCAL':'number','PAU':'number','PA':'number', 'R':'gender', 'RAT':'gender','RATIONAL':'gender',
               'TRN':'number','TRANSNUM':'number','TRANSNUMERAL':'number','COL':'number','ANIM':'gender','ANIMATE':'gender',
               'INANIM':'gender','INAN':'gender','INANIMATE':'gender','ZO':'gender','ZOIC':'gender','INCL':'person',
               'INCLUSIVE':'person', 'EXCL':'person','EXCLUSIVE':'person','EX':'person', "NON-F":"gender","NON-N":"gender",
               "NON-M":"gender", 'NON-FEM': 'gender', 'NON-MASC':'gender','NON-NEUT':'gender', 'NON-FEMININE':'gender',
               'NON-MASCULINE':'gender', 'NON-NEUTRAL':'gender','NON-ANIM':'gender','NON-RAT':'gender','NON-ANIM-RAT':'gender'
                }

gramsToFeatVal = {'SG': 'SG', 'SING': 'SG', 'SINGULAR':'SG', 'PL':'PL', 'PLURAL': 'PL',
                  'DU': 'DU', 'DUAL': 'DU', 'PAU':'PAU', 'PAUCAL': 'PAU', 'PA':'PAU',
                  'COL':'COL', 'COLLECTIVE': 'COL', 'COLL': 'COL', 'TRN':'TRN', 'TRANSNUM':'TRN',
                  'TRANSUMERAL':'TRN', 'INCL':'INCL', 'INCLUSIVE':'INCL','EXCL':'EXCL',
                  'EXCLUSIVE':'EXCL', 'EX':'EXCLUSIVE','ANIM':'ANIM','ANIMATE':'ANIM','INAN':'INAN',
                  'INANIMATE':'INAN','ZO':'ZO','ZOIC':'ZO', 'R':'R','RATIONAL':'R','RAT':'R',
               '1': '1', '1ST': '1','2': '2', '2D': '2', '2ND': '2','3': '3',
               '3RD': '3', '3D': '3', "NON-F":"NON-F","NON-FEM":"NON-F","NON-FEMININE":"NON-F",
                  "NON-M":"NON-M","NON-MASC":"NON-M","NON-MASCULINE":"NON-M",
                  "NON-N":"NON-N","NON-NEUT":"NON-N","NON-NEUTRAL":"NON-N",
               'MASC': 'M','M': 'M','MASCULINE': 'M', 'FEM': 'F','F': 'F','FEMININE': 'F',
               'NEUT': 'N', 'N': 'N','NEUTER': 'N'}


caseFeatures = {};
caseNoneFeatures = {};
caseFeatures.update(subjectFeatures);
caseFeatures.update(objectFeatures);
caseNoneFeatures.update(reflexiveFeatures);
caseNoneFeatures.update(noneFeatures);

#negFeatures = {"neg":"plus", "not":"plus"};
negFeatures = {};
tenseFeatures = {"past":"past", "pst":"past", "pres":"present", "prs":"present", "fut":"future", "npst":"nonpast", "nfut":"nonfuture", "future":"future", "present":"present"}; 
aspectFeatures = {"imp":"imperfective", "prf":"perfective"};
#moodFeatures = {"SBJV":"subjunctive"};
perNumFeatures = {"1pl":("1st","plural"), "1sg":("1st","singular"), "2pl":("2nd","plural"),
                  "2sg":("2nd","singular"), "3sg":("3rd","singular"), "3pl":("3rd","plural"),
                  "1plural":("1st","plural"), "1singular":("1st","singular"), "2plural":("2nd","plural"),
                  "2singular":("2nd","singular"), "3singular":("3rd","singular"), "3plural":("3rd","plural")};
perNumSplit = {"1pl":("1","pl"), "1sg":("1","sg"), "2pl":("2","pl"),
               "2sg":("2","sg"), "3sg":("3","sg"), "3pl":("3","pl")};

#OTHER = PTCP:"participle", INDEF:"indefinite", REFL:"reflexive", ANA:???, INF:"infinitive", LOC: locative, GEN:genative

nounFeatures = {};
verbFeatures = {};
nounFeatures.update(perFeatures);
nounFeatures.update(numFeatures);
nounFeatures.update(genFeatures);
nounFeatures.update(caseFeatures);
nounFeatures.update(caseNoneFeatures);
verbFeatures.update(tenseFeatures);
verbFeatures.update(aspectFeatures);
#verbFeatures.update(moodFeatures);
verbFeatures.update(negFeatures);

master_features = {};
master_features.update(nounFeatures);
master_features.update(verbFeatures);

internal_feature_name = ["p", "i", "a", "x", "e", "tense", "sf", "comp", "cat", "conj", "rel", "agr", "opt", "cont", "sign", "number", "adv", "topic", "complete", "det", "scratch", "word", "passive", "last", "mod", "head", "per", "activated", "altkey", "altkeyrel", "alts", "apparg", "arg", "args", "arg-st", "aspect", "body", "carg", "c-arg", "c-cont", "cform", "clause", "clause-key", "cog-st", "compkey", "comps", "conj-dtr", "coord", "coord-rel", "coord-strat", "ctxt", "dtr", "fc", "first", "gtop", "harg", "hc-light", "hcons", "head-dtr", "hook", "icons", "icons-key", "index", "inflected", "instloc", "key", "key-arg", "keycomp", "keyrel", "keys", "label-name", "larg", "lbl", "lcoord-dtr", "l-hndl", "light", "l-index", "list", "lkeys", "local", "l-periph", "ltop", "marker-dtr", "mc", "meta-prefix", "meta-suffix", "mkg", "modified", "mood", "needs-affix", "node", "nonconj-dtr", "non-head-dtr", "non-local", "non-marker-dtr", "ocompkey", "opt-cs", "periph", "png", "posthead", "pred", "presup", "que", "rcoord-dtr", "rels", "rest", "result", "r-hndl", "r-index", "r-periph", "rstr", "rule-name", "slash", "sort", "spec", "speci", "spr", "stem", "subj", "synsem", "target", "tp", "val", "wlink", "xarg"] 

###########
# classes
###########

class Choices:
    def __init__(self, iso, gloss_list, lexitem_list, pos_tag, hyphens=False):
        self.iso = iso;
        self.lexicon = Lexicon(pos_tag,hyphens);
        self.pos_tag = pos_tag
        self.feature_count = self.extract_features(gloss_list);
        self.feature_list = list(self.feature_count.keys());
        self.gloss_count = self.extract_features(gloss_list);
        self.gloss_preds = list(self.gloss_count.keys());
        self.processFeatures();
        self.build_lexitem_lexicon(gloss_list, lexitem_list);
        

    def build_lexitem_lexicon(self, gloss_list, lexitem_list):
        assert len(lexitem_list) == len(gloss_list), "Can't build lexicon from unmatched lists.";
        for i in range(len(lexitem_list)):
            self.lexicon.add_whole_lexitem(lexitem_list[i], gloss_list[i]);

    def extract_features(self, gloss_list):
        '''
        @param gloss_list (List of Strings) - Looks for features from this list.

        This takes a list of glosses and returns a dictionary with the 
        feature strings and frequency.
        '''
        feature_list = {};
        for gloss in gloss_list:
            for morpheme in gloss.get_tokens():
                if not morpheme.is_root:
                    for gram in morpheme.get_tokens():
                        if not self.pos_tag is None:
                            if gram.has_tag(self.pos_tag):
                                feature = validate_feature(str(gram));
                                if str(gram) in feature_list:
                                    feature_list[feature] += 1;
                                else:
                                    feature_list[feature] = 1;
        return feature_list;

    '''
        olzama's addition for 575 spring 2015 project
        Recognize more PNG features and values and add them to the choices.
    '''
    def processPNG(self):
        self.other_list = copy.deepcopy(self.feature_list);
        for feat in self.feature_list:
            if feat in gramsToFeatVal:
                featName = gramsToFeatName[feat]
                featVal = gramsToFeatVal[feat]
                if featName == 'person':
                    self.other_list.remove(feat);
                    self.person_list.append(featVal)
                elif featName == 'number':
                    self.other_list.remove(feat);
                    self.number_list.append(featVal)
                elif featName == 'gender':
                    self.other_list.remove(feat);
                    self.gender_list.append(featVal)

    def processFeatures(self):
        self.person_list = [];
        self.number_list = [];
        self.gender_list = [];
        self.case_list = [];
        self.neg_list = [];
        self.tense_list = [];
        self.aspect_list = [];
        self.mood_list = [];
        self.other_list = copy.deepcopy(self.feature_list);
        for feat in self.feature_list:
            if feat in perNumFeatures:
                #(personFeat, numberFeat) = perNumFeatures[feat];
                self.other_list.remove(feat);
                (per, num) = perNumFeatures[feat];
                self.person_list.append(per);
                self.number_list.append(num);
            elif feat in perFeatures:
                self.other_list.remove(feat);
                self.person_list.append(perFeatures[feat]);
            elif feat in numFeatures:
                self.other_list.remove(feat);
                self.number_list.append(numFeatures[feat]);
            elif feat in genFeatures:
                self.other_list.remove(feat);
                self.gender_list.append(genFeatures[feat]);
            elif feat in caseFeatures:
                self.other_list.remove(feat);
                self.case_list.append(caseFeatures[feat]);
            elif feat in negFeatures:
                self.other_list.remove(feat);
                self.neg_list.append(negFeatures[feat]);
            elif feat in tenseFeatures:
                self.other_list.remove(feat);
                self.tense_list.append(tenseFeatures[feat]);
            elif feat in aspectFeatures:
                self.other_list.remove(feat);
                self.aspect_list.append(aspectFeatures[feat]);
#            elif feat in moodFeatures:
#                self.other_list.remove(feat);
#                self.mood_list.append(moodFeatures[feat]);
            elif feat in caseNoneFeatures:
                self.other_list.remove(feat);

    def print_to(self, file=sys.stdout, language=None, version=23):
        file.write("version="+str(version) +"\n");
        self.print_gen_section(file, language_name=language);
        self.print_word_order(file);
        self.print_number_section(file);
        self.print_person_section(file);
        self.print_gender_section(file);
        self.print_case_section(file);
        file.write("section=direct-inverse\n\n");
        self.print_tense_section(file);
        self.print_other_features(file);
        self.print_sentential_negation(file);
        file.write("section=coordination\n\n");
        file.write("section=matrix-yes-no\n\n");
        file.write("section=arg-opt\n\n");
        self.lexicon.print_to(file);
        file.write("section=test-sentences\n\n");
        file.write("section=gen-options\n\n");

    def print_gen_section(self, file=sys.stdout, archive=False, language_name=None):
        '''
        @param file (File) - The location where this section will be written
        @param archive (Boolean) - If the choices file should be allowed to be archived
          default: Do not archive the choices file
        
        Writes the general section of this choices file to the proper location.
        '''
        file.write("section=general\n");
        if language_name:
            file.write("language="+language_name+"\n");
        else:
            language_name = file.name.split("/")[-1];
            file.write("language="+self.iso+"_"+language_name+"\n");
        file.write("iso-code="+self.iso+"\n");
        if archive:
            file.write("archive=yes\n");
        else:
            file.write("archive=no\n");
        file.write("comment=Automatically generated by MOM\n");
        file.write("\n");

    def print_word_order(self, file=sys.stdout):
        '''
        @param file (File) - The location where this section will be written

        This writes the minimal word order section, so the grammar will compile.
        '''
        file.write("section=word-order\n");
        file.write("word-order=svo\n");
        file.write("has-dets=no\n");
        file.write("has-aux=no\n");
        file.write("\n");

    #TODO:update-ify
    def print_other_features(self, file=sys.stdout):
        feature_list = self.other_list;
        mood_list = self.mood_list;
        file.write("section=other-features\n");
        for i in range(1,len(feature_list)+1):
            file.write("  feature"+str(i)+"_name="+feature_list[i-1]+"\n");
            file.write("  feature"+str(i)+"_type=head\n");
            file.write("    feature"+str(i)+"_value1_name="+feature_list[i-1]+"_T\n");
            file.write("      feature"+str(i)+"_value1_supertype1_name="+feature_list[i-1]+"\n");
        file.write("\n");
#        file.write("  feature"+str(len(feature_list)+2)+"_name=myMood\n");
#        file.write("  feature"+str(len(feature_list)+2)+"_type=head\n");
#        moods = list(set(mood_list));
#        for i in range(len(moods)):
#            file.write("    feature"+str(len(feature_list)+2)+"_value"+str(i+1)+"_name="+moods[i]+"\n");
#            file.write("    feature"+str(len(feature_list)+2)+"_value"+str(i+1)+"_supertype1_name=myMood\n");
                       
    #TODO:update-ify
    def print_number_section(self, file=sys.stdout):
        feature_list = self.number_list;
        file.write("section=number\n");
        number_set = list(set(feature_list));
        for i in range(len(number_set)):
            file.write("  number"+str(i+1)+"_name="+number_set[i]+"\n");
        file.write("\n");

    #TODO:update-ify
    #TODO?:clean up the if's by moving forth to the first check.
    def print_person_section(self, file=sys.stdout):
        feature_list = self.person_list;
        file.write("section=person\n");
        num_set = set(feature_list);
        if "1st" in num_set:
            if "non-1st" in num_set:
                file.write("  person=1-non-1\n");
            elif "4th" in num_set:
                file.write("  person=1-2-3-4\n");
            elif "2nd" in num_set:
                file.write("  person=1-2-3\n");
            elif "3rd" in num_set:
                file.write("  person=1-2-3\n");
        elif "2nd" in num_set:
            if "non-2nd" in num_set:
                file.write("  person=2-non-2\n");
            elif "4th" in num_set:
                file.write("  person=1-2-3-4\n");
            else:
                file.write("  person=1-2-3\n");
        elif "3rd" in num_set:
            if "non-3rd" in num_set:
                file.write("  person=3-non-3\n");
            elif "4th" in num_set:
                file.write("  person=1-2-3-4\n");
            else:
                file.write("  person=1-2-3\n");
        elif "4th" in num_set:
            file.write("  person=1-2-3-4\n");
        else:
            file.write("  person=none\n");
        file.write("  first-person=none\n");
        file.write("\n");
        #No code for incl-excl first person yet

    #TODO:update-ify
    #TODO?:add animacy handling
    def print_gender_section(self, file=sys.stdout):
        feature_list = self.gender_list;
        file.write("section=gender\n");
        gen_set = feature_list;
        if "masculine" in gen_set or "feminine" in gen_set:
            file.write("  gender1_name=masculine\n");
            file.write("    gender1_supertype1_name=gender\n");
            file.write("  gender2_name=feminine\n");
            file.write("    gender2_supertype1_name=gender\n");
        if "neuter" in gen_set:
            file.write("  gender3_name=neuter\n");
            file.write("    gender3_supertype1_name=gender\n");
        file.write("\n");

    #TODO:update-ify
    #TODO?:move all the print statements to the end...
    def print_case_section(self, file=sys.stdout):
        feature_list = self.case_list;
        file.write("section=case\n");
        case_set = set(feature_list);
        if "nominative" in case_set:
            if "accusative" in case_set:
                if "ergative" in case_set:
                    file.write("  case-marking=split-n\n");
                    file.write("    split-n-nom-case-name=nominative\n");
                    file.write("    split-n-acc-case-name=accusative\n");
                    file.write("    split-n-erg-case-name=ergative\n");
                    file.write("    split-n-abs-case-name=absolutive\n");
                elif "absolutive" in case_set:
                    file.write("  case-marking=split-n\n");
                    file.write("    split-n-nom-case-name=nominative\n");
                    file.write("    split-n-acc-case-name=accusative\n");
                    file.write("    split-n-erg-case-name=ergative\n");
                    file.write("    split-n-abs-case-name=absolutive\n");
                else:
                    file.write("  case-marking=nom-acc\n");
                    file.write("    nom-acc-nom-case-name=nominative\n");
                    file.write("    nom-acc-acc-case-name=accusative\n");
            elif "ergative" in case_set:
                file.write("  case-marking=split-n\n");
                file.write("    split-n-nom-case-name=nominative\n");
                file.write("    split-n-acc-case-name=accusative\n");
                file.write("    split-n-erg-case-name=ergative\n");
                file.write("    split-n-abs-case-name=absolutive\n");
            elif "absolutive" in case_set:
                file.write("  case-marking=split-n\n");
                file.write("    split-n-nom-case-name=nominative\n");
                file.write("    split-n-acc-case-name=accusative\n");
                file.write("    split-n-erg-case-name=ergative\n");
                file.write("    split-n-abs-case-name=absolutive\n");
            else:
                file.write("  case-marking=nom-acc\n");
                file.write("    nom-acc-nom-case-name=nominative\n");
                file.write("    nom-acc-acc-case-name=accusative\n");
        elif "accusative" in case_set:
            if "ergative" in case_set:
                file.write("  case-marking=split-n\n");
                file.write("    split-n-nom-case-name=nominative\n");
                file.write("    split-n-acc-case-name=accusative\n");
                file.write("    split-n-erg-case-name=ergative\n");
                file.write("    split-n-abs-case-name=absolutive\n");
            elif "absolutive" in case_set:
                file.write("  case-marking=split-n\n");
                file.write("    split-n-nom-case-name=nominative\n");
                file.write("    split-n-acc-case-name=accusative\n");
                file.write("    split-n-erg-case-name=ergative\n");
                file.write("    split-n-abs-case-name=absolutive\n");
            else:
                file.write("  case-marking=nom-acc\n");
                file.write("    nom-acc-nom-case-name=nominative\n");
                file.write("    nom-acc-acc-case-name=accusative\n");
        else:
            file.write("  case-marking=none\n");
            file.write("    nom-acc-nom-case-name=nominative\n");
            file.write("    nom-acc-acc-case-name=accusative\n");
        #TODO?:Add the other case choices... ie. erg-abs, tripartite, split-S, fluid-S
        if "nominative" in case_set:
            case_set.remove("nominative");
        if "accusative" in case_set:
            case_set.remove("accusative");
        if " ergative" in case_set:
            case_set.remove("ergative");
        if "absolutive" in case_set:
            case_set.remove("absolutive");
        case_list = list(case_set);
        for i in range(len(case_list)):
            file.write("  case"+str(i+1)+"_name="+case_list[i]+"\n");
        file.write("\n");

    #TODO:update-ify
    def print_tense_section(self, file=sys.stdout):
        tense_list = self.tense_list;
        aspect_list = self.aspect_list;
        file.write("section=tense-aspect\n");
        if len(tense_list) > 0: 
            file.write("  tense-definition=choose\n");
            if "past" in tense_list:
                file.write("  past=on\n");
            if "present" in tense_list:
                file.write("  present=on\n");
            if "future" in tense_list:
                file.write("  future=on\n");
            if "nonpast" in tense_list:
                file.write("  nonpast=on\n");
            if "nonfuture" in tense_list:
                file.write("  nonfuture=on\n");
        if len(aspect_list) > 0:
            aspects = list(set(aspect_list));
            for i in range(len(aspects)):
                file.write("  aspect"+str(i+1)+"_name="+aspects[i]+"\n");
                file.write("    aspect"+str(i+1)+"_supertype1_name=aspect\n");
        file.write("\n");

    def print_sentential_negation(self, file=sys.stdout):
        negFeats = self.neg_list;
        file.write("section=sentential-negation\n");
        if "plus" in negFeats:
            file.write("  infl-neg=on\n");
        file.write("\n");

class Lexicon:
    def __init__(self, pos_tag, hyphens=False):
        self.lexitems = [];
        self.pos_tag = pos_tag
        self.pcs = [];
        self.overlap_table = None
        self.pc_total = 0
        self.hyphens = hyphens
#        self.master_list = {};

    def remove_grandchild_inputs(self):
        for pc in self.pcs:
            pc.remove_grandchild_inputs();

    def print_overlap_table(self):
        s = [ [ row.get_id()+"||"] + [ e[0].get_id()[5:]+":"+str(e[1])[:6] for e in self.overlap_table[row] ]
                for row in self.overlap_table ]
        lens = [max(map(len,col)) for col in zip(*s)]
        fmt = '\t'.join('{{:{}}}'.format(x) for x in lens)
        table = [fmt.format(*row) for row in s]
        print ('\n'.join(table))

    def create_overlap_table(self):
        # create an overlap table for efficient overlap reading 
        # and compression
        # use pcs as keys for easy lookup later
        self.overlap_table = {}
        for pc_i in self.pcs:
            # make an entry in the overlap table
            # a list of overlaps with all other vpcs
            self.overlap_table[pc_i] = []
            for pc_j in self.pcs:
                if not pc_i == pc_j: # ignore overlap of identity 
                    if pc_j in self.overlap_table:
                    # if the value is already in the table, use that
                        overlap = [ i[1] for i in self.overlap_table[pc_j] 
                                    if i[0] == pc_i ][0]
                        self.overlap_table[pc_i].append((pc_j,overlap))
                    else: # otherwise compute it
                        self.overlap_table[pc_i].append(
                                (pc_j,vpc_i.input_overlap(pc_j)))



    def update_overlap_table(self,pc_update,pc_remove):
        '''pc2 has been merged into pc1, so we
           1. remove the row/columns corresponding to pc2
           2. update the row/columns correponding to pc1'''

        # remove the row
        self.overlap_table.pop(pc_remove)

        # remove the refs in others rows (ie, the column)
        # note should remove 1 each row
        num_removed = 0
        for k in self.overlap_table:
            for i in self.overlap_table[k]:
                if i[0] == pc_remove:
                    num_removed+=1
                    self.overlap_table[k].remove(i)

        # update row for pc_update (and refs to in other rows)
        self.overlap_table[pc_update] = [] 
        for pc_i in self.pcs:
          if not pc_update == pc_i:
              overlap = pc_update.input_overlap(pc_i)
              self.overlap_table[pc_update].append((pc_i,overlap))
              ## update converse entry in the table
              ## overlap is reciprocal
              for i in range(len(self.overlap_table[pc_i])-1):
                if self.overlap_table[pc_i][i][0] == pc_update:
                  self.overlap_table[pc_i][i] = (pc_update,overlap)

             
    def compact_lexicon(self, min_overlap=1.0):
        if self.overlap_table == None:
            self.create_overlap_table()
            # self.print_overlap_table()
        (pc1, pc2, overlap) = self.get_best_overlap();
        # print "best overlap was ",str(overlap),"pc1:",pc1.get_id(),"pc2:",pc2.get_id()
        if overlap >= min_overlap:
            #sys.stderr.write("Overlap:"+str(overlap)+"\n");
            pc1.merge(pc2)
            for pc in self.pcs:
                if not pc1 in pc.cycles_found:
                    if pc1 in pc.cycles_checked:
                        pc.cycles_checked.remove(pc1)
            self.replace(pc1, pc2)
            self.update_overlap_table(pc1,pc2)
            self.compact_lexicon(min_overlap)

    def replace(self, pc1, pc2):
        for pc in self.pcs:
            if pc.has_input(pc2):
                pc.add_input(pc1);
                pc.remove_input(pc2);
        self.pcs.remove(pc2);

    def get_best_overlap(self):
        best_1 = None;
        best_2 = None;
        best_over = -1.0;
        for pc_i in self.pcs:
            for j in range(len(self.pcs)-1):
                if not self.pcs[j] == pc_i:
                    over = self.overlap_table[pc_i][j][1]
                    if over > best_over:
                        best_1 = vpc_i;
                        best_2 = self.overlap_table[pc_i][j][0];
                        best_over = over;
        return (best_1, best_2, best_over);

    def print_to(self, file=sys.stdout):
        file.write("section=lexicon\n");
        file.write("  noun1_det=imp\n");
        file.write("    noun1_stem1_orth=n1\n");
        file.write("    noun1_stem1_pred=_n1_n_rel\n");
        file.write("  noun2_det=imp\n");
        file.write("    noun2_stem1_orth=n2\n");
        file.write("    noun2_stem1_pred=_n2_n_rel\n");
        for lexitem in self.lexitems:
            #print ("Verb stem, gloss:", verb.stems[0].orth, verb.stems[0].pred)
            lexitem.print_to(file);
        for pc in self.pcs:
            pc.print_to(file);
        file.write("\n");

    def add_whole_lexitem(self, lexitem, gloss):
        try:
            lexitem_unicode = str(lexitem)
            gloss_unicode = str(gloss)
        except UnicodeDecodeError:
            sys.stderr.write("Decode Error output skipped for "+str(lexitem)+"\n");
            return;
        root = lexitem.get_root(self.pos_tag);
        if root is not None:
            try:
                (features, pred) = self.aligned_features(root, gloss);
                assert pred is not None, "No predicate aligned with lexical item ("+str(lexitem)+") and gloss ("+str(gloss)+")";
                input = self.add_lexitem(pred, str(root), features);
                prefix = lexitem.get_left_token(root);
            except AssertionError as e:
                sys.stderr.write(str(e)+"\n");
                sys.stderr.write("Multiple Pred Error OR multiple tenses: output skipped for "+str(lexitem)+"\n");
                return;
            while (prefix):
                (features, pred) = self.aligned_features(prefix, gloss);
                input = self.add_prefix(features, prefix, input);
                prefix = lexitem.get_left_token(prefix);
            suffix = lexitem.get_right_token(root);
            while(suffix):
                (features, pred) = self.aligned_features(suffix, gloss);
                input = self.add_suffix(features, suffix, input);
                suffix = lexitem.get_right_token(suffix);

    def aligned_features(self, morph, gloss):
        features = [];
        pred = None;
        head_feature = None
        for gloss_morph in gloss.get_tokens():
            if gloss_morph.is_aligned(morph):
                tokens = gloss_morph.get_tokens()
                for gloss_gram in tokens:
                    feature = validate_feature(str(gloss_gram));
                    if self.pos_tag:
                        if gloss_gram.has_tag(self.pos_tag):
                            #TODO:Consider a better way of dealing with multiple
                            #potential predicates.
                            assert pred is None, "Already assigned pred";
                            pred = feature;
                        else:
                            #olzama 2015-05-28 addition to Baseline for Ling575 project PNG
                            if feature.upper() in subject_feature_vals or gloss_gram in object_feature_vals:
                                head_feature = feature.upper()
                            elif feature.upper() in gramsToFeatName or feature.upper().startswith('CL'):
                                features.append(Feature(feature,self.pos_tag,head_feature))
                            elif feature in perNumSplit:
                                (person, number) = perNumSplit[feature];
                                features.append(Feature(person, self.pos_tag));
                                features.append(Feature(number, self.pos_tag));
                            else:
                                features.append(Feature(feature, self.pos_tag));
                if pred is None:
                    if self.pos_tag:
                        if gloss_morph.has_tag(self.pos_tag):
                            if len(features) > 0:
                                pred = str(features.pop());
                            else:
                                pred = "NO_PREDVAL"
        self.check_features(features);
#        if not features:
#            sys.stderr.write("Gloss:"+str(gloss)+" Morph:"+str(morph)+"\n");
        return (features, pred);

    def check_features(self, features):
        tense_features = 0;
        aspect_features = 0;
        for feat in features:
            if str(feat) in tenseFeatures:
                tense_features += 1;
            elif str(feat) in aspectFeatures:
                aspect_features += 1;
        assert tense_features < 2, "too many tenses.";
        assert aspect_features < 2, "too many aspect.";
            

    def add_lexitem(self, pred, orth, features):
        stem = Stem(pred, orth, self.pos_tag);
        new_lexitem = LexicalItem(features, self.pos_tag);
        new_lexitem.add_stem(stem);
        new_lexitem.set_parent(self);
        for lexitem in self.lexitems:
            if lexitem.matches_features(features):
                lexitem.add_stem(stem);
                return lexitem;
        self.lexitems.append(new_lexitem);
        return new_lexitem;

    def add_prefix(self, features, prefix, input):
        return self.add_position_class(features, prefix, input, True);

    def add_suffix(self, features, suffix, input):
        return self.add_position_class(features, suffix, input, False);

    def add_position_class(self, features, affix, input, is_prefix):
        is_clitic = "clitic" in affix.tags
        lri = Lexical_Rule_Instance(str(affix), is_prefix, is_clitic, self.hyphens);
        lrt = Lexical_Rule_Type(features);
        lrt.add_instance(lri);
        for pc in self.pcs:
            if pc.has_lexical_rule_type(lrt) and pc.is_prefix == is_prefix:
                pc.add_input(input);
                return pc;
        new_pc = Position_Class(is_prefix, self.pos_tag);
        new_pc.add_lexical_rule_type(lrt);
        new_pc.add_input(input);
        new_pc.set_parent(self);
        self.pc_total = self.pc_total + 1
        #print ("setting a new ID for a new position class: %s", self.pc_total)
        new_pc.setID(self.pc_total)
        self.pcs.append(new_pc);
        return new_pc;

    def get_id(self):
        return None;

    def index(self, object):
        if object in self.lexitems:
            return self.lexitems.index(object);
        elif object in self.pcs:
            return self.pcs.index(object);
        else:
            raise Exception("Object not in Lexicon\n",str(object));

class Choice:
    def set_parent(self, parent):
        self.parent = parent;

    def get_id(self):
        if self.parent.get_id() is not None:
            return self.parent.get_id()+"_"+self.get_name()+str(self.parent.index(self)+1);
        else:
            return self.get_name()+str(self.parent.index(self)+1);

    def get_name(self):
        raise Exception("Prototype Function undefined");

class Feature(Choice):
    def __init__(self, feature, pos_tag,head_feature=None):
        assert feature not in perNumSplit, "Feature should have been split";
        if not head_feature is None:
           featHead = head_feature
        else:
           featHead = 'subj' #default
        if feature.upper() in gramsToFeatName:
           featName = gramsToFeatName[feature.upper()]
           featVal = gramsToFeatVal[feature.upper()]
           self.name = featName
           self.value = featVal
           self.head = featHead
        elif feature.upper().startswith('CL'):
           self.name = 'gender'
           self.value = feature.upper()
           self.head = featHead
        elif feature in master_features:
            self.value = master_features[feature];
            if feature in caseFeatures:
                self.name="case";
            if feature in caseNoneFeatures:
                self.name=None;
            elif feature in perFeatures:
                self.name="person";
            elif feature in numFeatures:
                self.name="number";
            elif feature in genFeatures:
                self.name="gender";
            elif feature in tenseFeatures:
                self.name="tense";
            elif feature in aspectFeatures:
                self.name="aspect";
            elif feature in negFeatures:
                self.name="negation";
            if feature in verbFeatures:
                self.head = "verb";
            elif feature in subjectFeatures:
                self.head = "subj";
                self.value = "nom";
            elif feature in objectFeatures:
                self.head = "obj";
                self.value = "acc";
            else:
                self.head = None;
        else:
            self.value = str(feature)+"_T";
            self.name = str(feature);
            #if "V" in self.pos_tag or "v" in self.pos_tag:
            self.head = "HEAD";
            #else:
                #self.head = "noun"

    def __eq__(self, object):
        if isinstance(object, Feature):
            if self.get_value() == object.get_value():
                return True;
            else:
                return False;
        else:
            return False;

    def __str__(self):
        return self.value;

    def get_value(self):
        return self.value;
    
    def get_name(self):
        return "feat";

    def print_to(self, subject, object, file=sys.stdout):
        if self.head:
            file.write("    "+self.get_id()+"_name="+self.name+"\n");
            file.write("    "+self.get_id()+"_value="+self.value+"\n");
            file.write("    "+self.get_id()+"_head="+self.head+"\n");
            if self.head == "subj":
                return (True, False);
            if self.head == "obj":
                return (False, True);
            if self.head == "verb" or self.head == "noun":
                return (subject, object);
        elif self.name is None:
            return(False, False);
        elif subject and object:
            file.write("    "+self.get_id()+"_name="+self.name+"\n");
            file.write("    "+self.get_id()+"_value="+self.value+"\n");
            file.write("    "+self.get_id()+"_head=subj\n");
            file.write("    "+self.get_id()+"_name="+self.name+"\n");
            file.write("    "+self.get_id()+"_value="+self.value+"\n");
            file.write("    "+self.get_id()+"_head=obj\n");
        elif subject:
            file.write("    "+self.get_id()+"_name="+self.name+"\n");
            file.write("    "+self.get_id()+"_value="+self.value+"\n");
            file.write("    "+self.get_id()+"_head=subj\n");
        elif object:
            file.write("    "+self.get_id()+"_name="+self.name+"\n");
            file.write("    "+self.get_id()+"_value="+self.value+"\n");
            file.write("    "+self.get_id()+"_head=obj\n");        
        return (subject, object);

class Stem(Choice):
    def __init__(self, pred, orth, pos_tag):
        self.orth = orth;
        if re.match("_\w+_\w_rel", pred):
            self.pred = pred;
        else:
            #TODO: this will screw up things like "adv"
            if "V" in pos_tag or "v" in pos_tag:
                self.pred = "_"+pred+"_v_rel";
            elif "N" in pos_tag or "n" in pos_tag:
                self.pred = "_"+pred+"_n_rel"

    def __eq__(self, object):
        if isinstance(object, Stem):
            if self.orth == object.get_orth() and self.pred == object.get_pred():
                return True;
            else:
                return False;
        else:
            return False;

    def print_to(self, file=sys.stdout):
        file.write("    "+self.get_id()+"_orth="+self.orth+"\n");
        #renormalize: if underscores in the pred were originally dots, change them back to dots now.
        file.write("    "+self.get_id()+"_pred="+re.sub("~",".",self.pred)+"\n");

    def get_orth(self):
        return self.orth;

    def get_pred(self):
        return self.pred;

    def get_name(self):
        return "stem";

class LexicalItem(Choice):
    def __init__(self, features, pos_tag):
        self.pos_tag = pos_tag
        self.features = [];
        self.stems = [];
        for feature in features:
            self.add_feature(feature);

    def __eq__(self, object):
        if isinstance(object, LexicalItem):
            if self.matches_features(object.features) and self.matches_stems(object.stems):
                return True;
            else:
                return False;
        else:
            return False;

    def print_to(self, file=sys.stdout):
        #TODO: this will screw up "adv"
        if "V" in self.pos_tag or "v" in self.pos_tag:
            file.write("  "+self.get_id()+"_valence=trans\n");
        subject = True;
        object = False
        for feature in self.features:
            (subject, object) = feature.print_to(subject, object, file);
        for stem in self.stems:
            stem.print_to(file);

    def matches_features(self, features):
        if len(features) != len(self.features):
            return False;
        for feature in features:
            if not self.has_feature(feature):
                return False;
        return True;

    def has_feature(self, feature):
        for feat in self.features:
            if feature == feat:
                return True;
        return False;

    def add_feature(self, feature):
        feature.set_parent(self);
        self.features.append(feature);

    def matches_stems(self, stems):
        if len(stems) != len(self.stems):
            return False;
        for stem in stems:
            if not self.has_stem(stem):
                return False;
        return True;

    def has_stem(self, s):
        for stem in self.stems:
            if stem == s:
                return True;
        return False;

    def add_stem(self, new_stem):
        for stem in self.stems:
            if stem == new_stem:
                return stem;
        new_stem.set_parent(self);
        self.stems.append(new_stem);
        return new_stem;

    def get_name(self):
        #TODO: fix this for adv
        if "V" in self.pos_tag or "v" in self.pos_tag:
            return "verb"
        else:
            return "noun"

    def index(self, object):
        if object in self.features:
            return self.features.index(object);
        elif object in self.stems:
            return self.stems.index(object);
        else:
            raise Exception("Object not in verb or noun");

class Lexical_Rule_Instance(Choice):
    def __init__(self, orth, is_prefix, is_clitic, hyphens, inflecting=True):
        self.orth = orth;
        self.inflecting = inflecting;
        if hyphens:
            delim = "-"
            if is_clitic:
                delim = "="
            if is_prefix:
                self.orth = self.orth + delim
            else:
                self.orth = delim + self.orth

    def __eq__(self, object):
        if isinstance(object, Lexical_Rule_Instance):
            if self.orth == object.orth and self.inflecting == object.inflecting:
                return True;
            else:
                return False;
        else:
            return False;

    def print_to(self, file=sys.stdout):
        if self.inflecting:
            file.write("    "+self.get_id()+"_inflecting=yes\n");
        else:
            raise Exception("Non Inflecting? Really?");
        file.write("    "+self.get_id()+"_orth="+self.orth+"\n");

    def get_name(self):
        return "lri";

class Lexical_Rule_Type(Choice):
    def __init__(self, features):
        self.features = [];
        self.instances = [];
        for feature in features:
            self.add_feature(feature);

    def __eq__(self, object):
        if isinstance(object, Lexical_Rule_Type):
            if object.matches_features(self.features) and object.matches_instances(self.instances):
                return True;
            else:
                return False;
        else:
            return False;

    def merge(self, other):
        assert self.matches_features(other.get_features()), "weird merge...";
        for instance in other.get_instances():
            self.add_instance(instance);

    def matches_instances(self, instances):
        if len(instances) != len(self.instances):
            return False;
        for instance in instances:
            if not self.has_instance(instance):
                return False;
        return True;

    def has_instance(self, instance):
        for inst in self.instances:
            if inst == instance:
                return True;
        return False;

    def matches_features(self, features):
        if len(features) != len(self.features):
            return False;
        for feature in features:
            if not self.has_feature(feature):
                return False;
        return True;

    def has_feature(self, feature):
        for feat in self.features:
            if feature == feat:
                return True;
        return False;

    def add_feature(self, feature):
        if not self.has_feature(feature):
            feature.set_parent(self);
            self.features.append(feature);

    def add_instance(self, instance):
        if not self.has_instance(instance):
            instance.set_parent(self);
            self.instances.append(instance);

    def get_name(self):
        return "lrt";

    def get_features(self):
        return self.features;

    def get_instances(self):
        return self.instances;

    def print_to(self, file=sys.stdout):
        subject = True;
        object = False
        for feature in self.features:
            #disable features output until we make feature names and values more meaningful for the Matrix.
            (subject, object) = feature.print_to(subject, object, file);
        for lri in self.instances:
            lri.print_to(file);

    def index(self, object):
        if object in self.features:
            return self.features.index(object);
        elif object in self.instances:
            return self.instances.index(object);
        else:
            raise Exception("Object not in lrt");

class Position_Class(Choice):
    def __init__(self, is_prefix, pos_tag):
        self.inputs = [];
        self.pos_tag = pos_tag
        self.cycles_checked = [self]
        self.cycles_found = [self]
        self.lexical_rule_types = [];
        self.is_prefix = is_prefix;
        self.id = "DEFAULT_ID"

    def setID(self,id):
        self.id = id
    
    def get_id(self):
        pos = "NONE"
        #TODO: fix this for "adv"
        if "V" in self.pos_tag or "v" in self.pos_tag:
            pos = "verb"
        if "N" in self.pos_tag or "n" in self.pos_tag:
            pos = "noun"
        return pos + "-pc" + str(self.id)

    def add_lexical_rule_type(self, lrt):
        if not self.has_lexical_rule_type(lrt):
            lrt.set_parent(self);
            self.lexical_rule_types.append(lrt);

    def merge(self, other):
        for input in other.get_inputs():
            self.add_input(input);
        for lrt in other.get_lexical_rule_types():
            match_found = False;
            for own_lrt in self.lexical_rule_types:
                if lrt.matches_features(own_lrt.get_features()):
                    own_lrt.merge(lrt);
                    match_found = True;
            if not match_found:
                self.add_lexical_rule_type(lrt);

    def input_overlap(self, pc2):
        if self.is_prefix != pc2.is_prefix or self.cycle_found(pc2) or pc2.cycle_found(self):
#            or self.has_input(pc2) or pc2.has_input(self)):
            return 0.0;
        else:
            shared = 0;
            for input in self.get_inputs():
                if pc2.has_input(input):
                    shared += 1;
            return float(shared)/float(len(pc2.get_inputs())+len(self.inputs)-shared);

    def add_input(self, input):
        if not self.has_input(input):
            if issubclass(input.__class__, Position_Class):
                if not input.cycle_found(self):
                    self.inputs.append(input);
            else:
                self.inputs.append(input);

    def cycle_found(self, node):
        if node in self.cycles_found:
            return True
        else:
            if not node in self.cycles_checked:
                self.cycles_checked.append(node)
                for input in self.get_inputs():
                    if issubclass(input.__class__, Position_Class):
                        res = input.cycle_found(node)
                        if res:
                            self.cycles_found.append(node)
                            return True
        return False

    def remove_input(self, input):
        assert input in self.inputs, "Can't remove what isn't there.";
        if len(self.inputs) == 1:
            raise Exception("Trying to remove last item.");
        if len(self.inputs) > 1:
            self.inputs.remove(input);

    def has_input(self, input):
        if input in self.inputs:
            return True;
        else:
            return False;

    def remove_grandchild_inputs(self):
        '''
        This function looks for inputs which can be reached by going through
        other inputs.

        e.g. If A -> B and C
        and B -> C
        C is removed as an input to A.
        '''
        for direct_input in self.get_inputs():
            if issubclass(direct_input.__class__, Position_Class):
#                if self in direct_input.get_inputs():
#                    direct_input.remove_input(self);
                self.remove_redundant_inputs([direct_input], direct_input);

    def remove_redundant_inputs(self, checked, current_input):
#        if self in current_input.get_inputs():
#            current_input.remove_input(self);
        #print "remove redundant inputs:"
        #print "checked:",[i.get_id() for i in checked]
        #print "current input:",current_input.get_id()
        for input in current_input.get_inputs():
            if input not in checked and self.has_input(input) :
                sys.stderr.write("REMOVE:"+input.get_id()+" from "+self.get_id()+"\n");
                self.remove_input(input);
            elif input not in checked and issubclass(input.__class__, Position_Class):
                self.remove_redundant_inputs(checked+[input], input);

    def remove_grandchild_inputs_old(self):
        raise Exception("Deprciated.");
        inputs = [];
        for direct_input in self.get_inputs():
            if issubclass(direct_input.__class__, Position_Class):
                inputs.extend(direct_input.get_inputs());
        debug = 0;
        while inputs:
            if debug > 100000:
                break;
            debug += 1;
#            print len(inputs);
            input = inputs.pop();
            if self.has_input(input):
                sys.stderr.write("REMOVE ("+str(debug)+"):"+input.get_id()+" from "+self.get_id()+"\n");
                self.remove_input(input);
            if issubclass(input.__class__, Position_Class):
                inputs.extend(input.get_inputs());

    def get_lexical_rule_types(self):
        return self.lexical_rule_types;

    def get_inputs(self):
        return self.inputs;

    def has_lexical_rule_type(self, lrt):
        for lexical_rule_type in self.lexical_rule_types:
            if lrt == lexical_rule_type:
                return True;
        return False;

    def get_name(self):
        #TODO: fix this for adv
        if "V" in self.pos_tag or "v" in self.pos_tag:
            return "verb"
        else:
            return "noun"

    def index(self, object):
        if object in self.inputs:
            return self.inputs.index(object);
        elif object in self.lexical_rule_types:
            return self.lexical_rule_types.index(object);
        else:
            raise Exception("Object not in pc");

    def print_to(self, file=sys.stdout):
        if self.is_prefix:
            file.write("  "+self.get_id()+"_order=prefix\n");
        else:
            file.write("  "+self.get_id()+"_order=suffix\n");
        file.write("  "+self.get_id()+"_inputs=");
        first = True;
        for input in self.inputs:
            if first:
                first = False;
            else:
                file.write(", ");
            file.write(input.get_id());
        file.write("\n");
        for lrt in self.lexical_rule_types:
            lrt.print_to(file);

    #OLD STUFF!
#class Junk:
#    def addFeature(self, feature):
#        self.features.append(feature);

#    def matchFeatures(self, feat_list):
#        match = 0;
#        for feat in feat_list:
#            if feat in perNumSplit:
#                (feat1, feat2) = perNumSplit[feat];
#                if self.containsFeature(feat1) and self.containsFeature(feat2):
#                    match += 2;
#                else:
#                    return False;
#            elif self.containsFeature(feat):
#                match += 1;
#            else:
#                return False;
#        if match == len(self.features):
#            return True;
#        else:
#            return False;

#    def containsFeature(self, f):
#        contains = False;
#        for feature in self.features:
#            if f == feature:
#                contains = True;
#        return contains;

#    def setOrth(self, spell):
#        if spell == "0":
#            self.spelling = "";
#        else:
#            self.spelling = spell;

#    def containsOrth(self, orth):
#        contains = false;
##        for spelling in self.spellings:
#        if self.spelling == orth:
#            contains = true;
#        return contains;

#    def printLexicalItem(self, i):
#        agr_sub = True;
#        agr_obj = False;
#        passes = 1;
##        for feat in self.features:
##            if feat in subjectFeatures:
##                agr_sub = self.features.index(feat);
##            if feat in objectFeatures:
##                agr_obj = self.features.index(feat);
#        for f in range(len(self.features)):
#            passes = 1;
#            if self.features[f] in subjectFeatures:
#                agr_sub = True;
#                agr_obj = False;
#            if self.features[f] in objectFeatures:
#                agr_sub = False;
#                agr_obj = True;
#            if self.features[f] in reflexiveFeatures:
#                agr_sub = True;
#                agr_obj = True;
#            if self.features[f] in noneFeatures:
#                agr_sub = False;
#                agr_obj = False;
#            if agr_sub and agr_obj:
#                passes += 1;
#            elif not (agr_sub or agr_obj):
#                passes += -1;
#            if self.features[f] in nounFeatures:
#                while(passes > 0):
#                    if self.features[f] in caseFeatures:
#                        print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_name=case");
#                    elif self.features[f] in perFeatures:
#                        print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_name=person");
#                    elif self.features[f] in numFeatures:
#                        print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_name=number");
#                    elif self.features[f] in genFeatures:
#                        print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_name=gender");
#                    if self.features[f] in subjectFeatures:
#                        print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_value=nom");
#                    elif self.features[f] in objectFeatures:
#                        print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_value=acc");
#                    elif self.features[f] not in caseNoneFeatures:
#                        print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_value="+nounFeatures[self.features[f]]);
#                    if agr_sub and (agr_obj == (passes > 1)) and self.features[f] not in reflexiveFeatures:
#                        print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_head=subj");
#                    elif agr_obj and  not (passes > 1) and self.features[f] not in reflexiveFeatures:
#                        print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_head=obj");
#                    passes += -1;
#            elif self.features[f] in verbFeatures:
#                if self.features[f] in tenseFeatures:
#                    print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_name=tense");
#                elif self.features[f] in aspectFeatures:
#                    print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_name=aspect");
##                elif self.features[f] in moodFeatures:
##                    print "    verb"+str(i+1)+"_feat"+str(f+1)+"_name=myMood";
#                elif self.features[f] in negFeatures:
#                    print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_name=negation");
#                print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_value="+verbFeatures[self.features[f]]);
#                print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_head=verb");
#            else:
#                print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_name="+self.features[f]);
#                print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_value="+self.features[f]+"_T");
#                print ("    verb"+str(i+1)+"_feat"+str(f+1)+"_head=verb");
#        print ("  verb"+str(i+1)+"_valence=trans"); #EDIT LATER?
#        print ("    verb"+str(i+1)+"_stem1_orth="+re.sub("\)","\\\)",self.spelling));
#        print ("    verb"+str(i+1)+"_stem1_pred="+self.pred);

class Affix:
    def __init__(self, pre_flag, gloss, orth, pos_tag):
        self.prefix = pre_flag;
        self.pos_tag = pos_tag
        self.features = [];
        feature_list = re.split("\.",gloss);
        for feature in feature_list:
            if feature in perNumSplit:
                (person, number) = perNumSplit[feature];
                if (not self.containsFeature(person)):
                    self.addFeature(person);
                if (not self.containsFeature(number)):
                    self.addFeature(number);
            elif(not self.containsFeature(feature)):
                self.addFeature(feature);
        if orth == "0":
            self.spelling = "";
        else:
            self.spelling = orth;

        self.inputs = set();
 
    def addFeature(self, f):
        self.features.append(f);
   
    def containsFeature(self, f):
        contains = False;
        for feature in self.features:
            if f == feature:
                contains = True;
        return contains;

    def matchFeatures(self, feat_list):
        match = 0;
        for feat in feat_list:
            if feat in perNumSplit:
                (feat1, feat2) = perNumSplit[feat];
                if self.containsFeature(feat1) and self.containsFeature(feat2):
                    match += 2
                else:
                    return False;
            elif self.containsFeature(feat):
                match += 1;
            else:
                return False;
        if match == len(self.features):
            return True;
        else:
            return False;

    def addInput(self, input):
        self.inputs.add(input);

    def printAffix(self, i):
        pos = "NONE"
        #TODO: fix this for "adv"
        if "V" in self.pos_tag or "v" in self.pos_tag:
            pos = "verb"
        if "N" in self.pos_tag or "n" in self.pos_tag:
            pos = "noun"
            
#        print "AFFIX!";
#        print "  verb-slot"+str(i+1)+"_name=VS-"+str(i+1);
        if(self.prefix):
            print ("   " + pos + "-slot"+str(i+1)+"_order=before");
        else:
            print ("   " + pos + "-slot"+str(i+1)+"_order=after");
        inputs = list(self.inputs);
        for p in range(len(inputs)):
            print ("   " + pos + "-slot"+str(i+1)+"_input"+str(p+1)+"_type="+inputs[p]);
        print ("   " + pos + "-slot"+str(i+1)+"_morph1_orth="+re.sub("\)","\\\)", self.spelling));
#        for f in range(len(self.features)):
#            print "      verb-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_name="+self.features[f];
#            print "      verb-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_value="+self.features[f]+"_T";
#            print "      verb-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_head=verb";
        agr_sub = True;
        agr_obj = False;
        passes = 1;
        for f in range(len(self.features)):
            passes = 1;
            if self.features[f] in subjectFeatures:
                agr_sub = True;
                agr_obj = False;
            if self.features[f] in objectFeatures:
                agr_sub = False;
                agr_obj = True;
            if self.features[f] in reflexiveFeatures:
                agr_sub = True;
                agr_obj = True;
            if self.features[f] in noneFeatures:
                agr_sub = False;
                agr_obj = False;
            if agr_sub and agr_obj:
                passes += 1;
            elif not (agr_sub or agr_obj):
                passes += -1;
            if self.features[f] in nounFeatures:
                while (passes > 0):
                    if self.features[f] in caseFeatures:
                        print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_name=case");
                    elif self.features[f] in perFeatures:
                        print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_name=person");
                    elif self.features[f] in numFeatures:
                        print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_name=number");
                    elif self.features[f] in genFeatures:
                        print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_name=gender");
                    if self.features[f] in subjectFeatures:
                    #THIS ONLY WORKS WITH NOM/ACC grammars... =/
                        print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_value=nom");
                    elif self.features[f] in objectFeatures:
                        print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_value=acc");
                    elif self.features[f] not in caseNoneFeatures:
                        print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_value="+nounFeatures[self.features[f]]);
                    if agr_sub and (agr_obj == (passes > 1)) and self.features[f] not in reflexiveFeatures:
                        print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_head=subj");
                    elif agr_obj and not (passes > 1) and self.features[f] not in reflexiveFeatures:
                        print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_head=obj");
                    passes += -1;
            elif self.features[f] in verbFeatures:
                if self.features[f] in tenseFeatures:
                    print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_name=tense");
                elif self.features[f] in aspectFeatures:
                    print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_name=aspect");
#                elif self.features[f] in moodFeatures:
#                    print "      verb-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_name=myMood";
                elif self.features[f] in negFeatures:
                    print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_name=negation");
                print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_value="+verbFeatures[self.features[f]]);
                print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_head=verb");
            else:
                print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_name="+self.features[f]);
                print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_value="+self.features[f]+"_T");
                print ("     " + pos + "-slot"+str(i+1)+"_morph1_feat"+str(f+1)+"_head=verb");


def validate_feature(gram):
    feature = gram.lower();
    if feature in internal_feature_name:
        feature = feature+"_feat";
    assert feature is not None, "None isn't a valid feature.";
    return feature;

