#!/opt/python-3.4/bin/python3.4
# -*- coding: utf-8 -*-
'''
@author David Wax, modified by Olga Zamaraeva
@version 2.0 Light, xigt support only
'''
import os
import re
import sys
import string
import copy
import random
import pickle
import tempfile
from operator import itemgetter
import alignables
import choices
xigt_path = os.path.abspath("../../xigt/branches/develop")
sys.path.append(xigt_path)
from xigt.codecs import xigtxml

#boosting flags for alignment
ONE2ONE_MATCH = "match"
ONE2ONE_SEARCH = "search"
ONE2ONE_EXACT = "exact"
ONE2ONE_MATCH_TYPES = [ONE2ONE_MATCH, ONE2ONE_SEARCH, ONE2ONE_EXACT]
ONE2ONE_ALIGN = "align"
ALIGN_BOOSTS = [ONE2ONE_MATCH, ONE2ONE_SEARCH, ONE2ONE_EXACT, ONE2ONE_ALIGN]

#boosting flags for morpheme splitting
BUILD_WORDS = "build words"
KNOWN_MORPHS = "known" #NOT IMPLEMENTED!
IGNORE = "ignore"
MORPH_BOOSTS = [BUILD_WORDS, KNOWN_MORPHS, IGNORE]

#flags for IGT identification
ORIGINAL = "original"
CLEANED = "clean"
ALL = "both"

#flags for line/word identification
LANGUAGE = "language"
GLOSS = "gloss"
TRANSLATION = "translation"
LANGUAGE_MORPHEME = "language_morpheme"
GLOSS_MORPHEME = "gloss_morpheme"
TRANSLATION_MORPHEME = "trans_morph"
LANGUAGE_FEATURE = "language_feature"
GLOSS_FEATURE = "gloss_feature"
TRANSLATION_FEATURE = "translation_feature"
LANGUAGE_WORD = "language_word"
LANGUAGE_WORD_MORPHEME = "language_word_morpheme"
LANGUAGE_WORD_FEATURE = "lang_word_feature"
GLOSS_WORD = "gloss_word"
GLOSS_WORD_MORPHEME = "gloss_word_morpheme"
GLOSS_WORD_FEATURE = "gloss_word_feature"
TRANSLATION_WORD = "translation_word"
TRANSLATION_WORD_MORPHEME = "translation_word_morpheme"
TRANSLATION_WORD_FEATURE = "translation_word_feature"

#Line level tags are used for aligning elements of a full line.
LINE_LEVEL = [LANGUAGE, GLOSS, TRANSLATION, 
              LANGUAGE_MORPHEME, GLOSS_MORPHEME, TRANSLATION_MORPHEME,
              LANGUAGE_FEATURE, GLOSS_FEATURE, TRANSLATION_FEATURE]
#Word level tags are used for aligning elements of words.
WORD_LEVEL = [LANGUAGE_WORD, GLOSS_WORD, TRANSLATION_WORD,
               LANGUAGE_WORD_MORPHEME, GLOSS_WORD_MORPHEME, TRANSLATION_WORD_MORPHEME,
               LANGUAGE_WORD_FEATURE, GLOSS_WORD_FEATURE, TRANSLATION_WORD_FEATURE]

WORD_TYPES = [LANGUAGE, GLOSS, TRANSLATION,
              LANGUAGE_WORD, GLOSS_WORD, TRANSLATION_WORD]
MORPHEME_TYPES = [LANGUAGE_MORPHEME, GLOSS_MORPHEME, TRANSLATION_MORPHEME,
                 LANGUAGE_WORD_MORPHEME, GLOSS_WORD_MORPHEME, TRANSLATION_WORD_MORPHEME]
FEATURE_TYPES = [LANGUAGE_FEATURE, GLOSS_FEATURE, TRANSLATION_FEATURE,
                 LANGUAGE_WORD_FEATURE, GLOSS_WORD_FEATURE, TRANSLATION_WORD_FEATURE]

LANGUAGE_TYPES = [LANGUAGE, LANGUAGE_MORPHEME, LANGUAGE_FEATURE,
                  LANGUAGE_WORD, LANGUAGE_WORD_MORPHEME, LANGUAGE_WORD_FEATURE]
GLOSS_TYPES = [GLOSS, GLOSS_MORPHEME, GLOSS_FEATURE,
               GLOSS_WORD, GLOSS_WORD_MORPHEME, GLOSS_WORD_FEATURE]
TRANSLATION_TYPES = [TRANSLATION, TRANSLATION_MORPHEME, TRANSLATION_FEATURE,
                     TRANSLATION_WORD, TRANSLATION_WORD_MORPHEME, TRANSLATION_WORD_FEATURE]


SOURCE_FLAGS = [LANGUAGE, GLOSS, TRANSLATION, 
                LANGUAGE_MORPHEME, GLOSS_MORPHEME, TRANSLATION_MORPHEME,
                LANGUAGE_FEATURE, GLOSS_FEATURE, TRANSLATION_FEATURE,
                LANGUAGE_WORD, GLOSS_WORD, TRANSLATION_WORD,
                LANGUAGE_WORD_MORPHEME, GLOSS_WORD_MORPHEME, TRANSLATION_WORD_MORPHEME,
                LANGUAGE_WORD_FEATURE, GLOSS_WORD_FEATURE, TRANSLATION_WORD_FEATURE]

known_glosses = {"A","ABE","ABL","ABS","ACC","ACCOM","ACT","ADJ","ADE","ADM","ADV","AFF","AG","AGT","AGR","ALL","AL",
                 "ALLOC","ALIEN","AN","AND","ANT","ANTE","ANTIC","ANTIP","AP","AOR","APP","APL","APPL","APPR","APRX",
                 "ART","ASP","ASS","AT","ATT","AUD","AUG","AUX","B","BE","BEN","C","CAP","CAU","CAUS","CENT","CF",
                 "CIRC","CIT","CL","CLF","CNJ","CNSQ","COL","COM","COMP","COMPL","CPL","CONC","COND","CONJ","CONN",
                 "CONT","CTN","CNTR","COP","COR","CRAS","CRS","CVB","D","DAT","DE","DEC","DECL","DEF","DEI","DEIX",
                 "DEL","DET","DEM","DEO","DEP","DES","DESI","DEST","DET","DETR","DH","DIM","DIR","DISJ","DIST",
                 "DISTR","DITR","DLM","DM","DO","DR","DS","DU","DUB","DUR","DY","DYAD","DYN","E","E","ELA","EMP",
                 "EPIS","ERG","ESS","EV","EVID","EVIT","EX","EXCL","EXCLAM","EX.DUR","EXH","EXIST","EXO","EXP",
                 "EXPER","EXPL","F","FEM","FACT","FAM","FIN","FOC","FORM","FP","FR","FREQ","FMR","FUT","G","GEN",
                 "GER","GNO","GT","HAB","HBL","HEST","HIST","HOD","HON","HORT","HSY","HUM","HYP","I","ID","IDENTIF",
                 "IDEO","IGNOR","ILL","IMM","IMP","IMPERF","IMPR","IMPREC","IMPRS","IN","INCL","INAL","INAN","INCH",
                 "INCHO","INCEP","INCL","IND","INDF","NDEF","INE","INF","INFER","INFR","INEL","INS","INT","INTEN",
                 "INTR","NTR","INV","IO","IPFV","IRR","IS","ITER","JUS","L","L2","LAT","LD","LK","LOC","LOG","M",
                 "MASC","MAN","MID","MIM","MIR","MOD","MOM","MVT","N","NEUT","N","NEG","NH","NMZ","NZ","NOMI","NOM",
                 "NS","NTR","INTR","NUM","O","OBJ","OB","OBL","OBV","OPT","P","P","PART","PAS","PASS","PAT","PA","PAU",
                 "PEG","PER","PERF","PRF","PERS","PFV","PL","PLU","PLUR","PM","PN","PRO","PO","POL","POS","POSS",
                 "POST","POSTE","POT","PP","PPP","PPFV","PR","PRED","PREC","PREP","PRES","PRS","PRESP","PRET","PRT",
                 "PRF","PERF","PRS","PRES","PROB","PROG","PROH","PROL","PROP","PROSP","PRSP","PROT","PROX","PST","PT",
                 "PTCL","PTC","PTCP","PCP","PTV","PURP","Q","QUOT","R","REAL","REC","RECP","REF","REFL","REL","REM",
                 "REP","RES","RET","RFR","S","SBJ","SUB","SBJV","SJV","SE","SEM","SENS","SEQ","SG","SGV","SIM","SJV",
                 "SBJV","SPEC","SS","STAT","STV","SUB","SU","SUBR","SUBORD","SBRD","SR","SUBE","SUBL","SUC","SUP(L)",
                 "SUPE","T","TAM","TEL","TEMP","TERM","TNS","TOP","TR","TRANS","TRANSL","TRI","TRN","TVF","U","UH",
                 "UND","UR","USIT","V","VB","VBZ","VD","VEN","VER","VIA","VIS","VI","VN","VOC","VOL","VT","WH.Q","Z",
                 "ZO","A","ABL","ABS","ACC","ADJ","ADV","AGR","ALL","ANTIP","APPL","ART","AUX","BEN","CAUS","CLF",
                 "COM","COMP","COMPL","COND","COP","CVB","DAT","DECL","DEF","DEM","DET","DIST","DISTR","DU","DUR",
                 "ERG","EXCL","F","FOC","FUT","GEN","IMP","INCL","IND","INDF","INF","INS","INTR","IPFV","IRR",
                 "LOC","M","N","N-","NEG","NMLZ","NOM","OBJ","OBL","P","PASS","PFV","PL","POSS","PRED",
                 "PRF","PRS","PROG","PROH","PROX","PST","PTCP","PURP","Q","QUOT","RECP","REFL","REL","RES","S",
                 "SBJ","SBJV","SG","TOP","TR","VOC","ANIM","INAN","RAT","NON-RAT","ANIM-RAT","INAN-RAT","NON-ANIM-RAT",
                 "NON-FEM","NON-MASC","NON-NEUT"}

def string_to_final(string):
    if string == ONE2ONE_MATCH:
        return ONE2ONE_MATCH
    if string == ONE2ONE_SEARCH:
        return ONE2ONE_SEARCH
    if string == ONE2ONE_EXACT:
        return ONE2ONE_EXACT
    if string == ONE2ONE_ALIGN:
        return ONE2ONE_ALIGN
    if string == BUILD_WORDS:
        return BUILD_WORDS
    if string == KNOWN_MORPHS:
        return KNOWN_MORPHS
    if string == IGNORE:
        return IGNORE
    if string == ORIGINAL:
        return ORIGINAL
    if string == CLEANED:
        return CLEANED
    if string == ALL:
        return ALL
    if string == LANGUAGE:
        return LANGUAGE
    if string == GLOSS:
        return GLOSS
    if string == TRANSLATION:
        return TRANSLATION
    if string == LANGUAGE_MORPHEME:
        return LANGUAGE_MORPHEME
    if string == GLOSS_MORPHEME:
        return GLOSS_MORPHEME
    if string == TRANSLATION_MORPHEME:
        return TRANSLATION_MORPHEME
    if string == LANGUAGE_FEATURE:
        return LANGUAGE_FEATURE
    if string == GLOSS_FEATURE:
        return GLOSS_FEATURE
    if string == TRANSLATION_FEATURE:
        return TRANSLATION_FEATURE
    if string == LANGUAGE_WORD:
        return LANGUAGE_WORD
    if string == LANGUAGE_WORD_MORPHEME:
        return LANGUAGE_WORD_MORPHEME
    if string == LANGUAGE_WORD_FEATURE:
        return LANGUAGE_WORD_FEATURE
    if string == GLOSS_WORD:
        return GLOSS_WORD
    if string == GLOSS_WORD_MORPHEME:
        return GLOSS_WORD_MORPHEME
    if string == GLOSS_WORD_FEATURE:
        return GLOSS_WORD_FEATURE
    if string == TRANSLATION_WORD:
        return TRANSLATION_WORD
    if string == TRANSLATION_WORD_MORPHEME:
        return TRANSLATION_WORD_MORPHEME
    if string == TRANSLATION_WORD_FEATURE:
        return TRANSLATION_WORD_FEATURE
    return string
    

############
# classes
############
class IGT_library:
    '''
    @author David Wax
    @parent None

    This stores examples of IGT across iso codes and provides
    the highest level functionality for working with
    these large sets of IGT examples.
    '''
    def __init__(self, hyphens=False):
        '''
        @param None
        @return IGT_library

        This creates an empty IGT_library instance.
        '''
        #The IGT catalogue stores IGT_Books (value) by iso code (key)
        self.pos_tag = "verb"
        self.hyphens = hyphens
        self.IGT_catalogue = {}

    ### ACCESSORS ###
    def get_ids(self, iso=None, mix=True, seed=None):
        if seed is not None:
            random.seed(seed)
        ids = []
        if iso:
            ids = self.IGT_catalogue[iso].get_ids(False)
        else:
            for iso_key in self.IGT_catalogue:
                ids.extend(self.IGT_catalogue[iso_key].get_ids(False))
        if mix:
            ids = shuffle(ids)
        return ids

    def get_size(self):
        return len(self.IGT_catalogue)

    def group_size(self, iso, groups, percent=True):
        if percent:
            return float(self.IGT_catalogue[iso].group_size(groups)) / float(len(self.IGT_catalogue[iso]))
        else:
            return self.IGT_catalogue[iso].group_size(groups)

    def get_igt_size(self, iso=None):
        total_size = 0
        if iso:
            return self.IGT_catalogue[iso].get_size()
        else:
            for iso in self.IGT_catalogue:
                total_size += self.IGT_catalogue[iso].get_size()
            return total_size

    def read_files(self, path, file_ex=None):
        '''
        @param path (String) - The path of the dir or file to read data from
        @param file_ex (String) - The file extension of ODIN files

        This reads in all the files with the proper extension which are
        found recursively from the path provided. If no file_ex is provided
        then all files under the given directory will be read (and thereby
        should only contain ODIN files).
        '''
        if os.path.isdir(path):
            subdirs = os.listdir(path)
            for subdir in subdirs:
                self.read_files(os.path.join(path, subdir), file_ex)
        else:
            if file_ex is None:
                self.add_file(path)
            elif path.find(file_ex, -len(file_ex)) != -1:
                print("using file",path)
                self.add_file(path)

    # Load a xigtxml igt corpus, create mom igts from that.
    def add_file(self, file_name):
        file = open(file_name, encoding='utf-8')
        xc = xigtxml.load(file)
        file.close()
        count = 0
        try:
            (language, iso) = (xc.metadata[0].text[0].attributes['name'], xc.metadata[0].text[0].attributes['iso-639-3'])
        except:
            (language, iso) = ("unknown", "unk")
        if iso not in self.IGT_catalogue:
            self.IGT_catalogue[iso] = IGT_book(language, iso)
        for igt in xc:
            count = count + 1
            #if count == 249:
            #    print ("debug!")
            #print("processed igt #", count)
            igt_lines = []
            doc_id = igt.attributes['doc-id']
            igt_lines.append(doc_id)
            odin_tier = igt['n']
            for item in odin_tier:
                content = item.get_content()
                if item.attributes['tag'] == "T":
                    if item.get_content() is None:
                        content = "NO TRANSLATION"
                if not item.attributes['tag'] == "M":
                    igt_lines.append("line=" + item.attributes['line'] + " " + "tag=" + item.attributes['tag'] + ": " + content)
            mom_igt = IGT(igt_lines, xigt_igt=igt)
            #if not mom_igt.successful:
            #    continue
            if not mom_igt.aligned:
                continue
            self.IGT_catalogue[iso].add_IGT(mom_igt)

    def process(self, iso=None, process=True, clean=True, token=True, delete=True):
        '''
        This takes the raw original lines and creates the proper
        types for further work with the other modules.
        This works recursively: from IGT_catalogue to IGT_book, from there to IGT.
        '''
        if iso is not None:
            self.IGT_catalogue[iso].process(process, clean, token, delete)
        else:
            for iso_key in self.IGT_catalogue:
                self.IGT_catalogue[iso_key].process(process, clean, token, delete)

    '''
    Find items in the IGTs that have a certain POS tag (verb by default). 
    They will later be used to build the Lexicon for the Choices.
    '''
    def mark_pos(self, iso=None, tag="verb"):
        self.pos_tag = tag
        if iso is not None:
            igt_book = self.IGT_catalogue[iso]
            igt_book.mark_pos(tag) 
        else:
            for iso_key in self.IGT_catalogue:
                igt_book = self.IGT_catalogue[iso]
                igt_book.mark_pos(tag) 

    ''' Core function: Create choices file based on the POS (such as verb) list and the corresponding glosses'''
    def make_choices(self, iso, groups=None):
        (gloss_list, pos_list) = self.IGT_catalogue[iso].get_lists(GLOSS_WORD_MORPHEME, LANGUAGE_WORD_MORPHEME, tag=self.pos_tag, groups=groups)
        i = 0
        choices_file = choices.Choices(iso, gloss_list, pos_list, pos_tag=self.pos_tag, hyphens = self.hyphens)
        return choices_file

    ### LEGACY CODE, associated with serialization, alignment and ODIN support
    ### ###
    def pickle(self, iso=None, file=None, protocol=pickle.HIGHEST_PROTOCOL):
        '''
        @param file (File) - The name of the file where the data should be pickled
          default: None - This creates a named persistent temporary file
        @param iso (String) - The key value iso code designed for pickling only one iso
          default: None - This pickles the entire library for all iso codes
        @param protocol (Int) - The pickle protocol used.
          default: The highest pickle protocol
        @return String - The name of the file where the IGT data was pickled

        This pickles the IGT data into either a provided file or a temporary file.
        If an iso is provided only that data will be pickled, otherwise the
        entire library of IGT examples will be pickled.
        '''
        if file is None:
            file = tempfile.NamedTemporaryFile(dir=".",delete=False)
        if iso:
            pickle.dump(self.IGT_catalogue[iso], file, protocol)

        else:
            pickle.dump(self.IGT_catalogue, file, protocol)

        file_name = file.name
        file.close()
        return file_name

    def load(self, file_name, iso=None, delete=False):
        '''
        @param file_name (String) - The pickled file name.
        @param iso (String) - if provided load the file as an IGT_book into the value
        with the iso being the key.
        @param delete - Delete the file after loading it.
        
        This function works with the pickle library as a means of loading states of the
        IGT library instance.
        '''
        if iso:
            self.IGT_catalogue[iso] = pickle.load(open(file_name, "rb"))
        else:
            self.IGT_catalogue = pickle.load(open(file_name, "rb"))
        assert isinstance(self.IGT_catalogue, dict), "Loaded catalogue not a dictionary"
        if delete:
            os.system("rm -f " + file_name)

    def free_memory(self, iso_list, exclude=False):
        '''
        Removes references to entire sets of IGT based on the iso provided.
        iso_list can be either a list of strings or a single string
        exclude reverses which IGT_Books are kept
        throws key exception UNLESS exclude is True (It doesn't make sure
        that all the iso in the list are there to be kept.)
        '''
        if not isinstance(iso_list, list):
            iso_list = [iso_list]
        if exclude:
            for iso in self.IGT_catalogue.keys():
                if iso not in iso_list:
                    del self.IGT_catalogue[iso]
        else:
            for iso in iso_list:
                del self.IGT_catalogue[iso]

    def make_training_groups(self, iso=None, size=None, seed=None):
        if iso:
            self.IGT_catalogue[iso].make_training_groups(size, seed)
        else:
            for iso_key in self.IGT_catalogue:
                self.IGT_catalogue[iso_key].make_training_groups(size, seed)

    def set_groups(self, file_name, group_name, default=None, override=False):
        '''
        @param file_name (String) - The name of the file with the IGT Ids.
        @param group_name (String) - The name of the group all the IGT objects
        will be assigned.
        @param default (String) - Sets all other IGT objects to this group.
        @param override (bool) - If true the group_name overrides previous groups,
        but in either case the default only applies to empty ones.
        '''
        file = open(file_name, "r")
        ids = []
        for line in file:
            if not(line.startswith("#") or line.isspace()):
                line = line.strip()
                ids.append(line)
        for iso in self.IGT_catalogue:
            for igt in self.IGT_catalogue[iso].get_igt_list():
                if igt.get_id() in ids:
                    if igt.get_group() is None or override:
                        igt.set_group(group_name)
                elif igt.get_group() is None and default:
                    igt.set_group(default)


    #Legacy function
    #TODO?: Add tag info?
    def resplit_morphemes(self, igt_flag, iso=None, boost=None, boost_mult=None, groups=None):
        if iso is None:
            sys.stderr("igt.py Resplit morphemes warning... this makes no sense\n")
        else:
            self.IGT_catalogue[iso].resplit_morphemes(igt_flag, boost, boost_mult, groups)


    ### PRINT functions ###
    def print_alignments(self, iso=None, tag=None, order=[TRANSLATION, GLOSS, LANGUAGE], file=sys.stdout):
        if iso:
            self.IGT_catalogue[iso].print_alignments(tag, order, file)
        else:
            for iso_key in self.IGT_catalogue:
                self.IGT_catalogue[iso_key].print_alignments(tag, order, file)

    #TODO?: use flags to show which lines are going to print
    def print_words(self, iso=None, flag=None, tag=None, groups=None, file=sys.stdout):
        if iso:
            self.IGT_catalogue[iso].print_words(flag, tag, groups, file)
        else:
            for iso_key in self.IGT_catalogue:
                self.IGT_catalogue[iso_key].print_words(flag, tag, groups, file)
    def print_ids(self, iso=None, shuffle=True, file=sys.stdout):
        ids = self.get_ids(iso, shuffle)
        #print ids;
        for id in ids:
            file.write(id)
            file.write("\n")

    #TODO?: allow percentage ammounts
    #TODO?!: Use a flag list to say what should be printed and in what order
    #TODO?!: Also ORIGINAL PROCESSED and BOTH flags
    def print_igt(self, iso=None, ammount=None, use_full=True, seed=None, file=sys.stdout):
        ids = self.get_ids(iso, True, seed)
        if ammount is not None:
            ids = ids[0:ammount]
        if iso:
            self.IGT_catalogue[iso].print_igt(ids, file)
        else:
            for iso_key in self.IGT_catalogue:
                self.IGT_catalogue[iso_key].print_igt(ids, file)

    def print_stats(self, file=sys.stdout):
        file.write("\tIGT\tclean\tclean-1\tclean-2\tn-clean\tcomp\tcomp-1\tcomp-2\tempty\n")
        IGT_books = sorted(self.IGT_catalogue.values(), key=lambda x: x.get_size(), reverse=True)
        for igt_book in IGT_books:
            igt_book.print_stats(file)

    def print_choices(self, iso, choices_file=None, language=None,
                      groups=None, file=sys.stdout):
        if file is None:
            file = tempfile.NamedTemporaryFile(dir=".", mode="w", encoding='utf-8', delete=False)
        if choices_file is None:
            (gloss_list, verb_list) = self.IGT_catalogue[iso].get_lists(GLOSS_WORD_MORPHEME, LANGUAGE_WORD_MORPHEME, tag="verb", groups=groups)
            choices_file = choices.Choices(iso, gloss_list, verb_list)
        choices_file.print_to(file, language)
        return file.name

    #TODO: allow for no iso
    def score(self, iso, flag, gold, groups=None, file=sys.stdout):
        return self.IGT_catalogue[iso].score(flag, gold, groups, file)

class IGT_book:
    #TODO!:have groups store the actual lists so that instead of checking
    #through all the IGT each time the lists can just be extended.
    def __init__(self, language, iso):
        self.IGT_list = []
        self.language = language
        self.iso = iso
        #self.groups = [];

    def __len__(self):
        return len(self.IGT_list)

    def add_IGT(self, igt):
        self.IGT_list.append(igt)

    def get_size(self):
        return len(self.IGT_list)

    def get_clean_count(self):
        ideal_cnt = [0, 0, 0, 0]
        for igt in self.IGT_list:
            ideal_cnt[igt.ammount_clean()] += 1
        return ideal_cnt

    def get_complete_count(self):
        comp_cnt = [0, 0, 0, 0]
        for igt in self.IGT_list:
            comp_cnt[igt.ammount_complete()] += 1
        return comp_cnt

    def get_count(self, flag_list):
        count = 0
        for igt in self.IGT_list:
            good = True
            for flag in flag_list:
                if igt.get_IGT(flag).is_bad():
                    good = False
            if good:
                count += 1
        return count

    def process(self, process=True, clean=True, token=True, delete=False):
        for igt in self.IGT_list:
            igt.process(process, clean, token, delete)

 
    def mark_pos(self, tag):
        #count = 1;
        for igt in self.IGT_list:
            #print ("marking pos for igt #",count)
            igt.mark_pos(tag)
            #count = count + 1

    '''
    This is used by Choices to create the lexicon (to get aligned gloss list and POS list)
    '''
    def get_lists(self, from_flag, to_flag, use_bad=False, tag=None, groups=None):
        from_list = []
        to_list = []
        for igt in self.IGT_list:
            (from_items, to_items) = igt.get_alignable_groups(from_flag, to_flag, use_bad, tag)
            from_list.extend(from_items)
            to_list.extend(to_items)
        return (from_list, to_list)

    ### PRINT functions ###
    def print_stats(self, file=sys.stdout):
        (clean_0, clean_1, clean_2, clean_3) = self.get_clean_count()
        (compl_0, compl_1, compl_2, compl_3) = self.get_complete_count()
        lang = self.get_count([LANGUAGE])
        gloss = self.get_count([GLOSS])
        tran = self.get_count([TRANSLATION])
        lang_gloss = self.get_count([LANGUAGE, GLOSS])
        gloss_tran = self.get_count([GLOSS, TRANSLATION])
        lang_tran = self.get_count([LANGUAGE, TRANSLATION])
        all = self.get_count([LANGUAGE, GLOSS, TRANSLATION])
        file.write(self.iso + "\t" + str(self.get_size()) + "\t" + str(clean_3) + "\t" + str(clean_2) + "\t" + str(clean_1) + "\t" + str(clean_0) + "\t" + str(compl_3) + "\t" + str(compl_2) + "\t" + str(compl_1) + "\t" + str(compl_0) + "\t" + str(lang) + "\t" + str(gloss) + "\t" + str(tran) + "\t" + str(lang_gloss) + "\t" + str(gloss_tran) + "\t" + str(lang_tran) + "\t" + str(all) + "\n")

    def print_customize(self, file=sys.stdout):
        for igt in self.IGT_list:
            igt.print_customize(file)
            file.write("\n")

    def print_words(self, flag=None, tag=None, groups=None, file=sys.stdout):
        for igt in self.IGT_list:
            igt.print_words(flag, tag, file)

    def print_alignments(self, tag=None, order=[TRANSLATION, GLOSS, LANGUAGE], file=sys.stdout):
        for igt in self.IGT_list:
            igt.print_alignments(tag, order, file)
            file.write("\n")

    def print_igt(self, ids=None, file=sys.stdout):
        #TODO this doesn't print in the right order...
        for igt in self.IGT_list:
            id = igt.get_id()
            if ids is None or id in ids:
                file.write(id + ":\n")
                igt.print_igt(ORIGINAL, file)
                file.write("\n")
                igt.print_igt(CLEANED, file)
                file.write("\n")

    ### ACCESSORS ###
    #TODO?: make id's a dict key for looking stuff up quickly
    #TODO?: Find out if this is being used and update to use flags
    def get_translation(self, id):
        for igt in self.IGT_list:
            if igt.get_translation().get_id() == id:
                return igt.get_translation()
        raise Exception("translation for id:" + id + " not found.\n")


    def get_gloss(self, id):
        for igt in self.IGT_list:
            if igt.get_translation().get_id() == id:
                return igt.get_gloss()
        raise Exception("translation for id:" + id + " not found in get_gloss().\n")

    #TODO: only get "super clean" igt
    def get_ids(self, use_full=True):
        ids = []
        for igt in self.IGT_list:
            if (igt.ammount_complete() == 3 or not use_full):
                ids.append(igt.get_id())
        return ids

    def get_igt_list(self):
        return self.IGT_list

class IGT:
    def __init__(self, lines, xigt_igt):
        self.group = None
        self.aligned = False
        self.lines = []
        self.header = {}
        self.language = Language_Line()
        self.gloss = Gloss_Line()
        self.translation = Translation_Line()
        self.roots = []
        self.successful = True
        if xigt_igt is None:
            raise Exception("xigt_igt passed to IGT constructor is None")
        self.xigt_igt = xigt_igt
        doc_id_line = lines.pop(0)
        self.id = doc_id_line
        for line in lines:
            normline = re.sub("[\s\t]+", " ", line.strip())
            if re.match("line=\d+", normline) != None:
                #if "REDUP:flatten" in line:
                #    print("stop")
                igt_line = IGT_line(normline, xigt_igt = xigt_igt)
                if not igt_line.successful:
                    self.successful = False
                self.lines.append(igt_line)
            else:
                self.add_header_info(line)
        # Gloss to Language word to word and morpheme to morpheme alignment
        if self.successful:
            self.aligned = self.xigt_align(xigt_igt, self.id)

    ### ACCESSORS ###
    def get_id(self):
        return self.id

    #TODO?: find a way to remove the token flag?
    def get_IGT(self, flag, use_bad=True):
        if flag in LANGUAGE_TYPES:
            return self.get_language(use_bad)
        elif flag in GLOSS_TYPES:
            return self.get_gloss(use_bad)
        elif flag in TRANSLATION_TYPES:
            return self.get_translation(use_bad)
        else:
            raise Exception("illegal flag:" + flag)

    def get_language(self, use_bad=True, token=False):
        return self.language

    def get_gloss(self, use_bad=True, token=False):
        return self.gloss
    

    def get_translation(self, use_bad=True, token=False):
        return self.translation

    ### XIGT CODE ###
    '''
    This relies on the format as in kor-ex.xml file
    '''
    def xigt_align(self, xigt_igt, doc_id):
        for line in self.lines:
                if line.o_tag == "L":
                    lang_line = line
                elif line.o_tag == "G":
                    gloss_line = line
                elif line.o_tag == "T":
                    transl_line = line
        if xigt_igt is None:
            return
        i = 0
        #trivial alignment to own line
        for line in self.lines:
            for t in line.tokens:
                ''' word-level alignment to own line'''
                j = 0
                '''morpheme-level alignment to own line'''
                for tt in t.tokens:
                    t.align_list.append(str(line.o_tag) + " " + str(i) + " " + str(j))
                    tt.align_list.append(str(line.o_tag) + " " + str(i) + " " + str(j))
                    j = j + 1
                i = i + 1
            i = 0
        ''' G to L alignment '''
        # if not INTENT-enriched and have morphemes and glosses ids in form "m1.1"
        if xigt_igt['g'].attributes['content'] == 'n':
            token_idx = 0
            #for each word in language line
            for t in lang_line.tokens:
                gloss_morpheme_idx = 0
                #for each gloss in xigt
                for gloss_item in xigt_igt['g']:
                    #find which morpheme this gloss is aligned to
                    morph_id = gloss_item.attributes['alignment']
                    word_seg = xigt_igt['m'][morph_id].attributes['segmentation']
                    word_matches = re.search("(\w(\d+))\[?", word_seg)
                    #find which word this morpheme is aligned to; this will also be
                    #the right word to align the gloss to.
                    word_idx = int(word_matches.group(2)) - 1 # zero-based!
                    if token_idx == word_idx:
                        if token_idx >= len(gloss_line.tokens) or gloss_morpheme_idx >= len(gloss_line.tokens[token_idx].tokens):
                            print("bug!")
                        t.align_list.append("G " + str(token_idx) + " " + str(gloss_morpheme_idx))
                        gloss_line.tokens[token_idx].align_list.append("L " + str(token_idx) + " " + str(gloss_morpheme_idx))
                        t.tokens[gloss_morpheme_idx].align_list.append("G " + str(token_idx) + " " + str(gloss_morpheme_idx))
                        gloss_line.tokens[token_idx].tokens[gloss_morpheme_idx].align_list.append("L " + str(token_idx) + " " + str(gloss_morpheme_idx))
                        gloss_morpheme_idx = gloss_morpheme_idx + 1
                token_idx = token_idx + 1
        else: #enriched by INTENT
            current_word = 0
            morphemes_in_cur_word = -1
            prev_morpheme = -1
            is_gram = False
            for gloss_item in xigt_igt['g']:
                aligned_morpheme = gloss_item.attributes['alignment']
                aligned_word = gloss_item.attributes['content']
                token_idx = re.search("(\d+)", aligned_word).group(1)
                token_idx = int(token_idx) - 1
                gloss_morpheme_idx = re.search("(\d+)", aligned_morpheme).group(1)
                gloss_morpheme_idx = int(gloss_morpheme_idx) - 1
                if gloss_morpheme_idx == prev_morpheme:
                    is_gram = True
                else:
                    is_gram = False
                    morphemes_in_cur_word += 1
                    prev_morpheme +=1
                    # means this is a gram; another gloss corresponding to the same morpheme
                if not token_idx == current_word:
                    morphemes_in_cur_word = 0
                    current_word = token_idx
                if token_idx >= len(lang_line.tokens):
                    print("bug")
                t = lang_line.tokens[token_idx]
                if not is_gram:
                    t.align_list.append("G " + str(token_idx) + " " + str(morphemes_in_cur_word))
                    try:
                        t.tokens[morphemes_in_cur_word].align_list.append("G " + str(token_idx) + " " + str(morphemes_in_cur_word))
                        gloss_line.tokens[token_idx].align_list.append("L " + str(token_idx) + " " + str(morphemes_in_cur_word))
                        gloss_line.tokens[token_idx].tokens[morphemes_in_cur_word].align_list.append("L " + str(token_idx)
                                                                                                 + " " + str(morphemes_in_cur_word))
                    except:
                        print("Could not align gloss line to language line in IGT {0}; skipping IGT".format(xigt_igt.id))
                        print("L: {0}".format(lang_line.line))
                        print("G: {0}".format(gloss_line.line))
                        self.successful = False
                        return False
        return True
                #else:
                #    for gram in gloss_line.tokens[token_idx].tokens[morphemes_in_cur_word].tokens:
                #        gram.align_list.append("L " + str(token_idx) + " " + str(morphemes_in_cur_word))


    def mark_pos(self, tag):
        tag_split = tag.split('/')
        pos_tier = self.xigt_igt.get('w-pos')
        if pos_tier == []:
            raise Exception("no w-pos tier in the IGT!")
        word_idx = 0
        #Language line:
        for word in self.language.tokens:
            for item in pos_tier:
                lang_align_str = item.attributes['alignment']
                lang_align_match = re.search("\w(\d+)", lang_align_str)
                str_wd_id = re.search("(\w\d+)", lang_align_str).group(1)
                word_id = int(lang_align_match.group(1)) - 1
                #if word_id >= len(self.xigt_igt['w']):
                #    print("bug!")
                lang_wd = self.xigt_igt['w'][word_id].get_content() # TODO: should get this through alignment instead
                if word_idx == word_id:
                    pos = item.get_content()
                    if pos in tag_split:
                        word.add_tag(tag) #TODO: cut the loop here?
                        #assume perfect alignment on token level with gloss
                                                                 #line:
                        self.gloss.tokens[word_idx].add_tag(tag)
                        (roots, root_indices) = find_roots(self.xigt_igt, str_wd_id)
                        clitic_indices = self.find_clitics(word, str_wd_id)
                        if root_indices is None or root_indices == []:
                            #print("No roots have been found via type attribute on morphemes tier, and no bilingual alignment tier in the IGT! No way to find roots in word:",lang_wd)
                            continue
                        # TODO disallow gram-level tokenization by dot for root
                        # grams
                        #word.tokenize(verb_roots=roots)
                        # Morpheme-level alignment:
                        t_idx = 0
                        for t in word.tokens:
                            if not clitic_indices == []:
                                if t_idx in clitic_indices:
                                    self.language.tokens[word_idx].tokens[t_idx].add_tag("clitic")
                            if t_idx in root_indices:
                                t.add_tag(tag)
                                self.gloss.tokens[word_idx].tokens[t_idx].add_tag(tag)
                                # Gram level alignment:
                                tt_idx = 0
                                for tt in t.tokens:
                                    if str(tt) in roots:
                                        tt.add_tag(tag)
                                        self.gloss.tokens[word_idx].tokens[t_idx].tokens[tt_idx].add_tag(tag)
                                    tt_idx = tt_idx + 1
                            t_idx = t_idx + 1
            word_idx = word_idx + 1

    def find_clitics(self,word, word_id):
        morph_tier = self.xigt_igt['m']
        clitic_indices = []
        for m_item in morph_tier:
            if word_id in m_item.attributes['segmentation']:
                morphType = m_item.type
                if (not morphType is None) and (morphType == "Enclitic" or morphType == "Proclitic" or morphType == "Clitic"):
                    match = re.search("m\d+\.(\d+)", m_item.id)
                    str_id = match.group(1)
                    id = int(str_id) - 1
                    clitic_indices.append(id)
        return clitic_indices

    ### CORE FUNCTIONS ###
    def process(self, process=True, clean=True, token=True, delete=True):
        if process:
            self.process_IGT()
        #if clean:
        #    self.clean_IGT()
        if delete:
            del self.lines
            del self.header

    def clean_IGT(self):
        self.language.clean(self)
        self.gloss.clean(self)
        self.translation.clean(self)

    def process_IGT(self):
        self.language = Language_Line(parent_id=self.id)
        self.gloss = Gloss_Line(parent_id=self.id)
        self.verb_gloss = Gloss_Line(parent_id=self.id)
        self.translation = Translation_Line(parent_id=self.id)
        previous_line = IGT_line("line=X tag=X:")
        for line in self.lines:
            if line.is_only_language():
                self.language.add_line(line)
                self.language.tokens = line.tokens
                previous_line = line
            elif line.is_only_gloss():
                self.gloss.add_line(line)
                self.gloss.tokens = line.tokens
                previous_line = line
            elif line.is_only_translation():
                self.translation.add_line(line)
                self.translation.tokens = line.tokens
                previous_line = line
            elif line.is_continuation():
                if previous_line.is_only_language():
                    self.language.add_continuation(line)
                elif previous_line.is_only_gloss():
                    self.gloss.add_continuation(line)
                elif previous_line.is_only_translation():
                    self.translation.add_continuation(line)
            #TODO?  collect mixed lines
            else: #catches M and B and mixed lines atm
                previous_line = line

    def tokenize_IGT(self):
        self.language.tokenize()
        self.gloss.tokenize(self.verb_roots)
        self.translation.tokenize()

    def add_header_info(self, str):
        line = str.strip()
        print("add header line:",line)
        match = re.match("\A(\w+):\s?(.*)\Z", line)
        if match != None:
            print("add header match group 1:",match.group(1))
            print("match group 2:",match.group(2))
            self.header[match.group(1)] = match.group(2)
        else:
            sys.stderr.write("IGT (" + self.id + ") line:\"" + line + "\" unexpected format.\n")

    def get_alignable_groups(self, from_flag, to_flag, use_bad=False, tag=None):
        '''
        @param from_flag (FINAL String) - The type of alignable group to be the source
        @param to_flag (FINAL String) - The type of alignable group to be the target.
        @param use_bad (Boolean) - @TODO?COMMENT
        @param tag (String) - Limits to those alignables which match the tag

        This creates two lists of alignable_groups.
        
        @return from_list (List of Alignables_groups)
        @return to_list (List of Alignables_groups)
        '''
        assert (from_flag in LINE_LEVEL and to_flag in LINE_LEVEL) or (from_flag in WORD_LEVEL and to_flag in WORD_LEVEL), "TO and FROM flags on different levels"
        from_list = []
        to_list = []
        from_line = self.get_IGT(from_flag, use_bad)
        to_line = self.get_IGT(to_flag, use_bad)
        if from_line and to_line:
            if from_flag in LINE_LEVEL and to_flag in LINE_LEVEL:
                assert tag is None, "Tag is given for IGT_lines which don't have tags.\n"
                from_list.append(from_line.get_alignable_group(from_flag))
                to_list.append(to_line.get_alignable_group(to_flag))
            elif from_flag in WORD_LEVEL and to_flag in WORD_LEVEL:
                lista = from_line.get_alignable_groups(from_flag, tag)
                listb = to_line.get_alignable_groups(to_flag, tag)
                for from_item in lista:
                    if from_item is not None:
                        for to_item in listb:
                            if to_item is not None:
                                if from_item.is_aligned(to_item):
                                    from_list.append(from_item)
                                    to_list.append(to_item)
                                else:
                                    for from_token in from_item.get_tokens():
                                        for to_token in to_item.get_tokens():
                                            if (from_token.is_aligned(to_token)):
                                                from_list.append(from_item)
                                                to_list.append(to_item)
            else:
                raise Exception("Sanity Error")
        return (from_list, to_list)

    def get_alignables(self, flag, tag=None, use_bad=False):
        '''
        @param flag - String - The level of alignables to return.
        @param tag - String - Only return values which match the tag.
        @param use_bad - boolean - return from IGT which is considered problematic.

        Return an array of alignables.
        '''
        if flag in LANGUAGE_TYPES:
            return self.language.get_alignables(flag, tag, use_bad)
        if flag in GLOSS_TYPES:
            return self.gloss.get_alignables(flag, tag, use_bad)
        
        if flag in TRANSLATION_TYPES:
            return self.translation.get_alignables(flag, tag, use_bad)
        else:
            raise Exception("illegal flag:" + flag)

    ### PRINT functions ###
    def print_words(self, flag=None, tag=None, file=sys.stdout):
        file.write(self.id + ":\n")
        if flag in TRANSLATION_TYPES or flag is None:
            self.get_translation().print_words(tag, file)
        if flag in GLOSS_TYPES or flag is None:
            self.get_gloss().print_words(tag, file)
        if flag in LANGUAGE_TYPES or flag is None:
            self.get_language().print_words(tag, file)

    def print_igt(self, flag=ORIGINAL, file=sys.stdout):
        if flag == ORIGINAL:
            for line in self.lines:
                file.write(str(line))
                file.write("\n")
        elif flag == CLEANED:
            #TODO?:create print_IGT function
            file.write(str(self.get_IGT(LANGUAGE)))
            file.write("\n")
            file.write(str(self.get_IGT(GLOSS)))
            file.write("\n")
            file.write(str(self.get_IGT(TRANSLATION)))
            file.write("\n")
            file.write("\n")
        elif flag == ALL:
            for line in self.lines:
                file.write(str(line))
                file.write("\n")
            file.write(str(self.get_IGT(LANGUAGE)))
            file.write("\n")
            file.write(str(self.get_IGT(GLOSS)))
            file.write("\n")
            file.write(str(self.get_IGT(TRANSLATION)))
            file.write("\n")
            file.write("\n")            

    def print_alignments(self, tag=None, order=[TRANSLATION, GLOSS, LANGUAGE], file=sys.stdout):
        file.write(self.id + ":\n")
        flag = order[0]
        words = self.get_IGT(flag).get_tokens()
        for word in words:
            if tag is None or word.has_tag(tag):
                file.write(str(word))
                self.print_aligned_words(word, tag, order[1:], file)
                file.write("\n")

    def print_aligned_words(self, previous_word, tag=None, order=[], file=sys.stdout, depth=1):
        if order:
            flag = order[0]
            words = self.get_IGT(flag).get_tokens()
            for word in words:
                if (tag is None or word.has_tag(tag)) and word.is_aligned(previous_word):
                    file.write(" ")
                    for i in range(depth):
                        file.write("-")
                    file.write("> ")
                    file.write(str(word))
                    self.print_aligned_words(word, tag, order[1:], file, depth + 1)

class IGT_line(alignables.alignables_group):
    def __init__(self, string, xigt_igt=None, doc_id=None):
        line = string.strip()
        match = re.match("line=(\S+) tag=([^:]+):(.*)\Z", line)
        if match != None:
            self.successful = True
            self.num = match.group(1).strip()
            self.o_tag = match.group(2).strip()
            self.process_tag(self.o_tag)
            self.line = match.group(3).strip()            
            if self.o_tag == "G":
                try:
                    self.line = self.normalize_gloss_root_grams(self.line, xigt_igt)
                except:
                    self.successful = False
                    print ("Can't normalize root grams in igt {0} in line {1}".format(xigt_igt.id, self.line))
            alignables.alignables_group.__init__(self, str(self.line))
            if self.successful:
                self.tokenize(xigt_igt)
                self.o_line = match.group(0)
        else:
            sys.stderr.write("igt line (\"" + line + "\") incorrect format.\n")

    def normalize_gloss_root_grams(self,line, xigt_igt):
        norm_line = ""
        start = 0
        root_types = ["Root", "Bound_Root", "Stem", "Bound_Stem"]
        morpheme_ids = []
        root_ids = []
        morph_tier = xigt_igt['m']
        gloss_tier = xigt_igt['g']
        for m_item in morph_tier:
            morphType = m_item.type
            morpheme_ids.append(m_item.id)  
            if (not morphType is None) and (morphType in root_types):
                root_ids.append(m_item.id)
        for g in gloss_tier:
            gloss = g.get_content()
            if g.attributes['alignment'] in root_ids:
                norm_line = norm_line + re.sub("\.","~",gloss)
            else:
                norm_line = norm_line + gloss
            if start + len(gloss) < len(line):
                end = start + len(gloss)
                sep = self.line[end]
                norm_line = norm_line + sep
                start = end + 1
        return norm_line.strip()



    def __str__(self):
        return self.line

    def process_tag(self, tag_info):
        tags = re.split("\+", tag_info)
        self.main_tags = re.split("-", tags[0])
        self.extra_tags = tags[1:]
        self.pos_tags = []

    def tokenize(self, xigt_igt):
        self.tokens = []
        if self.line != "":
            words = re.split("\s+", self.line.strip())
            try:
                for word_num in range(len(words)):
                    self.tokens.append(IGT_word(words[word_num], xigt_igt, self, word_num))
            except:
                self.successful = False
                print("Failed to tokenize IGT {0}; Line: {1} ".format(xigt_igt.id, self.line.strip()))

    def has_tag(self, tag): 
        return tag in self.main_tags or tag in self.extra_tags

    def get_tags(self):
        return self.main_tags + self.extra_tags

    ### LEGACY code ###
    def add_tags(self, tags, raise_tags=None):
        for tag in tags:
            self.add_tag(tag)

    def add_tag(self, tag):
        if len(tag) == 2:
            self.extra_tags.append(tag)
        elif len(tag) == 1:
            self.main_tags.append(tag)
        else:
            self.pos_tags.append(tag)

    def is_only_language(self):
        if "L" in self.main_tags and len(self.main_tags) == 1:
            return True
        else:
            return False
   
    def is_only_gloss(self):
        if "G" in self.main_tags and len(self.main_tags) == 1:
            return True
        else:
            return False

    def is_only_translation(self):
        if "T" in self.main_tags and len(self.main_tags) == 1:
            return True
        else:
            return False

    def get_line(self):
        return self.line

    def get_alignable_group(self, flag):
        '''
        @param flag (FINAL String) - The flag showing which type of group is requested 

        This returns the appropriate alignable group based on the flag
        and checks that it matches the type of the self.
        '''
        assert flag in self.get_type(), "Flag is non matching type.\n"
        assert flag in LINE_LEVEL, "Word level flag given for line level funcation.\n"
        if flag in WORD_TYPES:
            return self
        elif flag in MORPHEME_TYPES:
            new_group = alignables.alignables_group()
            for token in self.get_tokens():
                new_group.tokens.extend(token.get_tokens())
            return new_group
        elif flag in FEATURE_TYPES:
            new_group = alignables.alignables_group()
            #TODO: Use get_alignables to be more robust
            for token in self.get_tokens():
                for t in token.get_tokens():
                    new_group.tokens.extend(t.get_tokens())
            return new_group
        else:
            raise Exception("Flag unrecognized.\n")

    def get_alignable_groups(self, flag, tag=None):
        assert flag in self.get_type(), "Flag is non matching type.\n"
        assert flag in WORD_LEVEL, "Line level flag given for word level function.\n"
        list = []
        tokens = []
        if tag is None:
            tokens = self.get_tokens()
        else:
            for token in self.get_tokens():
                    if token.has_tag(tag):
                        tokens.append(token)
        for token in tokens:
            list.append(token.get_alignable_group(flag))
        return list

    def get_alignables(self, flag, tag=None, use_bad=False):
        alignables = []
        if use_bad or not self.is_bad():
            if flag in WORD_TYPES:
                for word in self.tokens:
                    if tag is None:
                        alignables.append(word)
                    else:
                         if word.has_tag(tag):
                              alignables.append(word)
            elif flag in MORPHEME_TYPES or flag in FEATURE_TYPES:
                for word in self.tokens:
                    alignables.extend(word.get_alignables(flag, tag))
            else:
                raise Exception("illegal flag:" + flag)
        return alignables

    def add_line(self, igt_line, p=False):
        if p:
            print(self.line + "+" + igt_line.get_line())
        self.line += igt_line.get_line()
        tags = igt_line.get_tags()
        for tag in tags:
            if not self.has_tag(tag):
                self.add_tag(tag)

    def print_line(self, file=sys.stdout):
        file.write(self.line)

    def print_words(self, tag=None, file=sys.stdout):
        for word in self.get_tokens():
            if word.has_tag(tag):
                file.write(str(word) + "\n")

class Language_Line(IGT_line):
    def __init__(self, line="", parent_id=None):
        if parent_id:
            self.id = parent_id + " L"
        else:
            self.id = None
        self.line = line
        self.main_tags = []
        self.extra_tags = []
        self.pos_tags = []

    def get_type(self):
        return LANGUAGE_TYPES

class Gloss_Line(IGT_line):
    def __init__(self, line="", parent_id=None):
        if parent_id:
            self.id = parent_id + " G"
        else:
            self.id = None
        self.line = line
        self.main_tags = []
        self.extra_tags = []
        self.pos_tags = []

    def get_type(self):
        return GLOSS_TYPES

class Translation_Line(IGT_line):
    def __init__(self, line="", parent_id=None):
        if parent_id is not None:
            self.id = parent_id + " T"
        else:
            self.id = None
        self.line = line
        self.main_tags = []
        self.extra_tags = []
        self.pos_tags = []

    def get_type(self):
        return TRANSLATION_TYPES

class IGT_element(alignables.alignable):
    def is_subset(self, item, match_flag=ONE2ONE_EXACT):
        if match_flag is ONE2ONE_SEARCH:
            if re.search(re.escape(str(item)), str(self), re.I):
                return True
            else:
                return False
        elif match_flag is ONE2ONE_MATCH:
            if re.match(re.escape(str(item)), str(self), re.I):
                return True
            else:
                return False
        elif match_flag is ONE2ONE_EXACT:
            if str(self) == str(item):
                return True
            else:
                return False
        else:
            raise Exception("illegal flag:" + match_flag)

    # TODO: modify and call this instead of appending align lists directly in
    # Xigt code.
    def add_alignment(self, aligned_to, raise_tags=True):
        '''
        @param aligned_to (giza.Alignable) - @TODO-COMMENT
        @TODO!:See if I can call the giza.alignable version...
        '''
        self.align_list.append(aligned_to.get_id())
        aligned_to.add_tags(self.get_tags(), raise_tags=True)

    def get_parent(self):
        if self.parent:
            return self.parent
        else:
            return None

    def get_tags(self):
        return self.tags

    def add_tags(self, tags, raise_tags=True):
        for tag in tags:
            self.add_tag(tag)
        if self.get_parent():
            self.get_parent().add_tags(tags, raise_tags)

    def add_tag(self, t):
        if t not in self.tags:
            self.tags.append(t)

    def set_tag(self, t):
        self.add_tag(t)

    '''
    If there is no tag given, and my own tags list is indeed empty, then this is the right 
    state of affairs, so return True (this is still confusing, so consider just NOT CALLING).
    Else, see if the tag is in my list of tags.
    '''
    def has_tag(self, t):
        return (not(t is None or self.tags == [])) and (t in self.tags)

class IGT_word(IGT_element, alignables.alignables_group):
    def __init__(self, string, xigt_igt, parent=None, wn=None):
        if re.search("\s+",string):
            raise IGTException("White space found in alignable word\n")
        if string == "":
            raise IGTException("Empty string given for IGT_word (" + parent.get_id() + ":" + str(wn) + ")\n")
        alignables.alignables_group.__init__(self, string)
        self.parent = parent
        if self.parent.get_id() is not None and wn is not None:
            self.id = self.parent.get_id() + " " + str(wn)
        else:
            self.id = wn
        self.word_num = wn
        self.value = string
        self.tags = []
        self.align_list = []
        word_idx = wn + 1
        if not parent.o_tag == "T":
            (roots, root_ids) = find_roots(xigt_igt,"w"+str(word_idx))
            self.tokenize(root_ids)

    def tokenize(self, root_ids=[], string=None):
        self.tokens = []
        if string is None:
            #TODO use a strip call to get rid of edge morpheme dividers
            value = re.sub("\A[-=:]|[-=:]\Z", "", self.value)
            morphs = re.split(re.compile("[\-\=\<\>]+"), value)
        else:
            morphs = re.split(re.compile("[-=]+"), string.strip())
        for i in range(len(morphs)):
            is_root = i in root_ids
            self.tokens.append(IGT_morpheme(morphs[i], is_root, self, i))

    ### CORE FUNCTIONS ###
    '''
    This assumes that the root is the morpheme with a tag.
    Consider just setting a field in Xigt code.
    '''
    def get_root(self, tag):
        for token in self.tokens:
            if token.is_root:
                return token
        return None

        #to_return = none
        #for token in self.tokens:
        #    if token.has_tag(tag):
        #        if to_return:
        #            return none
        #        to_return = token
        #return to_return

    def get_left_token(self, morph):
        return self.get_relative_token(morph, -1)

    def get_right_token(self, morph):
        return self.get_relative_token(morph, 1)

    def get_relative_token(self, morph, distance):
        index = self.tokens.index(morph)
        if index + distance >= 0 and index + distance < len(self.tokens):
            return self.tokens[index + distance]
        else:
            return None

    def get_alignables(self, flag, tag=None):
        alignables = []
        if flag in MORPHEME_TYPES:
            for morpheme in self.tokens:
                if tag is None:
                    alignables.append(morpheme)
                else:
                     if morpheme.has_tag(tag):
                         alignables.append(morpheme)
        elif flag in FEATURE_TYPES:
            for morpheme in self.tokens:
                alignables.extend(morpheme.get_alignables(tag))
        else:
            raise Exception("illegal flag:" + flag)
        return alignables

    def get_alignable_group(self, flag):
        assert flag in WORD_LEVEL, "Line level flag given for word level function.\n"
        if flag in WORD_TYPES:
            new_group = alignables.alignables_group()
            new_group.tokens.append(self)
            return new_group
        elif flag in MORPHEME_TYPES:
            return self
        elif flag in FEATURE_TYPES:
            new_group = alignables.alignables_group()
            for token in self.get_tokens():
                new_group.tokens.extend(token.get_tokens())
            return new_group
        else:
            raise Exception("Flag " + str(flag) + " unrecognized.\n") 

    def get_word_parent(self):
        return self

class IGT_morpheme(IGT_element, alignables.alignables_group):
    def __init__(self, string, is_root=False, parent=None, mn=None):
        '''
        @param string (String) - The string form of the morpheme
        @param parent_id (String) - The id of the parent IGT_word
        @param mn (Int or String) - The Morpheme number
        '''
        if re.search("\s+",string):
            raise IGTException("White space found in alignable word\n")
        if string == "":
            raise IGTException("Empty string given for IGT_word(" + str(parent.get_id()) + ":" + str(mn) + ")\n")
        alignables.alignables_group.__init__(self, string)
        self.parent = parent
        if self.parent is not None and mn is not None:
            self.id = str(self.parent.get_id()) + " " + str(mn)
        else:
            self.id = mn
        self.morph_num = mn
        self.value = string
        self.tags = []
        self.align_list = []
        self.is_root = is_root
        self.tokenize()

    def tokenize(self, string=None):
        self.tokens = []
        if not self.is_root:
            if string is None:
                string = re.sub("\A[\.\:]+|[\.\:]+\Z", "", self.value)
                features = re.split(re.compile("[\.\:]+"), string)
            for i in range(len(features)):
                self.tokens.append(IGT_feature(features[i], self, i))
        else:
            self.tokens.append(IGT_feature(self.value))

    def get_alignables(self, tag):
        alignables = []
        for feature in self.tokens:
            if tag is None: 
                alignables.append(feature)
            else:
                if freature.has_tag(tag):
                    alignables.append(feature)
        return alignables

    def get_word_parent(self):
        return self.parent.get_word_parent()

class IGT_feature(IGT_element):
    def __init__(self, string, parent=None, fn=None):
        if re.search("\s+",string):
            raise IGTException("White space found in alignable feature\n")
        if string == "":
            raise IGTException("Empty string given for IGT_element (" + parent.get_id() + ":" + str(fn) + ")\n")
        self.parent = parent
        if self.parent is not None and fn is not None:
            self.id = self.parent.get_id() + " " + str(fn)
        else:
            self.id = fn
        self.feature_num = fn
        self.value = string
        self.tags = []
        self.align_list = []

    def get_word_parent(self):
        return self.parent.get_word_parent()

        '''
    Attempt to identify which morpheme is the root.
    If there is no POS tier, then
    this relies on xigt tw -> gloss morpheme alignment,
    and also assumes that the gloss root segment will be exactly
    the same as the corresponding translation word 
    (which will not always be the case:
    talk -> talked, talk -> say, and so on.
    '''
def find_roots(xigt_igt, word_id):
    roots = []
    morph_tier = xigt_igt.get('m')
    if morph_tier is None:
        print ("No morphology tier found for igt {0}; no roots will be found".format(xigt_igt.id))
        return ([],[])
    morphemes = []
    # the indices of items in morpheme ids correspond to the indices of tokens in MOM IGT_Word object
    morpheme_ids = []
    root_morpheme_ids = []
    gloss_ids = []
    glosses = []
    root_gloss_tokens = []
    root_indices = []
    root_types = ["Root", "Bound_Root", "Stem", "Bound_Stem"]
    # find all the morphemes that correspond to the word
    for m_item in morph_tier:
        if word_id in m_item.attributes['segmentation']:
            morphemes.append(m_item)
            morpheme_ids.append(m_item.id)
    for g in xigt_igt['g']:
        if g.attributes['alignment'] in morpheme_ids:
            gloss_ids.append(g.id)
            glosses.append(g)
        '''
        first see if morphemes tier has morpheme types specified
        '''
    morph_idx = 0
    for m_item in morphemes:
        morphType = m_item.type
        if (not morphType is None) and (morphType in root_types):
            roots.append(m_item.get_content())
            root_morpheme_ids.append(m_item.id)
            root_indices.append(morph_idx)
            #root_indices.append(int(re.search("m\d+\.(\d+)", m_item.id).group(1)) - 1)
        morph_idx +=1
    for g in xigt_igt['g']:
        if g.attributes['alignment'] in root_morpheme_ids:
            root_gloss_tokens.append(g.get_content())        
        '''
        else, try to guess the root via blilngual alignments
        '''
    if root_indices == []:
        ba_tier = xigt_igt.get('a')
        if ba_tier is None:
            #raise Exception("No roots have been found via type attribute
            #on morphemes tier, and no bilingual alignment tier in the IGT!
            #No way to find roots...")
            return (None, None)
        for ba in ba_tier:
            ba_target = ba.attributes.get('target')
            if ba_target is None:
                continue
            ba_gloss_ids = ba_target.split(',')
            for gloss_id in ba_gloss_ids:
                if gloss_id in gloss_ids:
                    root_morph_id = xigt_igt['g'][gloss_id].attributes['alignment']
                    root_morph = xigt_igt['m'][root_morph_id].get_content()
                    # as a backoff, rely on the gloss morpheme being the
                    # same as the translation word.
                    if xigt_igt['g'][gloss_id].get_content() == xigt_igt['tw'][ba.attributes.get('source')].get_content():
                        roots.append(root_morph)
                        root_indices.append(morpheme_ids.index(root_morph_id))
    if root_indices == []:
        if len(gloss_ids) == 1:
            root_morph_id = xigt_igt['g'][gloss_ids[0]].attributes['alignment']
            root_morph = xigt_igt['m'][root_morph_id].get_content()
            roots.append(root_morph)
            root_indices.append(morpheme_ids.index(root_morph_id))
        else:
            for g_id in gloss_ids:
                gloss_segment = xigt_igt['g'][g_id].get_content()
                if not (gloss_segment.upper() in known_glosses or gloss_segment.isupper() or not gloss_segment.isalpha()):
                    root_morph_id = xigt_igt['g'][g_id].attributes['alignment']
                    root_morph = xigt_igt['m'][root_morph_id].get_content()
                    roots.append(root_morph)
                    root_indices.append(morpheme_ids.index(root_morph_id))
                    print("Guessing that {0} is the root because the gloss is not in known glosses, "
                              "not uppercase and doesn't have digits".format(root_morph))

    return (roots, root_indices)

def clean_list(list1, list2):
    for i in range(len(list1) - 1, -1, -1):
        if list1[i] is None or list2[i] is None:
            list1.pop(i)
            list2.pop(i)
