import codecs, os, sys
xigt_path = os.path.abspath("/Users/stephanie/projects/xigt/")
sys.path.append(xigt_path)
from xigt.codecs import xigtxml
# from xigt.query import (ancestors, descendants)

# author Stephanie Peterson slp24@uw.edu

# we will normalize everything coming in to be lowercase
case_grams = ["nom", "acc", "erg", "abs", "dat", "gen"]
pos_list = {"noun", "n", "nn", "pronoun", "pron", "determiner", "det", "d", "adjective", "adj", "adverb", "adv"}

class CaseParser():

    case_igt_list = []
    case_morphology_list = []
    case_lexicon_list = []

    def parse_morphology_case_features(self, path, file_ext):

        if os.path.isdir(path):
           for root, dirs, files in os.walk(path):
               for f in files:
                    if f.endswith(file_ext):
                        file = codecs.open(root+'/'+f,'r', encoding='utf8')
                        xc = xigtxml.load(file)
                        self.parse_IGT(xc)
                        self.collectCaseMarkingAffixes()
        return self.case_morphology_list

    def parse_lexicon_case_features(self, file_name):
        #file = open(file_name, "r")
        file = codecs.open(file_name,'r', encoding='utf8')
        xc = xigtxml.load(file)
        file.close()
        self.parse_IGT(xc)
        self.collectCaseMarkingWords()
        return self.case_lexicon_list

    def parse_IGT(self, xigt_corpus):
        #for tier in ancestors(gloss_tier, refattrs=('alignment', 'segmentation')):

        for igt in xigt_corpus:
            morphemes = {}
            words = {}
            morpheme_case_dict = {}
            word_morpheme_dict = {}
            word_classifier_pos_dict = {}
            word_projection_pos_dict = {}
            # keep an list of words we have inferred to be adpositions because we can
            # figure this out without pos tagging
            adposition_list = []

            gloss_tier = igt['g']
            morpheme_tier = igt['m']
            word_tier = igt['w']
            #probably want to get other pos data too and see which gives best results
            try:
                pos_tier_classifier = igt['w-pos']
                if 'w-pos_b' in igt.tiers:
                    pos_tier_projection = igt['w-pos_b']
                    for item in pos_tier_projection.items:
                        if item.alignment in word_morpheme_dict.keys():
                            pos = item.text.lower()
                            if pos in pos_list:
                                # w1 -> noun
                                word_projection_pos_dict[item.alignment] = item.text.lower()
            except:
                continue
            for item in word_tier.items:
                # w1 -> 'word'
                words[item.id] = item.value()
            for item in gloss_tier.items:
                gram = gloss_tier._dict[item.id].value().lower()
                if gram in case_grams:
                    # m1 -> NOM
                    morpheme_case_dict[item.alignment] = gram
            for item in morpheme_tier.items:
                if item.id in morpheme_case_dict.keys():
                    # m1 -> 'morpheme'
                    morphemes[item.id] = item.value()
                    # w1 -> m1
                    word_morpheme_dict[item.segmentation.rsplit('[')[0]] = item.id
                    # if w1's value equals m1's value assume it's an adposition todo not sure if this is the best way to do this
                    if morphemes[item.id] in words.values():
                        adposition_list.append(morphemes[item.id])
            for item in pos_tier_classifier.items:
                if item.alignment in word_morpheme_dict.keys():
                    pos = item.text.lower()
                    if pos in pos_list:
                        # w1 -> noun
                        word_classifier_pos_dict[item.alignment] = item.text.lower()

            # this might not have everything, as it depends on how good the pos tagging is
            # base what was written off what is here
            # future work: try to resolve what was not written
            self.case_igt_list.append(CaseIGT(igt.id, morphemes, words, morpheme_case_dict, word_morpheme_dict, word_classifier_pos_dict, word_projection_pos_dict, adposition_list))

    def collectCaseMarkingAffixes(self):
        print('collect the affixes to give to MOM')
        uniqueAffixesToPos = {}
        for case_igt in self.case_igt_list:
            for w in case_igt.word_morpheme_dict.keys():
                # todo need to only grab affixes here
                if "-" in case_igt.words.get(w):
                    if case_igt.morphemes.get(case_igt.word_morpheme_dict.get(w)) not in uniqueAffixesToPos.keys():
                        uniqueAffixesToPos[case_igt.morphemes.get(case_igt.word_morpheme_dict.get(w))] = case_igt.word_class_pos_dict.get(w)
                        self.case_morphology_list.append(CaseMorphologyEntry(case_igt.words.get(w),
                                                            case_igt.word_class_pos_dict.get(w),
                                                            case_igt.word_proj_pos_dict.get(w),
                                                            case_igt.morpheme_case_dict.get(case_igt.word_morpheme_dict.get(w)),
                                                            case_igt.morphemes.get(case_igt.word_morpheme_dict.get(w))))
                    else:
                        # if it's in already in there only add it if the pos is differnt
                        if uniqueAffixesToPos[case_igt.morphemes.get(case_igt.word_morpheme_dict.get(w))] != case_igt.word_class_pos_dict.get(w):
                                self.case_morphology_list.append(CaseMorphologyEntry(case_igt.words.get(w),
                                                                case_igt.word_class_pos_dict.get(w),
                                                                case_igt.word_proj_pos_dict.get(w),
                                                                case_igt.morpheme_case_dict.get(case_igt.word_morpheme_dict.get(w)),
                                                                case_igt.morphemes.get(case_igt.word_morpheme_dict.get(w))))

    def collectCaseMarkingWords(self):
        print('collect words with inherent case marking (including adpositions) to give to MY_MOM')
        uniqueWords = []
        for case_igt in self.case_igt_list:
            for w in case_igt.word_morpheme_dict.keys():
                # todo need to only grab roots and standalone words here
                if "-" not in case_igt.words.get(w):
                    if case_igt.words.get(w) not in uniqueWords:
                        uniqueWords.append(case_igt.words.get(w))
                        if case_igt.words.get(w) in case_igt.adpositions:
                            self.case_lexicon_list.append(CaseLexiconEntry("adp",
                                                          "adp",
                                                          case_igt.morpheme_case_dict.get(case_igt.word_morpheme_dict.get(w)),
                                                          case_igt.words.get(w)))
                        else:
                            self.case_lexicon_list.append(CaseLexiconEntry(case_igt.word_class_pos_dict.get(w),
                                                          case_igt.word_proj_pos_dict.get(w),
                                                          case_igt.morpheme_case_dict.get(case_igt.word_morpheme_dict.get(w)),
                                                          case_igt.words.get(w)))

# pass this to MOM
class CaseMorphologyEntry():
    def __init__(self, parent, class_pos, proj_pos, feature_value, affix):
        self.parent = parent
        self.class_pos = class_pos
        self.proj_pos = proj_pos
        self.feature_name = 'case'
        self.feature_value = feature_value # gram
        self.affix = affix

class CaseLexiconEntry():
    def __init__(self, class_pos, proj_pos, feature_value, word):
        self.class_pos = class_pos
        self.proj_pos = proj_pos
        self.feature_name = 'case'
        self.feature_value = feature_value
        self.word = word

class CaseIGT():
    def __init__(self, igt_id, morphemes, words, morpheme_case_dict, word_morpheme_dict, word_class_pos_dict, word_proj_pos_dict, adpositions):
        self.igt_id = igt_id
        self.morphemes = morphemes
        self.words = words
        self.morpheme_case_dict = morpheme_case_dict
        self.word_morpheme_dict = word_morpheme_dict
        self.word_class_pos_dict = word_class_pos_dict
        self.word_proj_pos_dict = word_proj_pos_dict
        self.adpositions = adpositions

def main(args):
    CaseParser().parse_lexicon_case_features(args)

#main('RUN_MOM_RUS/rus/testsuite-enriched-subset.xml')
main('RUN_MOM_JPN/jpn/testsuite-enriched-subset.xml')



