AGGREGATION
Automatic Generation of Grammars for Endangered Languages from Glosses and Typological Information

The AGGREGATION Project is concerned with automatically producing machine-readable grammars for low-resource and endangered languages on the basis of interlinear glossed text (IGT) data. Our grammar inference system produces a grammar specification file which can be input into the Grammar Matrix (Bender et al., 2002, 2010; http://matrix.ling.washington.edu/index.html) to automatically produce an implemented HPSG (Polard and Sag, 1996) grammar for the language.

The inference system has two primary components: 
(1) MOM (Wax 2014, Zamaraeva 2016; https://git.ling.washington.edu/agg/mom) for morphological and lexical inference
(2) BASIL: Building Analyses from Syntactic Inference in Low-resource languages (Howell 2020) for syntactic and additional lexical inference

This is the repository for BASIL, which takes an IGT corpus as input, leverages inference done by MOM and does additional inference to produce a grammar specification for a language.

Getting set up:
To run this project:
git clone https://git.ling.washington.edu/agg/aggregation.git
git clone https://git.ling.washington.edu/agg/mom.git (there are additional requirements for running MOM. See the MOM readme)
pip install xigt
python3 __main__.py config

BASIL uses the same config as MOM. A sample file and description can be found here: https://git.ling.washington.edu/agg/mom/-/tree/master/mom/config_files

Data requirements:
BASIL requires a Xigt corpus that has been enriched with INTENT (Georgi 2016). For instructions on converting a Toolbox, FLeX or text dataset to Xigt and enriching it with INTENT, see data_preparation.txt.

Using the output grammar specification:
The grammar specification can be input to the Grammar Matrix customization system to generate a grammar. This grammar can then be loaded into parsing software to parse sentences in the language.
An end-to-end pipeline that infers the grammar spec with BASIL, generates a grammar and parses sentences can be found here: https://git.ling.washington.edu/agg/repro/basil-2020

If you don't want to use the pipeline, here are some useful links:
Grammar Matrix: http://matrix.ling.washington.edu/index.html
ACE: http://sweaglesw.org/linguistics/ace/
LKB: http://moin.delph-in.net/LkbTop

Citing AGG:
BASIL: 
Kristen Howell. 2020. Inferring Grammars from Interlinear Glossed Text: Extracting Typological and Lexical Properties for the Automatic Generation of HPSG Grammars. PhD thesis, University of Washington.
@phdthesis{Howell:20,
  title={Inferring Grammars from Interlinear Glossed Text: Extracting Typological and Lexical Properties for the Automatic Generation of HPSG Grammars},
  author={Kristen Howell},
  year={2020},
  school={University of Washington}
}

Earlier work in grammar inference:
Zamaraeva, Olga, Kristen Howell and Emily M. Bender. 2019. Handling Cross-cutting Properties in Automatic Inference of Lexical Classes: A Case Study of Chintang. Proceedings of the 3rd Workshop on the Use of Computational Methods in the Study of Endangered Languages, Honolulu, HI. pp.28-38. [.bib]
Bender, Emily M., Joshua Crowgey, Michael Wayne Goodman and Fei Xia. 2014. Learning Grammar Specifications from IGT: A Case Study of Chintang. Proceedings of the 2014 Workshop on the Use of Computational Methods in the Study of Endangered Languages, ACL 2014, Baltimore, MD.
Bender, Emily M., Michael Wayne Goodman, Joshua Crowgey and Fei Xia. 2013. Towards Creating Precision Grammars from Interlinear Glossed Text: Inferring Large-Scale Typological Properties. In Proceedings of the ACL 2013 workshop on Language Technology for Cultural Heritage, Social Sciences and Humanities. [.bib]

Contact:
If you have questions, please contact Emily M. Bender or Kristen Howell (kphowell at u w dot edu)

Acknowledgments:
This research is based upon work supported by the National Science Foundation under Grant No. BCS-1561833 (PI Bender). Any opinions,Findings, and conclusions or recommendations expressed in this material are those of the author(s) and do not necessarily reflect the views of the National Science Foundation.