#!/usr/bin/env python3
# -*- coding: utf-8 -*
# Author: Kristen Howell

# This script takes a xigt file created by the toolbox module and adds the necessary tiers and tags to be
# enriched by INTENT.

# -*- coding: utf-8 -*-

import sys
from xigt.codecs import xigtxml
from xigt import Tier, Item


xc = xigtxml.load(open(sys.argv[1]))
for igt in xc:
    # create normalized tier
    ntier = Tier(id='n',
                 type='odin')
    ntier.attributes['state'] = 'normalized'

    # create language item
    phrases = igt.get('p')
    # this does not consider the possibility that their might be more than one item in this tier
    langline = phrases.get('p1')
    L = Item(id='n1',
             text=langline.text)
    L.attributes['tag'] = 'L'
    ntier.append(L)
    # make the language item in the normalized tier the content of the phrases line
    phrases.content = 'n'
    langline.content = 'n1'

    # create gloss item
    glosses = igt.get('g')
    morphemes = igt.get('m')
    words = igt.get('w')
    lastword = ''
    glossline = ''
    # build gloss line from each gloss
    for gloss in glosses:
        morpheme = morphemes.get(gloss.alignment)
        word = words.get(morpheme.segmentation)
        if word.id == lastword:
            glossline += gloss.text
        else:
            glossline += (' ' + gloss.text)
        lastword = word.id

    # create glossline item
    G = Item(id='n2',
             text=glossline)
    G.attributes['tag'] = 'G'
    ntier.append(G)

    # create translation item
    translation = igt.get('t')
    # this does not consider the possibility that their might be more than one item in this tier
    transline = translation.get('t1')
    T = Item(id='n3',
             text=transline.text)
    T.attributes['tag'] = 'T'
    ntier.append(T)
    # make the translation item in the normalized tier the content of the translation line
    translation.content = 'n'
    transline.content = 'n3'

    # add tier to igt
    igt.append(ntier)


xigtxml.dump(open(sys.argv[2], 'w'), xc)