# Author Kristen Howell
# This script was written to add pos tags from a toolbox lexicon to the Titan [ttv] corpus

import re
from xigt.codecs import xigtxml
from xigt.model import XigtCorpus, Igt, Tier, Item

def create_lexicon_dict(lex_file):
    lex_dict = {}
    i = 0
    while i < len(lex_file):
        line = lex_file[i]
        if line.startswith('\\lx'):
            orth = re.sub('^\\\\lx ', '', line)
            orth = re.sub('\n$', '', orth)
            #orth = line.lstrip('\\lx ').rstrip('\n')
            orth_list = [orth]
            if lex_file[i+1].startswith('\\va'):
                orth2 = re.sub('^\\\\va ', '', lex_file[i+1])
                orth2 = re.sub('\n$', '', orth2)
                orth_list.append(orth2)
            for o in orth_list:
                if o not in lex_dict:
                    lex_dict[o] = {}
                else:
                    print('duplicate orth!')
            i += 1
            while i < len(lex_file) and not lex_file[i].startswith('\\lx'):
                line = lex_file[i]
                if line.startswith('\\ps'):
                    pos = re.sub('^\\\\ps ', '', line)
                    pos = re.sub('\n$', '', pos)
                    pos = pos.split(', ')[0]
                    #pos = line.lstrip('\\ps ').rstrip('\n')
                    i += 1
                    while i < len(lex_file) and not lex_file[i].startswith('\\ps') and not lex_file[i].startswith('\\lx'):
                        line = lex_file[i]
                        if line.startswith('\\ge'):
                            gloss = re.sub('^\\\\ge ', '', line)
                            gloss = re.sub('\n$', '', gloss)
                            # gloss = line.lstrip('\\ge ').rstrip('\n')
                            if ',' in gloss:
                                glosses = gloss.split(', ')
                                for o in orth_list:
                                    for g in glosses:
                                        if g not in lex_dict[o]:
                                            lex_dict[o][g] = {pos}
                                        else:
                                            lex_dict[o][g].add(pos)
                                            print('multiple pos tags for one orth and gloss')
                            else:
                                for o in orth_list:
                                    if gloss not in lex_dict[o]:
                                        lex_dict[o][gloss] = {pos}
                                    else:
                                        lex_dict[o][gloss].add(pos)
                                        print('multiple pos tags for one orth and gloss')
                        i += 1
                else:
                    i += 1
        else:
            i += 1

    return lex_dict

lexicon = create_lexicon_dict(open('C:/Users/Kristen/Documents/code/agg/data/ttv/toolbox/corrlex1401', encoding='utf-8', errors='ignore').readlines())
corpus = open('C:/Users/Kristen/Documents/code/agg/data/ttv/xigt/ttv.xml', encoding='utf-8')
xc = xigtxml.load(corpus)
for igt in xc:
    pos_tier = Tier(id='m-pos', type='pos', alignment='m')
    pos_tier.attributes['alignment'] = 'm'
    morph_tier = igt.get('m')
    gloss_tier = igt.get('g')
    if gloss_tier:
        for g in gloss_tier:
            gloss = g.value()
            morph_id = g.alignment
            gloss_id = g.id
            morph = morph_tier.get(morph_id).value()
            pos = ''
            if morph in lexicon:
                if gloss in lexicon[morph]:
                    pos_list = [x for x in lexicon[morph][gloss]]
                    if len(pos_list) > 0:
                        pos = pos_list[0]
                    else:
                        print()
                else:
                    gloss_list = [x for x in lexicon[morph]]
                    if gloss_list:
                        pos_list = [x for x in lexicon[morph][gloss_list[0]]]
                        if len(pos_list) > 0:
                            pos = pos_list[0]
                        else:
                            print()
                    elif morph != 'va':
                        print()
            else:
                trans_tier = igt.get('t')
                if trans_tier:
                    trans = trans_tier[0].value()
                if morph[0].isupper():
                    if trans:
                        if morph in trans:
                            pos = 'propernoun'
            if pos:
                pos_item = Item(id='pos'+gloss_id[1:],text=pos,alignment=morph_id)
                pos_item.attributes['alignment'] = morph_id
                pos_tier.append(pos_item)
    else:
        print('no morph tier')
    igt.append(pos_tier)

xigtxml.dump('C:/Users/Kristen/Documents/code/agg/data/ttv/xigt/ttv_with_pos.xml', xc, encoding='utf-8', indent=2)
