#!/usr/bin/env python3
# -*- coding: utf-8 -*
# Author: Kristen Howell
# This script is specifically designed to take the Italian ODIN corpus and produce a new corpus of only the normalized
# tier with the clitics attached to the verbs, that can be re-enriched by INTENT

import sys
from xigt.codecs import xigtxml
from xigt.model import XigtCorpus, Igt, Tier, Item


#    rt = Tier(id = RAW_ID, type=ODIN_TIER_TYPE, attributes={STATE_ATTRIBUTE:RAW_STATE}, igt=inst)

#PRONOUNS = ['i', 'you','he','she','they','it','me','him','her','them']#leaving out genitive pronouns (his, its and their)
#CLITICS = {'i': ['io'], 'you': ['tu', 'lei', 'voi', 'loro', 'ti', 'vi'], 'he': ['lui'], 'she': ['lei'], 'we': ['noi'], 'they': ['loro', 'essi'], 'it': ['esso', 'essa'], 'me': ['mi'], 'him': ['lo'], 'her': ['la'], 'us': ['ci'], 'them': ['li','le']}
CLITICS ={'mi': {'count': 0, 'gloss': []}, 'ti': {'count': 0, 'gloss': []}, 'gli': {'count': 0, 'gloss': []}, 'le': {'count': 0, 'gloss': []}, 'ci': {'count': 0, 'gloss': []}, 'vi': {'count': 0, 'gloss': []}, 'ci': {'count': 0, 'gloss': []}, 'si': {'count': 0, 'gloss': []}, 'lo': {'count': 0, 'gloss': []}, 'la': {'count': 0, 'gloss': []}, 'li': {'count': 0, 'gloss': []}, 'le': {'count': 0, 'gloss': []}, 'ne': {'count': 0, 'gloss': []}, 'me': {'count': 0, 'gloss': []}, 'loro': {'count': 0, 'gloss': []}, 'lei': {'count': 0, 'gloss': []}, 'lui': {'count': 0, 'gloss': []}, 'no': {'count': 0, 'gloss': []}, 'non': {'count': 0, 'gloss': []}}
NOT_CLITICS ={'mi': {'count': 0, 'gloss': []}, 'ti': {'count': 0, 'gloss': []}, 'gli': {'count': 0, 'gloss': []}, 'le': {'count': 0, 'gloss': []}, 'ci': {'count': 0, 'gloss': []}, 'vi': {'count': 0, 'gloss': []}, 'ci': {'count': 0, 'gloss': []}, 'si': {'count': 0, 'gloss': []}, 'lo': {'count': 0, 'gloss': []}, 'la': {'count': 0, 'gloss': []}, 'li': {'count': 0, 'gloss': []}, 'le': {'count': 0, 'gloss': []}, 'ne': {'count': 0, 'gloss': []}, 'me': {'count': 0, 'gloss': []}, 'loro': {'count': 0, 'gloss': []}, 'lei': {'count': 0, 'gloss': []}, 'lui': {'count': 0, 'gloss': []}, 'no': {'count': 0, 'gloss': []}, 'non': {'count': 0, 'gloss': []}}
#some of these are real homophones (the, theirs), the rest are just bad alignments
NON_CLITIC_GLOSSES = ['the', 'will', 'has', 'thiers', 'of-it']

corpus = open(sys.argv[1], encoding='utf-8')
xc = xigtxml.load(corpus)
new_corpus = XigtCorpus(nsmap=xc.nsmap)
# add corpus signature: id = 'igt6-30', doc-id = xc.get('doc-id'), tag-types = xc.get('tag-types')
# add the corpus id to new corpus
# add the metadata tier to the corpus
for igt in xc:
    # add other igt info and metadata
    new_igt = Igt(id = igt.id, attributes=igt.attributes, metadata=igt.metadata)
    normalized_tier = igt.get('n')
    if not igt.get('w') or not igt.get('gw'):
        new_igt.append(normalized_tier)
    else:
        new_tier = Tier(id=normalized_tier.id, type=normalized_tier.type, attributes=normalized_tier.attributes)
        w_string = ''
        clitics = {}
        i = 1
        word_ids = [x.id for x in igt.get('w')]
        words = [x.value().lower() for x in igt.get('w')]
        i = 0
        while i < len(words):
            is_clitic = False
            if words[i] in CLITICS:
                if i+1 < len(words) and words[i+1] in CLITICS:
                    is_clitic = True
                else:
                    if igt.get('w-pos'):
                        for pos in igt.get('w-pos'):
                            if i+1 < len(words) and pos.attributes['alignment'] == word_ids[i+1] and pos.value() == 'VERB':
                                is_clitic = True
                if is_clitic:
                    for gw in igt.get('gw'):
                        if 'alignment' in gw.attributes:
                            if gw.attributes['alignment'] == word_ids[i] and gw.value() in NON_CLITIC_GLOSSES:
                                is_clitic = False
                if not is_clitic:
                    NOT_CLITICS[words[i]]['count'] += 1
                    NOT_CLITICS[words[i]]['gloss'].append(igt.id)
            if is_clitic:
                clitics[word_ids[i]] = words[i]
                w_string += words[i]
            else:
                w_string += (words[i]+' ')
            i+=1
        w_string = w_string[0:(len(w_string) - 1)]

        gw_string = ''
        gloss_words = igt.get('gw')
        for gw in gloss_words:
            if 'alignment' in gw.attributes and gw.attributes['alignment'] in clitics:
                gw_string += (gw.value()+'=')
                CLITICS[clitics[gw.attributes['alignment']]]['count'] += 1
                if gw.value() not in CLITICS[clitics[gw.attributes['alignment']]]['gloss']:
                    CLITICS[clitics[gw.attributes['alignment']]]['gloss'].append(gw.value())
            else:
                gw_string += gw.value()+' '
        gw_string = gw_string[0:(len(gw_string)-1)]

        trans_words = igt.get('tw')
        t_string = ''
        #sometimes there is not translation line... because we all speak italian
        if trans_words != None:
            for tw in trans_words:
                t_string += (tw.value()+' ')
            t_string = t_string[0:(len(t_string) - 1)]

        #Because I'm reconstructing the tiers by hand, I only manually add some attributes
        l_item = Item(id='n1', text=w_string, attributes={'tag': "L+CR"})
        new_tier.append(l_item)
        g_item = Item(id='n2', text=gw_string, attributes={'tag': "G"})
        new_tier.append(g_item)
        if t_string != '':
            t_item = Item(id='n3', text=t_string, attributes={'tag': "T"})
            new_tier.append(t_item)

        new_igt.append(new_tier)

    new_igt.append(igt.get('r'))
    new_igt.append(igt.get('c'))
    new_corpus.append(new_igt)

print('CLITICS')
for clitic in CLITICS:
    print(clitic + ': ' + str(CLITICS[clitic]['count']) + ': ' + str(CLITICS[clitic]['gloss']))
print('NOT CLITICS')
for clitic in NOT_CLITICS:
    print(clitic + ': ' + str(NOT_CLITICS[clitic]['count']) + ': ' + str(NOT_CLITICS[clitic]['gloss']))

xigtxml.dump('C:/Users/Kristen/Documents/code/enriched_odin/ita_new.xml', new_corpus, encoding='utf-8', indent=2)
