#!/usr/bin/env python3
# -*- coding: utf-8 -*
# Author: Kriten Howell
# This method uses a set of standard glosses (extracted from map_gloss/gloss/data/standard_grams.json by Michael Lockwood), to identify the root of a verb (both the language and translation) in an xigt igt
# Currently this method looks at the item's value for the orthographical representation. It has not been confirmed whether this value comes from the orth line or the language line of the IGT if both are present
# The intended output is a tuple contianing the verb lemma first and translation second
from xigt import ref
import re

def find_root(igt, item):
    leipzig_grams = ['1', '2', '3', 'SG', 'DU', 'PL', '1SG', '2SG', '3SG', '1DU', '2DU', '3DU', '1PL', '2PL', '3PL', 'A', 'ABL', 'ABS', 'ACC', 'ALL', 'ANIM', 'ANTIP', 'APPL', 'AUX', 'BEN', 'CAUS', 'COM', 'COMP', 'COMPL', 'COND', 'CONJ', 'CONT', 'COP', 'DAT', 'DECL', 'DEF', 'DEM', 'DIR', 'DIST', 'DISTR', 'DUB', 'DUR', 'EMPH', 'ERG', 'EXCL', 'F', 'FP', 'FV', 'FAM', 'FOC', 'FORM', 'FUT', 'GEN', 'HAB', 'IMP', 'INANIM', 'INCL', 'IND', 'INDF', 'INF', 'INS', 'INTR', 'INV', 'IPFV', 'IRR', 'LOC', 'M', 'N', 'N-', 'NEG', 'NFUT', 'NMLZ', 'NOM', 'NPRS', 'NPST', 'NRFUT', 'NRPRS', 'NRPST', 'OBJ', 'OBL', 'P', 'PASS', 'PERS', 'PFV', 'PL', 'POSS', 'PRED', 'PRET', 'PRF', 'PROG', 'PROH', 'PROX', 'PRS', 'PST', 'PTCP', 'PURP', 'Q', 'QUOT', 'REC', 'RECP', 'REFL', 'REL', 'REM', 'REP', 'RES', 'S', 'SBJ', 'SBJV', 'SG', 'STAT', 'TOP', 'TR', 'VOC']

    gloss_ids = set(i.id for i in igt.get('gw'))
    referrers = ref.referrers(igt, item.id, ['alignment'])['alignment']
    item_gloss_ids = [r for r in referrers if r in gloss_ids]
    glosses = ''
    for id in item_gloss_ids:
        glosses += igt.get_item(id).value()

    gloss_grams = re.split('-', glosses)
    orth_grams = re.split('-', item.value())
    if orth_grams == [''] or gloss_grams == ['']:
        return '', ''
    if len(orth_grams) != len(gloss_grams):
        #print("WARNING: find root skipped igt " + str(igt.id))
        return '', ''
    leipzig_ranks = []
    for each in gloss_grams:
        leipzig_ranks.append(0)
    lc_ranks = []
    for each in gloss_grams:
        lc_ranks.append(0)
    i = 0
    while i < len(gloss_grams):
        if gloss_grams[i].upper() not in leipzig_grams:
            leipzig_ranks[i] = 1
        i+=1
    not_standard_glosses = [i for i, x in enumerate(leipzig_ranks) if x == 1]
    if len(not_standard_glosses) == 1:
        return orth_grams[not_standard_glosses[0]], gloss_grams[not_standard_glosses[0]]
    i = 0
    while i < len(gloss_grams):
        if gloss_grams[i].islower() == True:
            lc_ranks[i] = 1
        i+=1
    lower_case_glosses = [i for i, x in enumerate(lc_ranks) if x == 1]
    if len(lower_case_glosses) == 1:
        return orth_grams[lower_case_glosses[0]], gloss_grams[lower_case_glosses[0]]
    possible_roots = list(set(not_standard_glosses) & set(lower_case_glosses))
    if len(possible_roots) == 1:
        return orth_grams[possible_roots[0]], gloss_grams[possible_roots[0]]
    else:
        i = 1
        if len(possible_roots) != 0:
            word = gloss_grams[possible_roots[0]]
        else:
            word = ''
        while i < len(possible_roots):
            word = word + '.' + gloss_grams[possible_roots[i]]
            i += 1
        return item.value(), word
