#!/opt/python-3.4/bin/python3.4
# -*- coding: utf-8 -*-

from xigt.codecs import xigtxml

# Some notes as I work this out:
# xc[0]['x'] gets me the syntax tier of the first item
# xc[0]['x'].get_content() gets me the actual label
# xc[0]['x'].attributes['alignment'] gets me the alignment
# xc[0]['x'].attributes['alignment'] gets me the tier it points to

def format_preterminal_node(igt,w,item,nodes):
    '''
    Given a Xigt syntax tier item corresponding to a preterminal node,
    and the entier Xigt igt to which it belongs, format the PTB-type 
    subtree for this node with the label as the mother and the aligned
    word as the daughter.
    '''
    #print(item.get_content())
    #print(w[item.attributes['alignment']].get_content())
    subtree = '(' + item.get_content() + ' ' + w[item.attributes['alignment']].get_content() + ')'
    nodes[item.id] = subtree
    return(nodes)


def format_intermediate_node(item,nodes,orphans):
    '''
    Given a Xigt item corresponding to an intermediate node and 
    a list of existing nodes and orphans, format the node and add
    it to nodes (removing its children from nodes) or add it to
    orphans.
    '''

    #print(item.get_content())

    children = item.attributes['children'].split()#(',')
    #print(children)
    orphan = False
    for child in children:
        if child in nodes.keys():
            continue
        else:
            orphan = True
    if orphan:
        if item not in orphans: # If we're on the first pass through, we'll need to add this to orphans.
            orphans.append(item)
            #print("found orphan "+item.get_content())
    else:
        if item in orphans:
            orphans.remove(item) # If we're not on the first pass through, we'll want to remove this from orphans.
        subtree = '(' + item.get_content() # Create the subtree
        for child in children: # Relying on the order of the children here.
            #print("used "+child)
            #print(nodes[child])
            subtree += ' ' + nodes[child]
            del nodes[child] # We're talking trees here, so I don't need the nodes once they're been incorporated into a larger subtree.
            #print(subtree)
        subtree += ')'
        nodes[item.id] = subtree
        #print(subtree)
    #print("done")
    return(nodes, orphans)


def format_ptb_tree(igt):
    '''
    Take in a Xigt igt item and read the syntax tier, outputting
    the tree in PTB format.  Assume that 'x' labels the syntax tier.
    '''
    x = igt['w-ps'] # The syntax tier.
    w = igt[x.attributes['alignment']] # The tier the syntax tier is aligned to
    nodes = {} #Dictionary of nodes with Xigt ids as keys.
    orphans = [] #Array of nodes whose children aren't yet in nodes.
    
    for item in x:  
        # If there's an alignemnt attribute, we're talking about a preterminal, where the alignment points to the word.
        # Enter these directly into the nodes dictionary.
        if 'alignment' in item.attributes:
            nodes = format_preterminal_node(igt,w,item,nodes)
                  
        # If there's a children attribute, we're talking about a non-preterminal.  Enter these if their children
        # are already all in the nodes dictionary.  If not, move to a separate array of nodes to loop back through.
        if 'children' in item.attributes:
            #print("doing "+item.get_content())
            (nodes,orphans) = format_intermediate_node(item,nodes,orphans)

    # If the orphans list is empty, then we're done, and the nodes list should contain exactly one node which
    # which is the complete tree.
    while orphans:
        #print(nodes)
        #print(orphans)
        for orphan in orphans:
            (nodes,orphans) = format_intermediate_node(orphan,nodes,orphans)
                    
    # Once the orphans list is empty, nodes should have exactly one thing on it:
    if len(nodes) != 1:
        sys.exit("Something's wrong: Finished all orphaned nodes, but nodes has more than one element in it.  Bad input tree?")
    else:
        tree = next(iter(nodes.values()))
    #print("finished with "+tree)
    return(tree)

    

