#!/usr/bin/env python3
# -*- coding: utf-8 -*
# Author: Kristen Howell
# this contains helper functions for making changes to morphology choices after mom, such as filtering out particular rules.


from XigtifiedToolbox import FeatDict
from mom import MatrixChoices
import re

known_features = FeatDict.FeatureDictionary().all_keys
feat_dict = FeatDict.FeatureDictionary().dictionaries

def filter_reduntant_zero_rules(choices):
    redundant_zero_rules = []
    for pos in ['verb','noun']:
        for pc in choices[pos+'-pc']:
            for lrt in pc['lrt']:
                for lri in lrt['lri']:
                    if lri['inflecting'] == 'no':
                        #kph need to also check for lris where one doesn't inflect but others do
                        #actually maybe not, because we only want to ditch themif they have matriching features, which will happen anyway
                        #if rule_with_matching_features(pc, lrt['feat'], lrt.full_key, redundant_zero_rules):
                        if rule_with_matching_features(pos, lrt['feat'], lrt.full_key, redundant_zero_rules, choices):
                            redundant_zero_rules.append(lri.full_key)

    for lri in redundant_zero_rules:
        del choices[lri] #note: this just removes the lri. if it's the only lri in the lrt, the lrt will still be in the grammar, but with no lri associated with it




    for pos in ['verb','noun']:
        for pc in choices[pos+'-pc']:
            zero_rule = False
            for lrt in pc['lrt']:
                for lri in lrt['lri']:
                    if lri['inflecting'] == 'no':
                        zero_rule = True
            if not zero_rule and pc['obligatory'] == 'on':
                del pc['obligatory']

    return choices
#def rule_with_matching_features(pc, feats, lri_id, redundant_zero_rules):
def rule_with_matching_features(pos, feats, lri_id, redundant_zero_rules, choices):
    match = False
    if not feats:
        return match
    for pc in choices[pos + '-pc']:
        for lrt in pc['lrt']:
            if any([lri for lri in lrt['lri'] if lri['inflecting'] == 'yes']):# kph added per this note: when comparing features, you need to make sure that within the lrt there is an lri that inflects
                for lri in lrt['lri']:
                    if lri.full_key != lri_id:
                        if lri.full_key not in redundant_zero_rules:
                            if matching_features(feats, lrt['feat']):
                                match = True
    return match

def matching_features(x_feats, y_feats):
    match = True
    if not x_feats:
        return False
    if not y_feats:
        return False
    for x in x_feats:
        if not any(y for y in y_feats if x['name'] == y['name'] and x['value'] == y['value']):
            match = False
    if match:
        for y in y_feats:
            if not any(x for x in x_feats if x['name'] == y['name'] and x['value'] == y['value']):
                match = False
    return match
