#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Allison Dods

from pickle import FALSE
from re import M, T
from socketserver import ThreadingUnixDatagramServer
from sys import prefix
from mom import MatrixChoices
from xigt import xigtpath as xp
from xigt import model 


"""
PHASE 3:
SETTING CHOICES IN THE CHOICES FILE







"""
# Create a nonpossesion lrt
def create_nonpossession_lrt(main_choices, new_pc_name, stratdict):

    lrtID = new_pc_name + '_lrt1'
    main_choices[lrtID + '_name'] = lrtID
    i = 1
    for strat_name in stratdict.keys():
        featID = '_feat' + str(i)
        main_choices[lrtID + featID + '_name'] = strat_name
        main_choices[lrtID + featID + '_value'] = 'nonpossessive'
        main_choices[lrtID + featID + '_head'] = 'itself'
        i += 1
    main_choices[lrtID + '_lri1_inflecting'] = 'no'

    return main_choices

def create_poss_strat_agr_lrt(main_choices, lrtcounter, strat, strat_name, new_pc_name, markloc, agr_dict, orth):

    lrtID = new_pc_name + '_lrt' + str(lrtcounter)
    main_choices[lrtID + '_name'] = lrtID

    # the first feature will always be the possessive one
    main_choices[lrtID + '_feat1_name'] = strat_name
    main_choices[lrtID + '_feat1_value'] = markloc
    main_choices[lrtID + '_feat1_head'] = 'itself'

    # add orthography
    main_choices[lrtID + '_lri1_inflecting'] = 'yes'
    main_choices[lrtID + '_lri1_orth'] = orth

    # subsequent features map onto each agreement feature for this strat+markloc combo
    featnum = 2
    for featname in agr_dict.keys():
        featID = lrtID + '_feat' + str(featnum)
        main_choices[featID + '_name'] = featname
        main_choices[featID + '_value'] = agr_dict[featname]
        if markloc == 'possessor':
            main_choices[featID + '_head'] = 'possessum'
        else:
            main_choices[featID + '_head'] = 'possessor'
        featnum += 1

    return main_choices

# Create 1 or more LRTs for the possessive affixes within a noun-pc
def create_poss_pron_lrts(main_choices, new_pc_name, prondict):

    lrtcounter = 1

    for pron_name in prondict.keys():

        pron = prondict[pron_name]
        feats_by_lrt = {}
        lris_by_lrt = {}

        for orth in pron['psmagrforms'].keys():
            agr_dict = pron['psmagrforms'][orth]

            neverseen_feats = True


            for previous_lrt in feats_by_lrt.keys():
                featval = feats_by_lrt[previous_lrt]
                if agr_dict == featval:
                    num_lris = lris_by_lrt[previous_lrt]
                    new_lri = num_lris + 1
                    main_choices[new_pc_name + '_lrt' + str(previous_lrt) + '_lri' + str(new_lri) + '_inflecting'] = 'yes'
                    main_choices[new_pc_name + '_lrt' + str(previous_lrt) + '_lri' + str(new_lri) + '_orth'] = orth
                    lris_by_lrt[previous_lrt] = new_lri
                    neverseen_feats = False

            if neverseen_feats == True: # we haven't seen this combination of features and values before, so create a new LRT
                lrtID = new_pc_name + '_lrt' + str(lrtcounter)
                main_choices[lrtID + '_name'] = lrtID

                # the first feature will always be the possessive one
                main_choices[lrtID + '_feat1_name'] = pron_name
                main_choices[lrtID + '_feat1_value'] = 'plus'
                main_choices[lrtID + '_feat1_head'] = 'itself'

                # add orthography
                main_choices[lrtID + '_lri1_inflecting'] = 'yes'
                main_choices[lrtID + '_lri1_orth'] = orth

                feats_by_lrt[lrtcounter] = agr_dict
                lris_by_lrt[lrtcounter] = 1

                
                featnum = 2
                for featname in agr_dict.keys():
                    featID = lrtID + '_feat' + str(featnum)
                    main_choices[featID + '_name'] = featname
                    main_choices[featID + '_value'] = agr_dict[featname]
                    main_choices[featID + '_head'] = 'itself'
                    featnum += 1
                lrtcounter += 1


    return main_choices

# Create 1 or more LRTs for the possessive affixes within a noun-pc
def create_poss_strat_lrts(main_choices, new_pc_name, stratdict):

    lrtcounter = 1 
    nonagr_lrt_exists = False
    nonagr_strat_counter = 1

    for strat_name in stratdict.keys():

        strat = stratdict[strat_name]
        markloc = strat['mark-loc']
        orth = ''
        if markloc == 'possessor' or markloc == 'both':
            if strat['psr agr'] == True:
                # create a new LRT for each orthography
                for orth in strat['psragrforms'].keys():
                    agr_dict = strat['psragrforms'][orth]
                    main_choices = create_poss_strat_agr_lrt(main_choices, lrtcounter, strat, strat_name, new_pc_name, 'possessor', agr_dict, orth)
                    lrtcounter += 1
            else:
                if nonagr_lrt_exists == False: # a nonagr lrt has not been set yet
                    lrtID = new_pc_name + '_lrt' + str(lrtcounter)
                    main_choices[lrtID + '_name'] = 'lrt' + str(lrtcounter)
                    nonagr_lrt_exists = True
                    lrtcounter += 1
                # ok, at this point we know we're dealing with a nonagr psr. we need to add that to the LRT
                # set features
                featID = '_feat' + str(nonagr_strat_counter)
                main_choices[lrtID + featID + '_name'] = strat_name
                main_choices[lrtID + featID + '_value'] = 'possessor'
                main_choices[lrtID + featID + '_head'] = 'itself'
                # set orthography
                orth = strat['psr orth']
                orthID = '_lri' + str(nonagr_strat_counter)
                main_choices[lrtID + orthID + '_inflecting'] = 'yes'
                main_choices[lrtID + orthID + '_orth'] = orth
                nonagr_strat_counter += 1


        if markloc == 'possessum' or markloc == 'both':
            if strat['psm agr'] == True:
                # create a new LRT for each orthography
                for orth in strat['psmagrforms'].keys():
                    agr_dict = strat['psmagrforms'][orth]
                    main_choices = create_poss_strat_agr_lrt(main_choices, lrtcounter, strat, strat_name, new_pc_name, 'possessum', agr_dict, orth)
                    lrtcounter += 1
            else:
                if nonagr_lrt_exists == False: # a nonagr lrt has not been set yet
                    lrtID = new_pc_name + '_lrt' + str(lrtcounter)
                    main_choices[lrtID + '_name'] = 'lrt' + str(lrtcounter)
                    nonagr_lrt_exists = True
                    lrtcounter += 1
                # ok, at this point we know we're dealing with a nonagr psm. we need to add that to the LRT
                # set features
                featID = '_feat' + str(nonagr_strat_counter)
                main_choices[lrtID + featID + '_name'] = strat_name
                main_choices[lrtID + featID + '_value'] = 'possessum'
                main_choices[lrtID + featID + '_head'] = 'itself'
                # set orthography
                orth = strat['psm orth']
                orthID = '_lri' + str(nonagr_strat_counter)
                main_choices[lrtID + orthID + '_inflecting'] = 'yes'
                main_choices[lrtID + orthID + '_orth'] = orth
                nonagr_strat_counter += 1
                

    return main_choices

# Create a position class in the morphology section to handle possession-marking affixes
def create_possessive_pc(affixtype, main_choices, num_pcs_created, allstratdict, type):

    # Figure out how many noun-pc classes are already in the choices file so we don't override existing ones.
    last_pc_number = 0
    if 'noun-pc' in main_choices:
        existing_noun_pcs = main_choices['noun-pc']
        for pc in existing_noun_pcs:
            name = pc['name']
            number = int(name.split('noun-pc')[1])
            if number > last_pc_number:
                last_pc_number = number
    
    new_pc_number = last_pc_number + 1 + num_pcs_created
    new_pc_name = 'noun-pc' + str(new_pc_number)
    
    main_choices[new_pc_name + '_name'] = new_pc_name
    #main_choices[new_pc_name + '_obligatory'] = 'on'
    main_choices[new_pc_name + '_order'] = affixtype
    main_choices[new_pc_name + '_inputs'] = 'noun'

    #main_choices = create_nonpossession_lrt(main_choices, new_pc_name, allstratdict)
    if type == 'strat':
        main_choices = create_poss_strat_lrts(main_choices, new_pc_name, allstratdict)
    else:
        main_choices = create_poss_pron_lrts(main_choices, new_pc_name, allstratdict)

    return main_choices


def set_choices(stratlist, pronlist, main_choices):

    posschoices = MatrixChoices.ChoiceDict()

    prefix_agr_strats = {}
    prefix_nonagr_strats = {}
    suffix_agr_strats = {}
    suffix_nonagr_strats = {}
    all_prefix_strats = {}
    all_suffix_strats = {}

    # Loop through strats and set info in the adnom poss section of the choices file
    for i, strat in enumerate(stratlist):

        stratID = i+1

        strat_name = 'poss-strat' + str(stratID)

        # Set basic info
        posschoices.__setitem__(strat_name + '_order', strat['order'])
        posschoices.__setitem__(strat_name + '_mod-spec', strat['mod-spec'])
        posschoices.__setitem__(strat_name + '_mark-loc', strat['mark-loc'])


        if strat['mark-loc'] == 'possessor' or strat['mark-loc'] == 'both':
            posschoices.__setitem__(strat_name + '_possessor-type', strat['possessor-type'])

            if strat['possessor-type'] == 'affix':
                if strat['psr agr'] == True:
                    posschoices.__setitem__(strat_name + '_possessor-affix-agr', 'agree')
                    if strat['psr affix position'] == 'prefix':
                        prefix_agr_strats[strat_name] = strat
                        all_prefix_strats[strat_name] = strat
                    else:
                        suffix_agr_strats[strat_name] = strat
                        all_suffix_strats[strat_name] = strat
                else:
                    if strat['psr affix position'] == 'prefix':
                        prefix_nonagr_strats[strat_name] = strat
                        all_prefix_strats[strat_name] = strat
                    else:
                        suffix_nonagr_strats[strat_name] = strat
                        all_suffix_strats[strat_name] = strat
                    posschoices.__setitem__(strat_name + '_possessor-affix-agr', 'non-agree')
            else: # if possessor is non-affix
                posschoices.__setitem__(strat_name + '_possessor-mark-order', strat['possessor-mark-order'])
                posschoices.__setitem__(strat_name + '_pronoun-allow', 'yes')
                if strat['psr agr'] == True:
                    posschoices.__setitem__(strat_name + '_possessor-agr', 'agree')
                else:
                    posschoices.__setitem__(strat_name + '_possessor-agr', 'non-agree')
                    posschoices.__setitem__(strat_name + '_possessor-orth', strat['psr orth'])
                if strat['gencase'] == True:
                    posschoices.__setitem__(strat_name + '_feat1_name', 'case')
                    posschoices.__setitem__(strat_name + '_feat1_value', 'gen')

        if strat['mark-loc'] == 'possessum' or strat['mark-loc'] == 'both':
            posschoices.__setitem__(strat_name + '_possessum-type', strat['possessum-type'])

            if strat['possessum-type'] == 'affix':
                if strat['psm agr'] == True:
                    if strat['psm affix position'] == 'prefix':
                        prefix_agr_strats[strat_name] = strat
                        all_prefix_strats[strat_name] = strat
                    else:
                        suffix_agr_strats[strat_name] = strat
                        all_suffix_strats[strat_name] = strat
                    posschoices.__setitem__(strat_name + '_possessum-affix-agr', 'agree')
                else:
                    if strat['psm affix position'] == 'prefix':
                        prefix_nonagr_strats[strat_name] = strat
                        all_prefix_strats[strat_name] = strat
                    else:
                        suffix_nonagr_strats[strat_name] = strat
                        all_suffix_strats[strat_name] = strat
                    posschoices.__setitem__(strat_name + '_possessum-affix-agr', 'non-agree')
            else: # if possessum is non-affix
                posschoices.__setitem__(strat_name + '_possessum-mark-order', strat['possessum-mark-order'])
                if strat['psm agr'] == True:
                    posschoices.__setitem__(strat_name + '_possessum-agr', 'agree')
                else:
                    posschoices.__setitem__(strat_name + '_possessum-agr', 'non-agree')
                    posschoices.__setitem__(strat_name + '_possessum-orth', strat['psm orth']) 
        if strat['mark-loc'] == 'neither':
            posschoices.__setitem__(strat_name + '_pronoun-allow', 'yes')
        if strat['mark-loc'] == 'possessum' or strat['mark-loc'] == 'neither':
            if strat['gencase'] == True:
                posschoices.__setitem__(strat_name + '_feat1_name', 'case')
                posschoices.__setitem__(strat_name + '_feat1_value', 'gen')


    # Loop through affix strats and set info in the morphology section of the choices file
    num_pcs_created = 0
    if len(all_prefix_strats) > 0:
        main_choices = create_possessive_pc('prefix', main_choices, num_pcs_created, all_prefix_strats, 'strat')
        num_pcs_created += 1
    if len(all_suffix_strats) > 0:
        main_choices = create_possessive_pc('suffix', main_choices, num_pcs_created, all_suffix_strats, 'strat')
        num_pcs_created += 1
    
    # pron stuff

    prefix_nonagr_prons = {}
    suffix_nonagr_prons = {}
    #all_prefix_prons = {} # currently we just have nonagr prons since I'm not inferring agr prons
    #all_suffix_prons = {}

    for i, pron in enumerate(pronlist):

        pronID = i+1
        pron_name = 'poss-pron' + str(pronID)

        # Set basic info
        posschoices.__setitem__(pron_name + '_type', 'affix')
        posschoices.__setitem__(pron_name + '_agr', 'non-agree') # hard-coded for now; I don't currently infer pronominal affixes on the possessum that agree with the possessum
        posschoices.__setitem__(pron_name + '_mod-spec', pron['mod-spec'])

        if pron['psm affix position'] == 'prefix':
            prefix_nonagr_prons[pron_name] = pron
        else:
            suffix_nonagr_prons[pron_name] = pron


    # Loop through affix prons and set info in the morphology section of the choices file

    if len(prefix_nonagr_prons) > 0:
        main_choices = create_possessive_pc('prefix', main_choices, num_pcs_created, prefix_nonagr_prons, 'pron')
        num_pcs_created += 1
    if len(suffix_nonagr_prons) > 0:
        main_choices = create_possessive_pc('suffix', main_choices, num_pcs_created, suffix_nonagr_prons, 'pron')


    return posschoices, main_choices


# Determines whether an IGT demonstrates possession, specifically by looking at the tw-ds tier for the "poss" relation.
def has_possession(igt):
    has_poss = False
    tw_ds = igt.get('tw-ds')
    for rlshp in tw_ds:
        if rlshp.value() == 'poss':
            has_poss = True
    
    return has_poss

# Determines whether we have enough info to identify at least one possessor-possessum pair per IGT. 
def psr_and_psm_ided(igt):

    both_ided = False
    
    if has_possession(igt):

        # Use the tw-ds tier to get the psr and psm in the translation language
        tw_ds = igt.get('tw-ds')
        for rlshp in tw_ds:
            if rlshp.value() == 'poss':
                tw_psr = rlshp.attributes['dep']
                tw_psm = rlshp.attributes['head']
        
                # Use the bilingual tier, if it exists, to look for the psr and psm in the source language
                biling_tier = xp.find(igt, 'tier[@type="bilingual-alignments"][@target="w"]')
                w_psr = ''
                w_psm = ''
                if biling_tier:
                    for alignment in biling_tier:
                        if alignment.attributes['source'] == tw_psr:
                            w_psr = alignment.attributes['target']
                        if alignment.attributes['source'] == tw_psm:
                            w_psm = alignment.attributes['target']
                if w_psr != '' and w_psm != '':
                    ided = True
    
    return both_ided



def get_word_order(psr, psm):
    psr_number = int(psr[1:])
    psm_number = int(psm[1:])
    if psr_number < psm_number: return 'head-final'
    else: return 'head-initial'

# Returns true if the given word has an affix whose gloss indicates that it marks possession
def has_possession_marking_affix(word, igt):

    result = False
    affixID = ''

    gTier = igt.get('g')
    if gTier:
        target_segmentation = 'g' + word #produces a string like 'gw2' or 'gw3'
        for gloss in gTier:
            if 'segmentation' in gloss.attributes:
                if gloss.attributes['segmentation'] == target_segmentation:

                    poss_glosses = ['POSS', 'poss', 'Poss', 'of', '1', '2', '3', 'POS', 'pos'] 

                    for gls in poss_glosses:
                        #if gls in gloss.value().split('.'):

                        if gls in gloss.value():
                                

                            result = True
                            affixID = gloss.attributes['alignment']

    return result, affixID


def is_not_verb_or_noun(word, igt):

    wTier = igt.get('w')
    for w in wTier:

        if w.id == word:

            # first, check the POS of the word itself
            gwposTier = igt.get('gw-pos')
            if gwposTier:
                gw_alignment = w.attributes['alignment']
                for gwpos in gwposTier:
                    if gwpos.attributes['alignment'] == gw_alignment:
                        if gwpos.value() == 'VERB' or gwpos.value() == 'NOUN':
                            return False

            # we can also check the POS of the translated word
            twposTier = igt.get('tw-pos')
            if twposTier:
                biling_tier = xp.find(igt, 'tier[@type="bilingual-alignments"][@target="w"]')
                if biling_tier:
                    for biling in biling_tier:
                        if biling.attributes['target'] == word:
                            source = biling.attributes['source']
                            for twpos in twposTier:
                                if twpos.attributes['alignment'] == source:
                                    if twpos.value() == 'VERB' or twpos.value() == 'NOUN':
                                        return False

    return True

# Returns true if the given word has an immediate neighbor whose gloss indicates that it marks possession.
def has_possession_marking_clitic_or_word(word, igt, word_order, psr, psm):

    result = False
    marker_wordID = ''
    word_number = int(word[1:])
    prenumber = word_number - 1
    preneighbor = 'w' + str(prenumber)
    postnumber = word_number + 1
    postneighbor = 'w' + str(postnumber)


    poss_glosses = ['POSS', 'poss', 'Poss', 'of', '1', '2' ,'3', 'POS', 'pos']

    # check that the neighbor isn't a verb or noun, since we're assuming those aren't possession markers.
    # pronouns are trickier - I guess they could be possession markers - so I'm leaving those in for now, even though they add some noise.
    if is_not_verb_or_noun(preneighbor, igt) and is_not_verb_or_noun(postneighbor, igt):



        gwTier = igt.get('gw')
        if gwTier:

            for gw in gwTier:

                if gw.attributes['alignment'] == preneighbor and gw.attributes['alignment'] != psr and gw.attributes['alignment'] != psm:

                    if word_order == 'head-final' and word == psr: # POSS psr psm
                        for gls in poss_glosses:
                            if gls in gw.value() and '1' not in gw.value() and '2' not in gw.value(): #hacky, but it's technically true -- if the possessor is marked for person, it won't be for 1st or 2nd person -- and it unblocks South Efate. would be great to have a more elegant solution, though.
                                result = True
                                marker_wordID = preneighbor
                                pTier = igt.get('p')

                    elif word_order == 'head-initial' and word == psm: # POSS psm psr
                        for gls in poss_glosses:
                            if gls in gw.value():
                                result = True
                                marker_wordID = preneighbor
                                pTier = igt.get('p')
                    elif word_order == 'head-initial' and word == psr: # psm POSS psr
                        for gls in poss_glosses:
                            if gls in gw.value() and '1' not in gw.value() and '2' not in gw.value():
                                result = True
                                marker_wordID = preneighbor
                                pTier = igt.get('p')

                elif gw.attributes['alignment'] == postneighbor and gw.attributes['alignment'] != psr and gw.attributes['alignment'] != psm:

                    if word_order == 'head-initial' and word == psr: # psm psr POSS
                        for gls in poss_glosses:
                            if gls in gw.value() and '1' not in gw.value() and '2' not in gw.value():
                                result = True
                                marker_wordID = postneighbor
                                pTier = igt.get('p')
                    elif word_order == 'head-final' and word == psm: # psr psm POSS
                        for gls in poss_glosses:
                            if gls in gw.value():
                                result = True
                                marker_wordID = postneighbor
                                pTier = igt.get('p')
                    elif word_order == 'head-final' and word == psr: # psr POSS psm
                        for gls in poss_glosses:
                            if gls in gw.value() and '1' not in gw.value() and '2' not in gw.value():
                                result = True
                                marker_wordID = postneighbor
                                pTier = igt.get('p')


    return result, marker_wordID


# Given a word (either the possessor or possessum) and IGT, determine whether that word appears to have some sort of possession-related marking (affix, clitic, or separate word) based on the gloss of the word and its immediate surroundings.
def is_marked_for_possession(word, igt, word_order, psr, psm):


    affix_marked, affix_ID = has_possession_marking_affix(word, igt)
    if affix_marked:
        return True, 'affix', affix_ID

    nonaffix_marked, nonaffix_ID = has_possession_marking_clitic_or_word(word, igt, word_order, psr, psm)
    if nonaffix_marked:
        return True, 'non-affix', nonaffix_ID

    return False, '', ''

# return true if the possessor is not itself a possessum in some phrase
def is_not_possessed(w_psr, igt):

    twdsTier = igt.get('tw-ds')
    for twds in twdsTier:
        if twds.value() == 'poss' and twds.attributes['head'] == w_psr:
            print('found one!')
            return False

    return True


def get_mark_loc_and_type(strat, w_psr, w_psm, igt, word_order):

    psr_marked, psr_mark_type, psr_marker_ID = is_marked_for_possession(w_psr, igt, word_order, w_psr, w_psm)
    #if psr_marked and is_not_possessed(w_psr, igt):
    if psr_marked:

        strat['possessor-type'] = psr_mark_type
        strat['psr marker ID'] = psr_marker_ID

                            
        psm_marked, psm_mark_type, psm_marker_ID = is_marked_for_possession(w_psm, igt, word_order, w_psr, w_psm)
        if psm_marked:
    

            strat['mark-loc'] = 'both'
            strat['possessum-type'] = psm_mark_type
            strat['psm marker ID'] = psm_marker_ID

                        

        else:

                             
            strat['mark-loc'] = 'possessor'
                                

                        
    else:
                            
        psm_marked, psm_mark_type, psm_marker_ID = is_marked_for_possession(w_psm, igt, word_order, w_psr, w_psm)
        if psm_marked:
                        
            strat['mark-loc'] = 'possessum'
            strat['possessum-type'] = psm_mark_type
            strat['psm marker ID'] = psm_marker_ID
                            

        else: # no possession markings           

            strat['mark-loc'] = 'neither'
                            


    return strat

# Given a possession-marking morpheme/word and another target word (the possessor or possessum), update the strat to indicate whether the morpheme agrees with the other word.
def get_agreement(igt, marking_morpheme):

    gloss = ''
    morphemetype = ''

    # for affixes
    gTier = igt.get('g')
    for g in gTier:
        if g.attributes['alignment'] == marking_morpheme:
            gloss = g.value()
            morphemetype = 'affix'
    
    # for standalone words/clitics
    gwTier = igt.get('gw')
    for gw in gwTier:
        if gw.attributes['alignment'] == marking_morpheme:
            gloss = gw.value()
            morphemetype = 'word'

    agr = False
    # If the morpheme has person information, assume the morpheme is inflecting to agree with one of the possessing NPs.
    person_grams = ['1', '2', '3', 'incl', 'excl']
    for gram in person_grams:
        if gram in gloss:
            agr = True

    return agr

# Update the strat with any features on the possessor to help determine whether the possessor appears to require certain features.
def add_psr_features(strat, igt, psr):

    # First attempt: get all standalone morphemes attached to the possessor.
    # Ideas for next steps if this doesn't work well:
    # 1) also get morphemes that don't stand alone in Xigt, like "1sg.gen" rather than "1sg" and "gen"
    # 2) focus only on morphemes that are likely to indicate possession, like genitive case

    morphemes = []
    mTier = igt.get('m')
    for m in mTier:
        if m.attributes['segmentation'] == psr:
            morphemes.append(m.id)

    glosses = []
    gTier = igt.get('g')
    for g in gTier:
        if g.attributes['alignment'] in morphemes:
            glosses.append(g.value())

    strat['psr feats'] = glosses
    return strat

# Given the identifier of a word, determine based on translation gloss whether that word is a pronoun
def is_pronoun(tw_psr, igt):

    pron = False 
    pronouns = ['my', 'your', 'his', 'her', 'its', 'our', 'their', 'his/her', 'her/his', 'My', 'Your', 'His', 'Her', 'Its', 'Our', 'Their', 'His/her', 'Her/his', 'whose', 'Whose', 'which', 'Which']

    twTier = igt.get('tw')
    for tw in twTier:
        if tw.id == tw_psr and tw.value() in pronouns:
            pron = True

    return pron

# Use the bilingual tier, if it exists, to look for the psr and psm in the source language
def match_tw_to_w(igt, tw_psr, tw_psm):

    w_psr = ''
    w_psm = ''

    biling_tier = xp.find(igt, 'tier[@type="bilingual-alignments"][@target="w"]')

    if biling_tier:
        for alignment in biling_tier:
            if alignment.attributes['source'] == tw_psr:
                w_psr = alignment.attributes['target']
            if alignment.attributes['source'] == tw_psm:
                w_psm = alignment.attributes['target']

    return w_psr, w_psm


def get_orth(igt, morpheme):
    
    orth = ''

    wTier = igt.get('w')
    for w in wTier:
        if w.id == morpheme:
            orth = w.value()

    return orth

"""
INFER AGREEMENT AND ORTHOGRAPHY
"""
def infer_agreement_and_orthography(strat, igt, target, feats):

    gloss = ''
    orth = ''

    if target == 'psr':
        markerID = strat['psr marker ID']
    else:
        markerID = strat['psm marker ID']

    # affixes
    if markerID[0] == 'm':
        # orth
        mTier = igt.get('m')
        for m in mTier:
            if m.id == markerID:
                orth = m.value()
        # agreement
        gTier = igt.get('g')
        for g in gTier:
            if g.attributes['alignment'] == markerID:
                gloss = g.value()
    
    # standalone words/clitics
    else:
        if target == 'psr':
            print('')
            print('')
            print(igt.id)
        # orth
        wTier = igt.get('w')
        for w in wTier:
            if w.id == markerID:
                orth = w.value()
        # agreement
        gwTier = igt.get('gw')
        for gw in gwTier:
            if gw.attributes['alignment'] == markerID:
                gloss = gw.value()


    agr = False
    # If the morpheme has person information, assume the morpheme is inflecting to agree with the other of the possessing NPs.
    person_grams = ['1', '2', '3'] 
    agrfeats = {}
    for gram in person_grams:
        if gram in gloss:
            if gram == '3':
                gram = '3rd'
            if gram == '2':
                gram = '2nd'
            if gram == '1':
                gram = '1st'
            agrfeats['person'] = gram
            agr = True

    # Set the orthography and agreement values in the strat
    if target == 'psr':
        strat['psr orth'] = orth
        strat['psr agr'] = agr
        if agr == True:
            strat['psr agr feats'] = agrfeats
    else:
        strat['psm orth'] = orth
        strat['psm agr'] = agr
        if agr == True:
            strat['psm agr feats'] = agrfeats

    return strat



"""
INFER IF MARKER WORD IS A PREPOSITION OR POSTPOSITION
"""
def is_preposition(marker, target):

    # Non-affixes
    if int(marker[1:]) < int(target[1:]): return True
    else: return False



"""
Infer spec vs. mod
"""
def infer_spec_vs_mod(strat, igt, w_psm):

    strat['mod-spec'] = 'spec' # the adnom poss library recommends that spec be the default in the absence of strong evidence either way

    if igt.get('w-ds'):
        for rlshp in igt.get('w-ds'):
            if rlshp.value() == 'det':
                if 'dep' in rlshp.attributes and 'head' in rlshp.attributes:
                    if rlshp.attributes['head'] == w_psm:
                        strat['mod-spec'] == 'mod'
                    

    return strat






""" 


PHASE 1:
COMPILING THE INITIAL STRAT LONGLIST








Loop through all IGT to collect information about each possessive phrase. 

Input: the Xigt corpus
Output: a list of dicts. Each dict is a "strat" that represents answers to the adnom poss questionnaire for each possessive phrase that can be analyzed.

 """
def identify_strats(xc, feats):

    # List of the various possible strategies
    strats = []
    poss_pron_affixes = []


    for igt in xc:

        if not igt.get('tw-ds') or not igt.get('tw') or not igt.get('m') or not igt.get('g') or not igt.get('gw') or not igt.get('w'):
            continue

        # Only look at IGT with a single instance of possession in them. When I used to allow multiple instances of possession, the nested possession phrases
        # (e.g. "my father's brother") made things noisy. In future it would be great to relax this constraint to be able to look at more examples.
        #poss_counter = 0
        
        #for rlshp in igt.get('tw-ds'):

       #     if rlshp.value() == 'poss':
        #        poss_counter += 1

        #if poss_counter == 1:

        for rlshp in igt.get('tw-ds'):
        

            if rlshp.value() == 'poss':

                w_psr, w_psm = match_tw_to_w(igt, rlshp.attributes['dep'], rlshp.attributes['head'])
            
                if w_psr != '' and w_psm != '':

                    if not is_pronoun(rlshp.attributes['dep'], igt):

                        strat = {}
                        
                        

                        """ 
                        INFERRING WORD ORDER
                        """
                        #Questionnaire:
                        #The word order within a possessive phrase is:
                        #    - possessor-first
                        #    - possessum-first
                        #    - either
                        #Since we're only dealing with one phrase at a time, the answer for each phrase can only be "possessor-first" or "possessum-first". It's only later, when we consolidate this strat with the others, that we can determine if the language's overall word order for the strat is "either".
                        #Possible values:
                        #order={head-final, head-initial, either}

                        word_order = get_word_order(w_psr, w_psm)
                        strat['order'] = word_order


                        """ 
                        INFERRING MARK LOC and PSR/PSM TYPE
                        """
                        strat = get_mark_loc_and_type(strat, w_psr, w_psm, igt, word_order)


                        """
                        Inferring spec vs. mod
                        """
                        strat = infer_spec_vs_mod(strat, igt, w_psm)


                        """
                        INFERRING AGREEMENT and ORTHOGRAPHY
                        """
                        if strat['mark-loc'] == 'possessor' or strat['mark-loc'] == 'both':
                            strat = infer_agreement_and_orthography(strat, igt, 'psr', feats)
                        if strat['mark-loc'] == 'possessum' or strat['mark-loc'] == 'both':
                            strat = infer_agreement_and_orthography(strat, igt, 'psm', feats)


                        """
                        INFERRING MARK ORDER
                        """
                        # Non-affixes
                        if strat['mark-loc'] == 'possessor' or strat['mark-loc'] == 'both':
                            if strat['possessor-type'] == 'non-affix':
                                if is_preposition(strat['psr marker ID'], w_psr):
                                    strat['possessor-mark-order'] = 'head-initial'
                                else:
                                    strat['possessor-mark-order'] = 'head-final'
                        if strat['mark-loc'] == 'possessum' or strat['mark-loc'] == 'both':
                            if strat['possessum-type'] == 'non-affix':
                                if is_preposition(strat['psm marker ID'], w_psm):
                                    strat['possessum-mark-order'] = 'head-initial'
                                else:
                                    strat['possessum-mark-order'] = 'head-final'

                        # Affixes
                        # This is reaaaaally not a good way of doing this, but I think doing it right will involve diving back into INTENT and trying to identify the root of the word in a way that's kind of tricky, so I'm trying a quick and dirty heuristic for now... basically, if there's only one hyphen in the orthography, we've got a clear indication of whether it's a prefix or a suffix. If there are two or zero hyphens, we don't have enough info, and I'm going to arbitrarily put prefix as a default for now.
                        if strat['mark-loc'] == 'possessor' or strat['mark-loc'] == 'both':
                            if strat['possessor-type'] == 'affix':
                                if strat['psr orth'][-1] == '-':
                                    strat['possessor-mark-order'] = 'head-initial'
                                else:
                                    strat['possessor-mark-order'] = 'head-final'
                        if strat['mark-loc'] == 'possessum' or strat['mark-loc'] == 'both':
                            if strat['possessum-type'] == 'affix':
                                if strat['psm orth'][-1] == '-':
                                    strat['possessum-mark-order'] = 'head-initial'
                                else:
                                    strat['possessum-mark-order'] = 'head-final'



                        """
                        INFERRING FEATURES ON THE POSSESSOR
                        """
                        # In some languages, the possessor must carry a feature that is not specific to possession. The most prominent version of this is marking the possessor with genitive case.
                        # This function collects all of the features on the possessor in every situation except when mark loc == psr. (This logic is in accordance with the matrix questionnaire.)
                        # Later, during the consolidation phase, we'll try to figure out which of these features are actual constraints.

                        if strat['mark-loc'] == 'possessor' or strat['mark-loc'] == 'both':
                            if strat['possessor-type'] == 'non-affix':
                                strat = add_psr_features(strat, igt, w_psr)
                        if strat['mark-loc'] == 'possessum' or strat['mark-loc'] == 'neither':
                                strat = add_psr_features(strat, igt, w_psr)



                        strats.append(strat)

                # if there is no identified overt possessor, we may be dealing with a possessum that has a pronominal affix
                elif w_psm != '':

                    affix_marked, affixID = has_possession_marking_affix(w_psm, igt)

                    if affix_marked:

                        poss_pron = {}

                        poss_pron['type'] = 'affix'
                        poss_pron['psm marker ID'] = affixID

                        poss_pron = infer_spec_vs_mod(poss_pron, igt, w_psm)

                        poss_pron = infer_agreement_and_orthography(poss_pron, igt, 'psm', feats)

                        if poss_pron['psm orth'][-1] == '-':
                            poss_pron['possessum-mark-order'] = 'head-initial'
                            poss_pron['order'] = 'head-final'
                        else:
                            poss_pron['possessum-mark-order'] = 'head-final'
                            poss_pron['order'] = 'head-initial'

                        poss_pron_affixes.append(poss_pron)

    return(strats, poss_pron_affixes)


def count_affix_nonaffix(strats, psr, psm):

    affixcount = 0
    nonaffixcount = 0

    for strat in strats:
        if psr == True:
            if strat['possessor-type'] == 'affix':
                affixcount += 1
            else:
                nonaffixcount += 1
        if psm == True:
            if strat['possessum-type'] == 'affix':
                affixcount +=1
            else:
                nonaffixcount += 1

    return affixcount, nonaffixcount


def gencase_required(stratlist):

    totalstrats = 0
    stratswithgen = 0
    gen_grams = ['gen', 'GEN', 'Gen', 'genitive']

    for strat in stratlist:
        totalstrats += 1
        for psrfeat in strat['psr feats']:
            if psrfeat in gen_grams:
                stratswithgen += 1

    if stratswithgen/totalstrats >= .5: return True
    else: return False

# Given a list of non-agreeing strats, return the most common orthography for the marker.
def identify_most_common_orth(stratlist, markloc):

    possible_orths = {}

    for strat in stratlist:
        if markloc == 'psr':
            current_orth = strat['psr orth']
            if current_orth in possible_orths:
                possible_orths[current_orth] += 1
            else:
                possible_orths[current_orth] = 1
        else:
            current_orth = strat['psm orth']
            if current_orth in possible_orths:
                possible_orths[current_orth] += 1
            else:
                possible_orths[current_orth] = 1

    return max(possible_orths, key=possible_orths.get)

# Take a list of similar strats and return a consolidated mark order value
def get_mark_word_order(stratlist, markloc, fixed):
    hi_count = 0
    hf_count = 0

    if markloc == 'psr':
        for strat in stratlist:
            if strat['possessor-mark-order'] == 'head-initial':
                hi_count += 1
            else:
                hf_count += 1
    else:
        for strat in stratlist:
            if strat['possessum-mark-order'] == 'head-initial':
                hi_count += 1
            else:
                hf_count += 1

    if fixed:
        if hi_count > hf_count:
            return 'head-initial'
        else:
            return 'head-final'
    else:
        if hi_count > 0:
            if hf_count > 0:
                return 'either'
            else:
                return 'head-initial'
        else:
            return 'head-final'
            

# Take a list of similar strats and return a consolidated word order value ('psr-psm', 'psm-psr', or 'either')
def get_phrase_word_order(stratlist, fixed_wo):

    psr_first_count = 0
    psm_first_count = 0

    for strat in stratlist:
        if strat['order'] == 'head-final':
            psr_first_count += 1
        else:
            psm_first_count += 1
    
    if fixed_wo:
        if psr_first_count > psm_first_count:
            return 'head-final'
        else:
            return 'head-initial'
    else:
        if psr_first_count > 0:
            if psm_first_count > 0:
                return 'either'
            else:
                return 'head-final'
        else:
            return 'head-initial'

# Add appropriate agreement features to a consolidated strat.
def add_agr_feats(stratlist, markloc):

    agrforms = {}
    featstr = markloc + ' agr feats'
    orthstr = markloc + ' orth'

    for strat in stratlist:
        if featstr in strat:
            agr_feats = strat[featstr]
            orth = strat[orthstr]
            agrforms[orth] = agr_feats

    return agrforms

def consol_spec_or_mod(stratlist):

    totalstratcounter = 0
    totalspeccounter = 0

    for strat in stratlist:
        totalstratcounter += 1
        if strat['mod-spec'] == 'spec':
            totalspeccounter += 1
    
    if totalspeccounter / totalstratcounter >= .5:
        return 'spec'
    else: return 'mod'

# Take a list of similar strats and return a single consolidated strat
def consolidate(stratlist, markloc, psraffix, psragr, psmaffix, psmagr):

    consolidated_strat = {}
    consolidated_strat['mark-loc'] = markloc

    #spec vs mod
    consolidated_strat['mod-spec'] = consol_spec_or_mod(stratlist)

    if markloc == 'possessor' or markloc == 'both':
        if psraffix == False:
            consolidated_strat['possessor-type'] = 'non-affix'
            consolidated_strat['possessor-mark-order'] = get_mark_word_order(stratlist, 'psr', fixed=False)     
            consolidated_strat['gencase'] = gencase_required(stratlist)          
        else:
            consolidated_strat['possessor-type'] = 'affix'
            if get_mark_word_order(stratlist, 'psr', fixed=True) == 'head-initial':
                consolidated_strat['psr affix position'] = 'prefix'
            else:
                consolidated_strat['psr affix position'] = 'suffix'
        if psragr == False:
            consolidated_strat['psr orth'] = identify_most_common_orth(stratlist, 'psr')
            consolidated_strat['psr agr'] = False
        else:
            consolidated_strat['psragrforms'] = add_agr_feats(stratlist, 'psr')
            consolidated_strat['psr agr'] = True
            

    if markloc == 'possessum' or markloc == 'both':
        if psmaffix == False:
            consolidated_strat['possessum-type'] = 'non-affix'
            consolidated_strat['possessum-mark-order'] = get_mark_word_order(stratlist, 'psm', fixed=False)
        else:
            consolidated_strat['possessum-type'] = 'affix'
            if get_mark_word_order(stratlist, 'psm', fixed=True) == 'head-initial':
                consolidated_strat['psm affix position'] = 'prefix'
            else:
                consolidated_strat['psm affix position'] = 'suffix'
        if psmagr == False:
            consolidated_strat['psm orth'] = identify_most_common_orth(stratlist, 'psm')
            consolidated_strat['psm agr'] = False
        else:
            consolidated_strat['psmagrforms'] = add_agr_feats(stratlist, 'psm')
            consolidated_strat['psm agr'] = True
    if markloc == 'possessum' or markloc == 'neither':
        consolidated_strat['gencase'] = gencase_required(stratlist)
    
    # Get phrase word order
    if markloc == 'neither' and consolidated_strat['gencase'] == False:
        consolidated_strat['order'] = get_phrase_word_order(stratlist, fixed_wo=True)
    else:
        consolidated_strat['order'] = get_phrase_word_order(stratlist, fixed_wo=False)

    return consolidated_strat



# Take a list of affix poss-pron strats and return consolidated strats
def consolidate_pron(stratlist):

    consolidated_list = []


    consolidated_strat = {}

    consolidated_strat['mod-spec'] = consol_spec_or_mod(stratlist)

    consolidated_strat['type'] = 'affix'

    consolidated_strat['psmagrforms'] = add_agr_feats(stratlist, 'psm')
    consolidated_strat['psm agr'] = True

    if get_mark_word_order(stratlist, 'psm', fixed=True) == 'head-initial':
        consolidated_strat['psm affix position'] = 'prefix'
    else:
        consolidated_strat['psm affix position'] = 'suffix'

    consolidated_list.append(consolidated_strat)

    return consolidated_list



"""



Phase 2:
CONSOLIDATING THE STRAT LIST

Given a longlist of strats, consolidate them to get rid of duplicates and produce a shortlist.






"""

def consolidate_strats(longlist, minimum_strat_examples_required):

    shortlist = []

    psrlist = list(filter(lambda x: x['mark-loc'] is 'possessor',longlist))
    psmlist = list(filter(lambda x: x['mark-loc'] is 'possessum',longlist))
    bothlist = list(filter(lambda x: x['mark-loc'] is 'both',longlist))
    neitherlist = list(filter(lambda x: x['mark-loc'] is 'neither',longlist))


    """
    CONSOLIDATING PSR-MARKED STRATS
    """
    # First, we separate the psr-marking strats into affix and nonaffix (word or clitic).
    if len(psrlist) >= minimum_strat_examples_required:
        psraffixagrlist = list(filter(lambda x: x['possessor-type'] == 'affix' and x['psr agr'] == True,psrlist))
        psraffixnonagrlist = list(filter(lambda x: x['possessor-type'] == 'affix' and x['psr agr'] == False,psrlist))
        psrnonaffixagrlist = list(filter(lambda x: x['possessor-type'] == 'non-affix' and x['psr agr'] == True,psrlist))
        psrnonaffixnonagrlist = list(filter(lambda x: x['possessor-type'] == 'non-affix' and x['psr agr'] == False,psrlist))

        if len(psraffixagrlist) >= minimum_strat_examples_required:
            shortlist.append(consolidate(psraffixagrlist, 'possessor', psraffix=True, psragr=True, psmaffix=False, psmagr=False))
    
        if len(psraffixnonagrlist) >= minimum_strat_examples_required:
            shortlist.append(consolidate(psraffixnonagrlist, 'possessor', psraffix=True, psragr=False, psmaffix=False, psmagr=False))
    
        if len(psrnonaffixagrlist) >= minimum_strat_examples_required:
            shortlist.append(consolidate(psrnonaffixagrlist, 'possessor', psraffix=False, psragr=True, psmaffix=False, psmagr=False))

        if len(psrnonaffixnonagrlist) >= minimum_strat_examples_required:
            shortlist.append(consolidate(psrnonaffixnonagrlist, 'possessor', False, False, False, False))


    """
    CONSOLIDATING PSM-MARKED STRATS
    """
    # Similar to possessor-marked strats.
    # First, let's divide the strats into affix and nonaffix.

    if len(psmlist) >= minimum_strat_examples_required:
        possible_affix_vals = ['affix', 'non-affix']
        possible_agr_vals = [True, False]

        for affix_val in possible_affix_vals:
            for agr_val in possible_agr_vals:
                stratlist = list(filter(lambda x: x['possessum-type'] == affix_val and x['psm agr'] == agr_val,psmlist))

                if len(stratlist) >= minimum_strat_examples_required:
                    shortlist.append(consolidate(stratlist, 'possessum', False, False, affix_val, agr_val))

    """
    CONSOLIDATING "BOTH" STRATS
    """
    if len(bothlist) >= minimum_strat_examples_required:
        possible_affix_vals = ['affix', 'non-affix']
        possible_agr_vals = [True, False]

        for psr_affix_val in possible_affix_vals:
            for psr_agr_val in possible_agr_vals:
                for psm_affix_val in possible_affix_vals:
                    for psm_agr_val in possible_agr_vals:
                        stratlist = list(filter(lambda x: x['possessor-type'] == psr_affix_val and x['psr agr'] == psr_agr_val and x['possessum-type'] == psm_affix_val and x['psm agr'] == psm_agr_val,bothlist))


                        if len(stratlist) >= minimum_strat_examples_required:
                            shortlist.append(consolidate(stratlist, 'both', psr_affix_val, psr_agr_val, psm_affix_val, psm_agr_val))
    
    """
    CONSOLIDATING "NEITHER" STRATS
    """
    # Currently I allow each language to have up to only one neither strat for simplicity's sake. I suppose it's possible that we might want multiple in the future, but I can't think of a strong use case right now.
    # The relevant questions are:
    # 1) Does the neither strat appear to require the psr to have genitive case? (i.e. is the psr marked for genitive case in over half of the neither strats?)
    # 2) What is the word order of the neither strat?
    #   2a) If we've determined that it must be marked for genitive case, and we see examples of both word orders, put "either".
    #   2b) If the strat does NOT require genitive case marking, we're dealing with simple juxtaposition and cannot put "either" word order because then there'd be nothing to syntactically distinguish the psr from the psm. In this situation, pick whichever is the more common word order.
    if len(neitherlist) >= minimum_strat_examples_required:
        shortlist.append(consolidate(neitherlist, 'neither', psraffix=False, psragr=False, psmaffix=False, psmagr=False))


    return shortlist


"""
MAIN METHOD
"""
def infer_adnom_poss(xc, feats, main_choices):


    """
    Step 1 of 3: Collect strat information from data
    """
    strat_longlist, poss_pron_affix_longlist = identify_strats(xc, feats)


    """
    Step 2 of 3: Consolidate strats

    Takes the exhaustive list of strategies and consolidates them into a shorter de-duplicated list. 
    If minimum_strat_examples_required is 1, any (de-duplicated) strat that shows up in the data at least once will remain in the final list. Set the minimum higher to require more evidence of the strat.
    """
    mincounter = 3
    strat_shortlist = []
    while mincounter != 0 and len(strat_shortlist) == 0:
        strat_shortlist = consolidate_strats(strat_longlist, minimum_strat_examples_required=mincounter)
        mincounter -= 1

    minproncounter = 3
    poss_pron_affix_shortlist = []
    while minproncounter != 0 and len(poss_pron_affix_shortlist) == 0:
        if len(poss_pron_affix_longlist) >= minproncounter:
            poss_pron_affix_shortlist = consolidate_pron(poss_pron_affix_longlist)
        minproncounter -= 1

    # print(poss_pron_affix_shortlist)



    # """
    # Step 3 of 3: Set choices using the consolidated strat list
    # """
    choices, main_choices = set_choices(strat_shortlist, poss_pron_affix_shortlist, main_choices)

    return choices, main_choices