#!/usr/bin/python3
# -*- coding: utf-8 -*

import sys
import os
import re
from collections import defaultdict, Counter
from xigt import xigtpath as xp
from xigt import ref
from xigt.codecs import xigtxml


def grams(node):
    return [gram for tok in token_re.findall(node)
            for gram in gram_punc_re.sub(' ', tok).split()
            if len(gram) > 1 and gram.isupper()]

def infer_case(xc):
    instances = {'trans':[], 'intrans':[]}
    arggrams = Counter()
    vgrams = Counter()

    s_inst = 0
    a_inst = 0
    o_inst = 0

    igt_count = 0
    dep_count = 0
    subj_dep_count = 0
    obj_dep_count = 0
    for igt in xc:
        igt_count += 1
        if igt.get('g') is None:
            continue
        gloss_ids = set(i.id for i in igt.get('gw')) #if igt.get('gw') else set() #need to be able to use 'g'>'m'>'w' here instead.
        trs = xp.findall(igt, 'tier[@type="dependencies"][@dep="w"]')
        for tier in trs:
            dep_count += 1
            subj_grams = []
            obj_grams = []
            for item in tier:
                if item.value() == 'nsubj':
                    subj_dep_count += 1
                    subj_item = igt.get_item(item.attributes['dep'])
                    referrers = ref.referrers(igt, subj_item.id, ['alignment'])['alignment']
                    glosses = [r for r in referrers if r in gloss_ids]
                    for each in glosses:
                        subj_gloss_word = igt.get_item(each).value().lower()
                        subj_grams = re.split('-|\.|\(\)|:|\/', subj_gloss_word)
                elif item.value() == 'dobj':
                    obj_dep_count += 1
                    obj_item = igt.get_item(item.attributes['dep'])
                    referrers = ref.referrers(igt, obj_item.id, ['alignment'])['alignment']
                    glosses = [r for r in referrers if r in gloss_ids]
                    for each in glosses:
                        obj_gloss_word = igt.get_item(each).value().lower()
                        obj_grams = re.split('-|\.|\(\)|:|\/', obj_gloss_word)
                        
                #this is just to match the expected form from the old code
                inst = ([], subj_grams, obj_grams, '', '')
        
                vgrams.update(inst[0])
                arggrams.update(inst[1])
                if obj_grams == [] and subj_grams != []:
                    instances['intrans'] += [inst]
                    s_inst += 1
                elif subj_grams == [] and obj_grams != []:
                    instances['trans'] += [inst]
                    arggrams.update(inst[2])
                    o_inst += 1
                elif subj_grams != [] and obj_grams != []:
                    instances['trans'] += [inst]
                    arggrams.update(inst[2])
                    a_inst += 1
                    o_inst += 1



    # Only keep vgrams that occur in 2% or more of the instances
    thresh = (len(instances['trans']) + len(instances['intrans']))/50
    sthresh = s_inst / 50
    if sthresh < 2: sthresh = 2
    othresh = o_inst / 50
    if othresh < 2: othresh = 2
    athresh = a_inst / 50
    if athresh < 2: athresh = 2


    s_counts = Counter()
    a_counts = Counter()
    o_counts = Counter()
    for inst in instances['intrans']:
        s_counts.update(inst[1])
    for inst in instances['trans']:
        a_counts.update(inst[1])
        o_counts.update(inst[2])

    # first filter out grams occuring less than the threshold (or less than 2)
    for g in arggrams.keys():
        if s_counts[g] < sthresh: del s_counts[g]
        if a_counts[g] < athresh: del a_counts[g]
        if o_counts[g] < othresh: del o_counts[g]

    # # then filter out grams occuring in each category more than the threshold
    for g in arggrams.keys():
        if s_counts[g] >= sthresh and \
            a_counts[g] >= athresh and \
            o_counts[g] >= othresh:
            del s_counts[g]
            del a_counts[g]
            del o_counts[g]
    # 


    s = s_counts.most_common()[:1] or None
    a = a_counts.most_common()[:1] or None
    o = o_counts.most_common()[:1] or None

    case = 'none'

    if not s or s == None:
        s = [('no-case',sthresh+10)]

    if not a or a == None:
        a = [('no-case',athresh+10)]

    if not o or o == None:
        o = [('no-case',othresh+10)]


    if s and a and o:
        s = s[0][0]
        a = a[0][0]
        o = o[0][0]
        if s == a and s != o and s != 'no-case': case = 'nom-acc'
        elif s == o and s != a and s != 'no-case': case = 'erg-abs'
        elif a_counts[s] < athresh and o_counts[s] < othresh and\
            s_counts[a] < sthresh and o_counts[a] < othresh and\
            s_counts[o] < sthresh and a_counts[o] < athresh and\
            s != 'no-case' and a != 'no-case' and o != 'no-case':### added so that if there are no case grams, it goes to none, not tripartite
            case = 'tripartite'
        elif s != a and s != o and a != o and \
            s_counts[a] > sthresh and s_counts[o] > sthresh:
            case = 'split-s'
        # split-n?
        # split-v?

    #sys.stdout.write(os.path.basename(file_name)[:3] + '\t' + 'case-marking: \t {}'.format(case) + '\n')
    sys.stdout.write('case-marking: \t {}'.format(case) + '\n')
    sys.stdout.write('s: ' + s + ' a: ' + a + ' o: ' + o + '\n')
    sao_output = {'s': s, 'a': a, 'o': o, 'case': case}
    return sao_output

if __name__ == '__main__':
    file_name = sys.argv[1]
    infer_case(file_name)