#!/usr/bin/env python3
# -*- coding: utf-8 -*
# Author: Kriten Howell

# helper functions for person inference from a set of features provided by mom
from XigtifiedToolbox import FeatDict

def person(features):
    person = 'none'
    if 'person' in features:
        if FeatDict.FOURTH in features['person']:
            person = '1-2-3-4'
        elif FeatDict.THIRD in features['person'] and (FeatDict.SECOND in features['person'] \
                    or FeatDict.FIRST in features['person'] or FeatDict.FIRSTINCL in features['person'] \
                    or FeatDict.FIRSTEXCL in features['person']):
            person = '1-2-3'
        elif FeatDict.FIRST in features['person'] or FeatDict.FIRSTINCL in features['person'] \
                    or FeatDict.FIRSTEXCL in features['person']:
            person = '1-non-1'
        elif FeatDict.SECOND in features['person']:
            person = '2-non-2'
        elif FeatDict.THIRD in features['person']:
            person = '3-non-3'
    return(person)

def first_person(features):
    first_per_choices = 'none', 'none'
    if 'pernum' in features:
        first_person = 'incl-excl'
        # right now, we are just saying that the inclusive/exclusive disctinction is possible for all numbers, except singular
        nums = []
        for num in features['number']:
            if num != FeatDict.SINGULAR:
                nums.append(num)

        first_per_choices = first_person, ", ".join(nums)
    return(first_per_choices)

def consolidate_pernum_feats(item, choices):
    ## the goal of this method is to standardize to the pernum heirarchy that the matrix uses
    pernum_feats = {'self':{'person': set(), 'number': set()},'subj':{'person': set(), 'number': set()}, 'obj':{'person': set(), 'number': set()}}
    feat_keys = []
    new_feats = []
    for feat in item['feat']:
        if feat['name'] == 'person':
            if 'noun' in item.full_key or 'det' in item.full_key:
                pernum_feats['self']['person'].add(feat['value'])
            elif 'verb' in item.full_key or 'aux' in item.full_key:#auxes don't have features now, but might eventually
                pernum_feats[feat['head']]['person'].add(feat['value'])
            if item.full_key not in feat.full_key:
                feat_keys.append(item.full_key+'_'+feat.full_key)
            else:
                feat_keys.append(feat.full_key)
        if feat['name'] == 'number':
            if 'noun' in item.full_key or 'det' in item.full_key:
                pernum_feats['self']['number'].add(feat['value'])
            elif 'verb' in item.full_key or 'aux' in item.full_key:#auxes don't have features now, but might eventually
                pernum_feats[feat['head']]['number'].add(feat['value'])
            if item.full_key not in feat.full_key:
                feat_keys.append(item.full_key+'_'+feat.full_key)
            else:
                feat_keys.append(feat.full_key)
        if feat['name'] == 'pernum':
            new_feats.append(feat['value'])
            if item.full_key not in feat.full_key:
                feat_keys.append(item.full_key+'_'+feat.full_key)
            else:
                feat_keys.append(feat.full_key)
    for head in ['self','obj','subj']:
        if pernum_feats[head]['person']:
            for per_feat in pernum_feats[head]['person']:
                if pernum_feats[head]['number']:
                    for num_feat in pernum_feats[head]['number']:
                        full_feat = ''
                        if 'non' in per_feat:
                            full_feat = 'n'+num_feat
                        if '1' in per_feat:
                            full_feat = '1'+num_feat
                        if '2' in per_feat:
                            full_feat = '2'+num_feat
                        if '3' in per_feat:
                            full_feat = '3'+num_feat
                        if '4' in per_feat:
                            full_feat = '4'+num_feat
                        if 'incl' in per_feat:
                            full_feat = full_feat + '_incl'
                        if 'excl' in per_feat:
                            full_feat = full_feat + '_excl'
                        if full_feat != '':
                            new_feats.append(full_feat)
                else:
                    for per_feat in pernum_feats[head]['person']:
                        new_feats.append(per_feat)
        elif pernum_feats[head]['number']:
            for num_feat in pernum_feats[head]['number']:
                new_feats.append(num_feat)
    if new_feats:
        choices[feat_keys[0]+'_name'] = 'pernum'
        choices[feat_keys[0]+'_value'] = ', '.join(new_feats)
        if len(feat_keys) > 0:
            for key in feat_keys[1:]:
                del choices[key]


def update_pernum_feats(choices):
    pos_list = ['verb-pc', 'noun-pc']
    for pos in pos_list:
        for pc in choices[pos]:
            for lrt in pc['lrt']:
                consolidate_pernum_feats(lrt, choices)
                # for feat in lrt['feat']:
                #     if feat['name'] == 'person' or feat['name'] == 'number':
                #         feat['name'] = 'pernum'
                # if len(lrt['feat']) > 1:
                #     pernum_feats = [feat for feat in lrt['feat'] if feat['name'] == 'pernum']
                #     if len(pernum_feats) > 1:
                #         i = 1
                #         while i < len(pernum_feats):
                #             pernum_feats[0]['value'] = pernum_feats[0]['value'] + ', '+pernum_feats[i]['value']
                #             i += 1
                #         for pernum_feat in pernum_feats[1:]:
                #             del choices[pernum_feat.full_key]

    pos_list = ['verb', 'noun', 'aux', 'det']
    for pos in pos_list:
        if pos in choices:
            for word in choices[pos]:
                consolidate_pernum_feats(word, choices)
                # for feat in word['feat']:
                #     if feat['name'] == 'person' or feat['name'] == 'number':
                #         feat['name'] = 'pernum'
                # if len(word['feat']) > 1:
                #     pernum_feats = [feat for feat in word['feat'] if feat['name'] == 'pernum']
                #     if len(pernum_feats) > 1:
                #         i = 1
                #         while i < len(pernum_feats):
                #             pernum_feats[0]['value'] = pernum_feats[0]['value'] + ', ' + pernum_feats[i]['value']
                #             i += 1
                #         for pernum_feat in pernum_feats[1:]:
                #             del word[pernum_feat.full_key]
    return choices
