#!/opt/python-3.4/bin/python3.4
# -*- coding: utf-8 -*-

import sys
from nltk import *
import nltk_tgrep
from xigt.codecs import xigtxml
from aggutils.xigt_nltk_tree import *

# Step through Xigt corpus and create array of NLTK ParentedTree objects, one per item.
# FIXME: Check first that each item actually has a syntax tier.

NOUNS = ['n', 'pro', 'NOUN', 'PRP', 'NN', 'NP', 'NNS', 'NNP', 'NNPS', 'PRON',
         'WP']  # KPH: removed PRP$-- this is for posessive pronouns, added WP (wh-pronouns)
VERBS = ['vi', 'vt', 'VBG', 'VBP', 'VP', 'VBN', 'VB', 'VBD', 'VBZ', 'VERB']

def get_wo_counts(xc):
    word_order_counts = build_word_order_dict()
    for igt in xc:
        up_tree, down_tree, vps, root = create_trees(igt)
        #traverse tree: find constituents
        subj, verb, obj = find_constituents(up_tree, down_tree, vps, root, igt)
        #get constituent order
        order = get_constituent_order(tree)
        word_order_counts = add_to_wo_counts(word_order_counts, order)
    return order



def find_constituents(up_tree, down_tree, vps, root, igt):
    subj = ''
    obj = ''
    verb = ''
    trans_subj = ''
    trans_obj = ''
    trans_verb = ''
    trans_nouns = get_translation_nouns(igt)
    trans_verbs = get_translation_verbs(igt)
    #look

    return subj, verb, obj

def get_constituent_order(constituents):
    return order

def create_trees(igt):
    # From VerbValenceFromIGT.py
    # Creates dictionaries useful for crawling up the phrase structure tree, or down the tree
    # Takes a phrase structure tier element (Xigt)
    # Returns a tuple containing:
    # - dictionary for reading up
    # - dictionary for reading down
    # - list of the verb phrase nodes
    # - reference to the root element of the phrase structure tree
    up_tree = {}
    down_tree = {}
    vps = []
    root = False
    translation_nouns = get_translation_nouns(igt)
    for ps in igt.get('tw-ps'):
        # These are the nodes for the words themselves
        if ps.alignment:
            up_tree[ps.alignment] = ps.id
            down_tree[ps.id] = ps.alignment
        # These are the intermediate nodes
        else:
            for child in ps.get_attribute('children').split(' '):
                up_tree[child] = ps.id
            down_tree[ps.id] = ps.get_attribute('children').split(' ')
            if ps.get_content() == 'VP':
                vps.append(ps.id)
            elif ps.get_content() in NOUNS:
                translation_nouns.append(ps.id)
            elif ps.get_content() == 'ROOT':
                root = ps.id

    return (up_tree, down_tree, vps, root)

def get_translation_nouns(igt):
    tw_ps_tier = igt.get('tw-ps')
    translation_nouns = []
    for pos in tw_ps_tier:
        if pos.alignment and pos.get_content() in NOUNS:
            translation_nouns.append(pos.alignment)
    return translation_nouns

def get_translation_verbs(igt):
    # Go through all nodes of the translation phrase structure tree to get a list of the nouns and verbs
    # that will be needed in the next step to find transitivity
    tw_ps_tier = igt.get('tw-ps')
    translation_verbs = []
    if not tw_ps_tier:
        return translation_verbs
    for pos in tw_ps_tier:
        if pos.alignment and pos.get_content() in VERBS:
            translation_verbs.append(pos.alignment)
    return translation_verbs

def build_word_order_dict():
    orders = {}
    orders['svo'] = 0
    orders['sov'] = 0
    orders['vos'] = 0
    orders['ovs'] = 0
    orders['vso'] = 0
    orders['osv'] = 0
    orders['sv'] = 0
    orders['vs'] = 0
    orders['ov'] = 0
    orders['vo'] = 0
    orders['v'] = 0
    orders['det_n'] = 0
    orders['n_det'] = 0
    orders['adj_n'] = 0
    orders['n_adj'] = 0
    return orders

def add_to_wo_counts(word_order_counts, order):
    return word_order_counts


if __name__ == '__main__':
    file_name = sys.argv[1]
    file = open(file_name, encoding='utf-8')
    xc = xigtxml.load(file)
    file.close()
    orders = get_wo_counts(xc)