#!/opt/python-3.4/bin/python3.4
# -*- coding: utf-8 -*-
__author__ = 'David Inman'
# Revision to incorporate into extract_choices made by Kristen Howell

import sys, os
#sys.path.append("/../xigt/xigt")
from xigt.codecs import xigtxml

DATA_567 = "../../data/567/"
OUTFOLDER = "../../data/ling575sp15/aux/out/"
DEV1 = "dev1/"
DEV2 = "dev2/"
TEST = "test/"
LEAFNAME = "testsuite-enriched.xml"
OUTFILE = "aux.txt"


ANG = DATA_567 + DEV1 + "ang/" + LEAFNAME
MNK = DATA_567 + DEV1 + "mnk/" + LEAFNAME
NAN = DATA_567 + DEV1 + "nan/" + LEAFNAME
SNA = DATA_567 + DEV1 + "sna/" + LEAFNAME
JPN = DATA_567 + DEV1 + "jpn/" + LEAFNAME
PBT = DATA_567 + DEV1 + "pbt/" + LEAFNAME
RUS = DATA_567 + DEV1 + "rus/" + LEAFNAME
NAN = DATA_567 + DEV1 + "nan/" + LEAFNAME
CYM = DATA_567 + DEV2 + "cym/" + LEAFNAME
QUB = DATA_567 + DEV2 + "qub/" + LEAFNAME
SCI = DATA_567 + DEV2 + "sci/" + LEAFNAME
FXN = ['ANTIP', 'APPL', 'BEN', 'CAUS', 'COMPL', 'DECL', 'DUR', 'FUT', 'IMP', 'IND', 'INF', 'INTR', 'IMPV', 'IMPF', 'IRR',
       'PASS', 'PFV', 'PERF', 'PRF', 'PRS', 'PRES', 'PROG', 'PROH', 'PST', 'PAST', 'PTCP', 'PURP', 'SBJV', 'TR',
       'HEARSAY', 'PERSONAL', 'RESULTATIVE', 'CONTINUATIVE', 'ABLE', 'ABIL']
MOD = ['can', 'may', 'might', 'should', 'would', 'could', 'do', 'does', 'did', 'have', 'has', 'had'] #HAVE???
COP = ['COP', 'be']
INF = 'INF'
# what about neg, quot, q?
#what about can, would, modals, be, COP?
PUNC = ['.', ',', '?', '!']
#for i in range(len(dev1_langs)):
#    print (dev1_langs[i] + ' ' + str(auxes[i]))

#looking for AUXes
#CASE1: there is "AUX" in the gloss!
#CASE2: A word consisting solely of grammatical glosses
def find_auxes(xc):
    auxes = set()
    cops = set()
    copLoc = {'left': 0, 'right': 0} #there should be all one or the other, or a clear majority(? I hope...)
    for igt in xc:
        #have to do this for some badly formatted data in qub
        if 'w-pos' not in [tier.id for tier in igt.tiers]:
            print ("cannot find w-pos; this is bad for " + igt.id)
            continue
        glosses = igt['g']
        words   = igt['w']
        morphs  = igt['m']
        pos     = igt['w-pos']
        wordsFxnOnly = [True] * len(words)
        for gloss in glosses:
            if gloss.get_content() == 'AUX': #we're done
                #print ("words " + ' '.join([w.get_content() for w in words]))
                #print ("morphs " + ' '.join([m.get_content() for m in morphs]))
                #print (gloss.get_content() + " " + str(gloss.alignment) + " " + morphs[gloss.alignment].get_content() + " is AUX")
                auxes.add((strip_punc(morphs[gloss.alignment].get_content()), '.'.join([glosses.get(a).get_content() for a in morphs[gloss.alignment].referrers()['alignment']])))
            if gloss.get_content() not in FXN + MOD:
                word_align = morphs[gloss.alignment].segmentation
                if (word_align.find('[') > 0):
                    word_align = word_align[:word_align.find('[')]
                wordsFxnOnly[int(word_align[1:]) - 1] = False
            if gloss.get_content() in COP: # Russian-type case where 'be.FUT' is one morph requires this second half
                #print ("*" + str(gloss.get_content()))
                currMorph = morphs[gloss.alignment]
                #print ("-" + str(currMorph.get_content()))
                currWord  = currMorph.segmentation
                if (currWord.find('[') != -1):
                    currWord = currWord[0:currWord.find('[')]
                currWordInt = int(currWord[currWord.find('w')+1:]) - 1 #because words are zero-aligned
                if currWordInt - 1 >= 0 and pos[currWordInt-1].get_content() == "VERB": #and INF not in words[currWordInt - 1].get_content():
                    copLoc['left'] += 1
                    cops.add((strip_punc(currMorph.get_content()), gloss.get_content()))
                elif currWordInt + 1 < len(pos) and pos[currWordInt+1].get_content() == "VERB": #and INF not in words[currWordInt + 1].get_content():
                    copLoc['right'] += 1
                    cops.add((strip_punc(currMorph.get_content()), gloss.get_content()))
        for i in range(len(words)):
            if (wordsFxnOnly[i] == True):
                #find gloss
                #ignore case where punctuation is the only thing.
                if len(strip_punc(words[i].get_content())) == 0 and len(strip_punc(igt['gw'][i].get_content())) == 0:
                    continue
                auxes.add((strip_punc(words[i].get_content()), strip_punc(igt['gw'][i].get_content())))
    #print ("cops" + str(cops))
    #print ("auxes" + str(auxes))
    return auxes.union(cops)

def is_aux_after_for_this_word(currWord, pos):
    if (currWord.find('[') != -1):
        currWord = currWord[0:currWord.find('[')]
    currWordInt = int(currWord[currWord.find('w')+1:]) -1 #because words are 0-aligned
    verbBefore = -1
    verbAfter = -1
    for i in reversed(range(0, currWordInt)):
        if pos[i].get_content() == 'VERB':
            verbBefore = i
            break
    for i in range(currWordInt+1, len(pos)):
        if pos[i].get_content() == 'VERB':
            verbAfter = i
            break
    if verbBefore == -1 and verbAfter == -1:
        #can't do anything
        return 0
    elif verbBefore == -1: #no verbs occur before the aux, => aux is before complement
        return 1 #before += 1
    elif verbAfter == -1: #no verbs occur after the aux, => aux is after comp
        return -1 #after += 1
    else: #choose closest
        if currWordInt - verbBefore < verbAfter - currWordInt:
            return -1 #after += 1
        else:
            return 1 #before +=1

def is_aux_after(auxes, xc):
    after  = 0
    before = 0
    for igt in xc:
        #have to do this for some badly formatted data in qub
        if 'w-pos' not in [tier.id for tier in igt.tiers]:
            continue
        words   = igt['w']
        morphs  = igt['m']
        glosses = igt['g']
        glowords = igt['gw']
        pos     = igt['w-pos']
        for morph in morphs:
            if 'alignment' not in morph.referrers().keys():
                continue
            currGloss = '.'.join([glosses.get(a).get_content() for a in morph.referrers()['alignment']])
            if (morph.get_content(), currGloss) in auxes: #a single uncomplex morph is an aux
                after_or_before = is_aux_after_for_this_word(morph.segmentation, pos)
                if (after_or_before == -1): #after
                    after += 1
                elif (after_or_before == 1): #before
                    before += 1
        for gloword in glowords: #a whole word complex is an aux
            if 'alignment' not in gloword.referrers().keys() or 'content' not in gloword.referrers().keys(): #because sometimes punctuation is separate
                continue
            currGloss = '.'.join([glosses.get(a).get_content() for a in gloword.referrers()['content']])
            if (gloword.get_content(), currGloss) in auxes:
                currWord = morph.segmentation
                after_or_before = is_aux_after_for_this_word(currWord, pos)
                if (after_or_before == -1): #after
                    after += 1
                elif (after_or_before == 1): #before
                    before += 1
    if (after > before):
        return True
    else:
        return False

def what_is_comp(auxes, aux_after, xc):
    comp = 'v'
    for igt in xc:
        #have to do this for some badly formatted data in qub
        if 'w-pos' not in [tier.id for tier in igt.tiers]:
            print ("this is bad for " + igt.id)
            continue
        words   = igt['w']
        morphs  = igt['m']
        glosses = igt['g']
        pos     = igt['w-pos']
        for morph in morphs:
            if 'alignment' not in morph.referrers().keys():
                continue
            currGloss = '.'.join([glosses.get(a).get_content() for a in morph.referrers()['alignment']])
            if (morph.get_content(), currGloss) in auxes:
                currWord = morph.segmentation
                if (currWord.find('[') != -1):
                    currWord = currWord[0:currWord.find('[')]
                currWordInt = int(currWord[currWord.find('w')+1:]) - 1 #because words are 0-aligned
                comp_words = []
                if aux_after:
                    for i in range(0, currWordInt):
                        comp_words.append(words[i])
                else:
                    for i in range(currWordInt+1, len(words)):
                        comp_words.append(words[i])
                if len(comp_words) > 0:
                    numCompOne = int(comp_words[0].id[comp_words[0].id.find('w')+1:]) - 1 #because words are 0-aligned
                    if pos[numCompOne].get_content() != 'VERB':
                        comp = 'vp'
    #print ('comp ' + comp)
    return comp

def is_multiple_aux(auxes, xc):
    multi = False
    for igt in xc:
        words  = igt['w']
        morphs = igt['m']
        glosses = igt['g']
        wordsAuxes = [False] * len(words)
        for morph in morphs:
            if 'alignment' not in morph.referrers().keys():
                continue
            currGloss = '.'.join([glosses.get(a).get_content() for a in morph.referrers()['alignment']])
            if (morph.get_content(), currGloss) in auxes:
                currWord = morph.segmentation
                if (currWord.find('[') != -1):
                    currWord = currWord[0:currWord.find('[')]
                currWordInt = int(currWord[currWord.find('w')+1:]) - 1 #because words are 0-aligned
                wordsAuxes[currWordInt] = True
        for i in range(0, len(wordsAuxes)-1):
            if wordsAuxes[i] == True and wordsAuxes[i+1] == True:
                multi = True
    return multi

def condense_stems(auxes):
    ret = {}
    preds = set([aux[1] for aux in auxes])
    for pred in preds:
        ret[pred] = []
    for aux in auxes:
        ret[aux[1]].append(aux[0])
    return ret

def print_auxes(auxes, aux_after, wic, multi, out):
    file = open(out, 'w')
    if (len(auxes) == 0):
        file.write("has-aux=no")
    else:
        file.write("has-aux=yes\n")
        if (aux_after):
            file.write("aux-comp-order=after\n")
        else:
            file.write("aux-comp-order=before\n")
        file.write("aux-comp="+wic+"\n")
        if (multi):
            file.write("multiple-aux=yes\n")
        else:
            file.write("multiple-aux=no\n")
        i = 1
        for aux in condense_stems(auxes).items():
            j = 1
            for orth in aux[1]:
                file.write("aux" + str(i) + "_stem"+str(j)+"_orth=" + orth + '\n')
                file.write("aux" + str(i) + "_stem"+str(j)+"_pred=" + aux[0] + '\n')
                j += 1
            file.write('\n')
            i += 1
    file.close()

def strip_punc(str):
    s = str
    for p in PUNC:
        s = s.replace(p, '')
    return s

def main(argv):
    directory = DATA_567
    outdir = OUTFOLDER
    if argv[0] == 'dev1':
        directory += DEV1
        outdir += DEV1
    elif argv[0] == 'dev2':
        directory += DEV2
        outdir += DEV2
    elif argv[0] == 'test':
        directory += TEST
        outdir += TEST
    else:
        print ("Run this program by calling python aux_extract.py [dev1|dev2|test]")
    for lang in os.listdir(directory):
        if (lang == '.DS_Store'):
            continue
        file = open(directory + lang + '/' + LEAFNAME)
        xc = xigtxml.load(file)
        file.close()
        print ('Doing lang ' + lang)
        auxes = find_auxes(xc)
        aux_after = is_aux_after(auxes, xc)
        print_auxes(auxes, aux_after, what_is_comp(auxes, aux_after, xc), is_multiple_aux(auxes, xc), outdir + lang + '.txt')

if __name__ == '__main__':
    main(sys.argv[1:])
