#!/opt/python-3.4/bin/python3.4
# -*- coding: utf-8 -*-

import sys
from xigt.codecs import xigtxml


# CMC Copied from EMB's xigt_nltk_tree.py; modified to work
# with the w-ps and tw-ps tiers (at least as they appear in the
#   current enriched ctn data).
# Changes from EMB's original:
# - intermediate nodes are processed in reverse order
# - uses node values instead of str(node)
# - allows tier to be specified (defaults to 'w-ps')

def format_preterminal_node(igt, w, item, nodes):
    '''
    Given a Xigt syntax tier item corresponding to a preterminal node,
    and the entier Xigt igt to which it belongs, format the PTB-type 
    subtree for this node with the label as the mother and the aligned
    word as the daughter.
    '''
    subtree = '(' + item.value() + ' ' + w[item.attributes['alignment']].value() + ')'
    nodes[item.id] = subtree
    return(nodes)


def format_intermediate_node(item, nodes, orphans):
    '''
    Given a Xigt item corresponding to an intermediate node and 
    a list of existing nodes and orphans, format the node and add
    it to nodes (removing its children from nodes) or add it to
    orphans.
    '''
    children = item.attributes['children'].split(' ')
    orphan = False
    for child in children:
        if child in nodes.keys():
            continue
        else:
            orphan = True
    if orphan and item not in orphans: # If we're on the first pass through, we'll need to add this to orphans.
        orphans.append(item)
    else:
        if item in orphans:
            orphans.remove(item) # If we're not on the first pass through, we'll want to remove this from orphans.
        subtree = '(' + item.value() # Create the subtree
        for child in children: # Relying on the order of the children here.
            subtree += ' ' + nodes[child]
            del nodes[child] # We're talking trees here, so I don't need the nodes once they're been incorporated into a larger subtree.
        subtree += ')'
        nodes[item.id] = subtree

    return(nodes, orphans)


def format_ptb_tree(igt,tier='w-ps'):
    '''
    Take in a Xigt igt item and read the specified tier, outputting
    the tree in PTB format.
    '''
    x = igt[tier] # The syntax tier.
    w = igt[x.attributes['alignment']] # The tier the syntax tier is aligned to
    nodes = {} #Dictionary of nodes with Xigt ids as keys.
    orphans = [] #Array of nodes whose children aren't yet in nodes.

    for item in x:
        # If there's an alignemnt attribute, we're talking about a preterminal, where the alignment points to the word.
        # Enter these directly into the nodes dictionary.
        if 'alignment' in item.attributes:
            nodes = format_preterminal_node(igt,w,item,nodes)

        # If there's a children attribute, we're talking about a non-preterminal.  Enter these if their children
        # are already all in the nodes dictionary.  If not, move to a separate array of nodes to loop back through.
        if 'children' in item.attributes:
            (nodes,orphans) = format_intermediate_node(item,nodes,orphans)

    # If the orphans list is empty, then we're done, and the nodes list should contain exactly one node which
    # which is the complete tree.
    while orphans:
        for orphan in reversed(orphans):
            (nodes,orphans) = format_intermediate_node(orphan,nodes,orphans)

    # Once the orphans list is empty, nodes should have exactly one thing on it:
    if len(nodes) != 1:
        sys.exit("Something's wrong: Finished all orphaned nodes, but nodes has more than one element in it.  Bad input tree?")
    else:
        tree = next(iter(nodes.values()))

    return(tree)

    

