
import glob
import os
from os import listdir
from os.path import isfile, join
from collections import Counter, defaultdict
import sys


if len(sys.argv) < 3:
    print("usage: python eval-yesno.py analysis_dir gold_choices_dir")
    print("intended gold choices in aggregation/data/ling575sp15/aux/567_yesno_ques_gold")
else:
    analysis_dir, gold_dir = sys.argv[1:3]


analysis_files = [ f for f in listdir(analysis_dir) if isfile(join(analysis_dir,f)) ]

num_files = len(analysis_files)

yesno_strats = {"q-infl", "q-inv", "q-part"}
yesno_options = {"q-infl":["q-part-order", "q-part-orth"], "q-inv":["q-inv-verb"], "q-part":[]}
strat_choice_tp = Counter()
strat_choice_tn = Counter()
strat_choice_fp = Counter()
strat_choice_fn = Counter()

strat_languages = defaultdict(list)

strat_incorrect = Counter()

def read_choices(file):
    stratagies = set()
    options = defaultdict(str)
    morph = set()
    for line in open(file,'r'):
        o, spl, value = line.partition("#")[0].strip().partition("=")
        if o in yesno_strats:
            stratagies.add(o)
        elif o.startswith("q-"):
            options[o] = value
    return (stratagies, options, morph)


error_string = "{}: {}: {} instead of {}"


for f in analysis_files:
    a_strats, a_options, a_morph = read_choices(join(analysis_dir,f))
    g_strats, g_options, g_morph = read_choices(join(gold_dir,f))

    for s in yesno_strats:
        if (s in g_strats) != (s in a_strats):
            print(error_string.format(f, s, (s in a_strats), (s in g_strats)))
            if (s in a_strats):
                strat_choice_fp[s] += 1
            else:
                strat_choice_fn[s] += 1
                strat_languages[s].append(f)
        else:
            if (s in g_strats):
                strat_choice_tp[s] += 1
                strat_languages[s].append(f)
                incorrect = False
                for o in yesno_options[s]:
                    if g_options[o] != a_options[o]:
                        print(error_string.format(f, o, a_options[o], g_options[o]))
                        incorrect = True
                if incorrect:
                    strat_incorrect[s] += 1
            else:
                strat_choice_tn[s] += 1

for s in yesno_strats:
    accuracy = float(strat_choice_tp[s] + strat_choice_tn[s]) / num_files
    precision = float(strat_choice_tp[s]) / (strat_choice_tp[s] + strat_choice_fp[s])
    recall = float(strat_choice_tp[s]) / (strat_choice_tp[s] + strat_choice_fn[s])
    print("{} choice data: {} out of {} languages".format(s, len(strat_languages[s]), num_files))
    print(strat_languages[s])
    print("{} choice accuracy: {} precision: {} recall: {}".format(s, accuracy, precision, recall))
    if s != "q-infl":
        print("{} of {} have correct options".format(strat_choice_tp[s]-strat_incorrect[s], strat_choice_tp[s]))